/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
#ifndef __HSS_DEFINITION
#define __HSS_DEFINITION

/* Includes */
#include <stdint.h>
#include <stdbool.h>
#include "usblib/usblib.h"
#include "usblib/usbhid.h"
#include "usblib/device/usbdevice.h"
#include "usblib/device/usbdhid.h"
#include <FreeRTOS.h>
#include <semphr.h>

//*****************************************************************************
//
// The size of the input reports
//
//*****************************************************************************
#define HSS_IN_REPORT_SIZE 32
#define HSS_OUT_CONF_REPORT_SIZE 41
#define HSS_OUT_SNS_REPORT_SIZE 10
#define HSS_OUT_FAULT_REPORT_SIZE 12
#define HSS_OUT_RAWREAD_REPORT_SIZE 3

/* Descriptor Information */
#define SIZEOF_DEVICE_DESCRIPTOR  0x12
#define SIZEOF_HID_DESCRIPTOR  8
#define SIZEOF_REPORT_DESCRIPTOR  83

/* Report IDs for the USB Reports */
#define HSS_USB_FEATURE_RAWREAD_ID 0x3B
#define HSS_USB_FEATURE_FAULT_ID 0x3C
#define HSS_USB_FEATURE_SNS_ID 0x3D
#define HSS_USB_FEATURE_CONF_ID 0x3E
#define HSS_USB_FEATURE_IN_ID 0x3F

typedef enum
{
    //
    // Unconfigured.
    //
    HID_HSS_STATE_UNCONFIGURED,

    //
    // No keys to send and not waiting on data.
    //
    HID_HSS_STATE_IDLE,

    //
    // Waiting on report data from the host.
    //
    HID_HSS_STATE_WAIT_DATA,

    //
    // Waiting on data to be sent out.
    //
    HID_HSS_STATE_SEND
}
tHighSideSwitchState;


//*****************************************************************************
//
// PRIVATE
//
// This structure defines the private instance data structure for the USB HID
// device.
//
//*****************************************************************************
typedef struct
{
    //
    // The USB configuration number set by the host or 0 of the device is
    // currently unconfigured.
    //
    uint8_t ui8USBConfigured;

    //
    // The protocol requested by the host, USB_HID_PROTOCOL_BOOT or
    // USB_HID_PROTOCOL_REPORT.
    //
    uint8_t ui8Protocol;

    //
    // A flag to indicate that the application pressed or released a key
    // but that we couldn't send the report immediately.
    //
    volatile bool bChangeMade;

    //
    // A buffer used to receive output reports from the host.
    //
    uint8_t pui8DataBuffer[HSS_IN_REPORT_SIZE+1];

    //
    // A buffer used to hold the last input report sent to the host.
    //
    uint8_t pui8ConfigReport[HSS_OUT_CONF_REPORT_SIZE+1];
    
    //
    // A buffer used to hold the last input report sent to the host.
    //
    uint8_t pui8SenseReport[HSS_OUT_SNS_REPORT_SIZE+1];

    //
    // A buffer used to hold the last input report sent to the host.
    //
    uint8_t pui8FaultReport[HSS_OUT_FAULT_REPORT_SIZE+1];
    
    //
    // A buffer used to hold the last input report sent to the host.
    //
    uint8_t pui8RawReadReport[HSS_OUT_RAWREAD_REPORT_SIZE+1];

    //
    // An integer used to store the current report size
    //
    uint8_t curReportSize;
    
    //
    // An integer used to store the current report ID
    //
    uint8_t curReportID;

    //
    // The idle timeout control structure for our input report.  This is
    // required by the lower level HID driver.
    //
    tHIDReportIdle sReportIdle;

    //
    // This is needed for the lower level HID driver.
    //
    tUSBDHIDDevice sHIDDevice;
}
tHighSideSwitchInstance;

//*****************************************************************************
//
//! This structure is used by the application to define operating parameters
//! for the HID keyboard device.
//
//*****************************************************************************
typedef struct
{
    //
    //! The vendor ID that this device is to present in the device descriptor.
    //
    const uint16_t ui16VID;

    //
    //! The product ID that this device is to present in the device descriptor.
    //
    const uint16_t ui16PID;

    //
    //! The maximum power consumption of the device, expressed in milliamps.
    //
    const uint16_t ui16MaxPowermA;

    //
    //! Indicates whether the device is self- or bus-powered and whether or not
    //! it supports remote wakeup.  Valid values are \b USB_CONF_ATTR_SELF_PWR
    //! or \b USB_CONF_ATTR_BUS_PWR, optionally ORed with
    //! \b USB_CONF_ATTR_RWAKE.
    //
    const uint8_t ui8PwrAttributes;

    //! A pointer to the callback function which is called to notify
    //! the application of general events and those related to reception of
    //! Output and Feature reports via the (optional) interrupt OUT endpoint.
    //
    const tUSBCallback pfnCallback;

    //
    //! A client-supplied pointer which is sent as the first
    //! parameter in all calls made
    //
    void *pvCBData;

    //
    //! A pointer to the string descriptor array for this device.
    //
    const uint8_t * const *ppui8StringDescriptors;

    //
    //! The number of descriptors provided in the ppStringDescriptors
    //! array.  This must be (1 + (5 * (num languages))).
    //
    const uint32_t ui32NumStringDescriptors;

    //
    //! The private instance data for this device.
    //
    tHighSideSwitchInstance sPrivateData;
}
tUSBHighSideSwitchDevice;

#endif
