/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef HSS_HANDLER_TASK_H_
#define HSS_HANDLER_TASK_H_

#include <stdint.h>
#include "tps274C65.h"

#define HSS_DEV_PAYLOAD_SIZE 4
#define HSS_DEV_QUEUE_LENGTH 128
#define HSS_SPI_FREQUENCY 8000000
#define NUM_INPUT_REPORTS 16

/* External Functions */
void tps274c65_getInputFeatureReport(uint8_t reportID, void** report,
                                     uint8_t* size);
void tps274c65_handleGenericWrite(uint8_t addr, uint8_t reg,
                                uint8_t state, uint8_t enableChans);
void tps274c65_handleGenericRead(uint8_t addr, uint8_t reg, uint8_t state,
                                    uint8_t* data);

/* Header used for responses */
typedef __packed struct sResponseHeader
{
    unsigned reserved0 : 3;
    unsigned error : 1;
    unsigned addr : 3;
    unsigned reserved1 : 1;
} tResponseHeader;

/* Read Response Structure */
typedef __packed struct sReadResponse
{
    tResponseHeader header;
    uint8_t fault_status;
    uint8_t payload;
} tReadResponse;

/* Write Response Structure */
typedef __packed struct sWriteResponse
{
    tResponseHeader header;
    uint8_t fault_status;
    uint8_t per_channel_fault;
} tWriteResponse;

/* Header used for write commands */
typedef __packed struct sWriteHeader
{
    unsigned state : 4;
    unsigned rw : 1;
    unsigned addr : 3;
} tWriteHeader;

typedef __packed struct sSPIFrameWrite
{
    tWriteHeader header;
    uint8_t reg;
    uint8_t payload;
} tSPIFrameWrite;

/* Structure used for the configuration block of TPS274C65 */
typedef __packed struct sHSSDataConfig
{
    uint8_t dirtyByte;
    uint8_t addr;
    TPS274C65_DEV_CONFIG1 config1;
    TPS274C65_DEV_CONFIG2 config2;
    TPS274C65_DEV_CONFIG3 config3;
    TPS274C65_DEV_CONFIG4 config4;
    TPS274C65_DEV_CONFIG5 config5;
    TPS274C65_DEV_CONFIG6 config6;
    TPS274C65_SW_STATE stateSW;
    TPS274C65_LED1_4_CTL led1;
    TPS274C65_LED1_4_CTL led2;
    TPS274C65_FAULT_MASK faultMask;
    TPS274C65_EN_WB_OFF wbMaskOffEn;
    TPS274C65_EN_WB_ON wbMaskOnEn;
    TPS274C65_EN_SHRT_VS shrtVsMaskEn;
    TPS274C65_ADC_ISNS_DIS isnsDis;
    TPS274C65_ADC_TSNS_DIS tsnsDis;
    TPS274C65_ADC_VSNS_DIS vsnsDis;
    TPS274C65_ADC_CONFIG1 adcConf;
    TPS274C65_FS_SW_STATE wdFaultState;
} tHSSDataConfig;

/* Structure used for the fault block of TPS274C65 */
typedef __packed struct sHSSDiagnosticData
{
    uint8_t dirtyByte;
    uint8_t addr;
    TPS274C65_FAULT_TYPE_STAT faultType;
    TPS274C65_FAULT_CH_STAT channelFault;
    TPS274C65_FAULT_GLOBAL_TYPE globalFault;
    TPS274C65_SHRT_VS_CH_STAT shortFault;
    TPS274C65_WB_OFF_CH_STAT wireOffStateFault;
    TPS274C65_WB_ON_CH_STAT wireOnStateFault;
    TPS274C65_ILIMIT_CH_STAT ilimFault;
    TPS274C65_THERMAL_SD_CH_STAT thermalFault;
    TPS274C65_THERMAL_WRN_CH_STAT overTempWarning;
    TPS274C65_RVRS_BLK_CH_STAT revCurrentFault;
} tHSSDiagnosticData;

typedef __packed struct sHSSRawReadResponse
{
    uint8_t dirtyByte;
    uint8_t addr;
    uint8_t data;
} tHSSRawReadResponse;

typedef __packed struct sHSSHostCommand
{
    uint8_t opcode;
    uint8_t addr;
    uint8_t payload[HSS_DEV_PAYLOAD_SIZE];
} tHSSHostCommand;

/* Device Addresses */
#define TPS274C65_ADDR1 0x02
#define TPS274C65_ADDR2 0x05

/* Command OpCodes */

/* Enable Event */
#define HSS_CHANNEL_ENABLE_EVT 0x01
#define HSS_LED_ENABLE_EVT 0x02
#define HSS_UPDATE_CONFIG_REPORT 0x03
#define HSS_CHANGE_SENSE_REPORTING 0x04
#define HSS_UPDATE_FAULT_STATUS 0x05
#define HSS_RAW_REGISTER_READ 0x06
#define HSS_RAW_REGISTER_WRITE 0x07
#define HSS_INITIATE_FIRMWARE_UPDATE 0x08

#endif
