/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
#include <stdint.h>
#include <stdbool.h>
#include "hss_handler_task.h"
#include "sense_handler_task.h"
#include "hss_definitions.h"
#include "usb_spi_dev.h"
#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "driverlib/sysctl.h"
#include "driverlib/interrupt.h"
#include "driverlib/timer.h"

/* RTOS header files */
#include <FreeRTOS.h>
#include <task.h>
#include <queue.h>
#include <semphr.h>

/* Static variables */
volatile bool enableSenseReading = false;
extern TPS274C65_SW_STATE curEnableState1;
extern TPS274C65_SW_STATE curEnableState2;
static tHSSSenseData senseData;
static tReadResponse senseResp;
static tReadResponse configResp;
static SemaphoreHandle_t samplingSemaphore;
static uint8_t readReg1 = TC65_SENSE_CURRENT;
static uint8_t readReg2 = TC65_SENSE_CURRENT;
static uint8_t readReg3 = TC65_SENSE_CURRENT;
static uint8_t readReg4 = TC65_SENSE_CURRENT;
volatile uint8_t senseStateChange = SENSE_NO_STATE_CHANGE;

void tps274c65SenseTask(void *pvParameters)
{
    TPS274C65_DEV_CONFIG5 curConfigByte1;
    TPS274C65_DEV_CONFIG5 curConfigByte2;

    /* Making the sampling semaphore */
    samplingSemaphore = xSemaphoreCreateBinary();

    /* Setting up the timer */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER0);
    TimerConfigure(TIMER0_BASE, TIMER_CFG_SPLIT_PAIR | TIMER_CFG_B_PERIODIC);
    TimerLoadSet(TIMER0_BASE, TIMER_B, SysCtlClockGet() / 1000);
    TimerIntEnable(TIMER0_BASE, TIMER_TIMB_TIMEOUT);
    IntEnable(INT_TIMER0B);

    /* We assume that SPI has already been setup as this task is lower priority
        than the command handler task */
    while(1)
    {
        /* Waiting for the timer to start */
        xSemaphoreTake(samplingSemaphore, portMAX_DELAY);
        
        /* If current sensing was turned on/off, send the enable/disabled to
                the configuration bit */
        if(senseStateChange != SENSE_NO_STATE_CHANGE)
        {
            /* Reading values for CONFIG5 */
            tps274c65_handleGenericRead(TPS274C65_ADDR1,
                                        TPS274C65_DEV_CONFIG5_REG,
                                        curEnableState1.byte,
                                        (uint8_t*)&configResp);
            curConfigByte1.byte = configResp.payload;
            
            tps274c65_handleGenericRead(TPS274C65_ADDR2,
                                        TPS274C65_DEV_CONFIG5_REG,
                                        curEnableState2.byte,
                                        (uint8_t*)&configResp);
            curConfigByte2.byte = configResp.payload;

            /* Enabling the ADC */
            if(senseStateChange == SENSE_DISABLE_CHANGE)
            {
                curConfigByte1.bits.ADC_EN = 0;
                curConfigByte2.bits.ADC_EN = 0;
            }
            else if(senseStateChange == SENSE_ENABLE_CHANGE)
            {
                curConfigByte1.bits.ADC_EN = 1;
                curConfigByte2.bits.ADC_EN = 1;
            }
            tps274c65_handleGenericWrite(TPS274C65_ADDR1,
                                         TPS274C65_DEV_CONFIG5_REG,
                                         curEnableState1.byte,
                                         curConfigByte1.byte);
            tps274c65_handleGenericWrite(TPS274C65_ADDR2,
                                         TPS274C65_DEV_CONFIG5_REG,
                                         curEnableState2.byte,
                                         curConfigByte2.byte);
            
            /* If we disabled the current change, just skip the trasnaction and
                 continue */
            if(senseStateChange == SENSE_DISABLE_CHANGE)
            {
                /* Disable the timer */
                TimerDisable(TIMER0_BASE, TIMER_B);
                
                /* Making sure that we didn't get a semaphore pend while we 
                    were doing our state operation */
                if(uxSemaphoreGetCount(samplingSemaphore) == 1)
                {
                    xSemaphoreTake(samplingSemaphore, 1000);
                }
                
                senseStateChange = SENSE_NO_STATE_CHANGE;
                continue;
            }
            else
            {
                senseStateChange = SENSE_NO_STATE_CHANGE;
            }
        }

        /* Reading values for channel 1 */
        tps274c65_handleGenericRead(TPS274C65_ADDR1,
                         readReg1,
                         curEnableState1.byte,
                         (uint8_t*)&senseResp);
        senseData.channel1_1 = senseResp.payload;

        tps274c65_handleGenericRead(TPS274C65_ADDR1,
                       readReg2,
                       curEnableState1.byte,
                       (uint8_t*)&senseResp);
        senseData.channel1_2 = senseResp.payload;

        tps274c65_handleGenericRead(TPS274C65_ADDR1,
                       readReg3,
                       curEnableState1.byte,
                       (uint8_t*)&senseResp);
        senseData.channel1_3 = senseResp.payload;

        tps274c65_handleGenericRead(TPS274C65_ADDR1,
                       readReg4,
                       curEnableState1.byte,
                       (uint8_t*)&senseResp);
        senseData.channel1_4 = senseResp.payload;

        /* Reading values for channel 2 */
        tps274c65_handleGenericRead(TPS274C65_ADDR2,
                       readReg1,
                       curEnableState2.byte,
                       (uint8_t*)&senseResp);
        senseData.channel2_1 = senseResp.payload;

        tps274c65_handleGenericRead(TPS274C65_ADDR2,
                       readReg2,
                       curEnableState2.byte,
                       (uint8_t*)&senseResp);
        senseData.channel2_2 = senseResp.payload;

        tps274c65_handleGenericRead(TPS274C65_ADDR2,
                       readReg3,
                       curEnableState2.byte,
                       (uint8_t*)&senseResp);
        senseData.channel2_3 = senseResp.payload;

        tps274c65_handleGenericRead(TPS274C65_ADDR2,
                       readReg4,
                       curEnableState2.byte,
                       (uint8_t*)&senseResp);
        senseData.channel2_4 = senseResp.payload;
        
        /* Input Voltage */
        tps274c65_handleGenericRead(TPS274C65_ADDR1,
                       TPS274C65_ADC_RESULT_VS_REG,
                       curEnableState1.byte,
                       (uint8_t*)&senseResp);
        senseData.inputVoltage = senseResp.payload;

        /* Setting the dirty byte */
        senseData.dirtyByte = 1; 

        /* Setting the report to be sent. If the semaphore times out, assume
            the host isn't doing what it's supposed to do and disable/reset
            the sensing logic */
        if( USBHID_HSS_sendReport(HSS_USB_FEATURE_SNS_ID, &senseData, 
                                portTICK_PERIOD_S) == false)
        {
            /* Disable the timer and reset data */
            TimerDisable(TIMER0_BASE, TIMER_B);
            senseData.dirtyByte = 0;
            
            /* Making sure that we didn't get a semaphore pend while we 
                were doing our state operation */
            if(uxSemaphoreGetCount(samplingSemaphore) == 1)
            {
                xSemaphoreTake(samplingSemaphore, portTICK_PERIOD_MS);
            }
                            
        }
    }
}

/* Enables the current sense polling from the device */
void setCurrentSenseReporting(SENSE_READ_MODE mode, bool runSense)
{
    if(mode == TC65_OUTPUT_VOLTAGE)
    {
        readReg1 = TPS274C65_ADC_RESULT_CH1_V_REG;
        readReg2 = TPS274C65_ADC_RESULT_CH2_V_REG;
        readReg3 = TPS274C65_ADC_RESULT_CH3_V_REG;
        readReg4 = TPS274C65_ADC_RESULT_CH4_V_REG;
    }
    else if(mode == TC65_JUNCTION_TEMPERATURE)
    {
        readReg1 = TPS274C65_ADC_RESULT_CH1_T_REG;
        readReg2 = TPS274C65_ADC_RESULT_CH2_T_REG;
        readReg3 = TPS274C65_ADC_RESULT_CH3_T_REG;
        readReg4 = TPS274C65_ADC_RESULT_CH4_T_REG;
    }
    else
    {
        readReg1 = TPS274C65_ADC_RESULT_CH1_I_REG;
        readReg2 = TPS274C65_ADC_RESULT_CH2_I_REG;
        readReg3 = TPS274C65_ADC_RESULT_CH3_I_REG;
        readReg4 = TPS274C65_ADC_RESULT_CH4_I_REG;
    }
    
    if(runSense == true)
    {
        senseStateChange = SENSE_ENABLE_CHANGE;
        TimerEnable(TIMER0_BASE, TIMER_B);
    }
    else
    {
        senseStateChange = SENSE_DISABLE_CHANGE;
    }
}

void Timer0BIntHandler(void)
{
    /* Clear the timer interrupt flag. */
    TimerIntClear(TIMER0_BASE, TIMER_TIMB_TIMEOUT);

    xSemaphoreGiveFromISR(samplingSemaphore, NULL);
}

