/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
 /* Standard includes */
 #include <stdbool.h>
 #include <stdint.h>
 #include <string.h>

/* Device/Driver Includes */
#include "tps274c65.h"
#include "hss_handler_task.h"
#include "sense_handler_task.h"
#include "inc/hw_gpio.h"
#include "inc/hw_memmap.h"
#include "inc/hw_types.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_ssi.h"
#include "inc/hw_nvic.h"
#include "driverlib/sysctl.h"
#include "driverlib/gpio.h"
#include "driverlib/ssi.h"
#include "driverlib/pin_map.h"
#include "driverlib/rom.h"
#include "driverlib/rom_map.h"
#include "driverlib/usb.h"
#include "hss_definitions.h"
#include "usb_spi_dev.h"

/* RTOS header files */
#include <FreeRTOS.h>
#include <task.h>
#include <queue.h>

/* RTOS Elements */
QueueHandle_t xHSSCommandQueue;
void tps274c65CommandTask(void *pvParameters);

/* Statics Functions */
static tHSSHostCommand curHssCommand;
static void spiSendData(uint8_t *data, uint8_t* respData,uint8_t len);
static bool updateConfigurationData(tHSSDataConfig* hssConfig,
                                    TPS274C65_SW_STATE curEnableState);
static bool updateFaultStatus(tHSSDiagnosticData* hssDiag,
                                    TPS274C65_SW_STATE curEnableState);

/* Static Variables */
TPS274C65_SW_STATE curEnableState1;
TPS274C65_SW_STATE curEnableState2;
static tWriteResponse writeResp;
static tReadResponse readResp;
static SemaphoreHandle_t mutexSPISem;
static tHSSDataConfig hssConfig1;
static tHSSDataConfig hssConfig2;
static tHSSDiagnosticData hssFault1;
static tHSSDiagnosticData hssFault2;
static tHSSRawReadResponse rawReadData;

void tps274c65CommandTask(void *pvParameters)
{
    uint8_t curState;

    /* Create the queue used by the queue send and queue receive tasks. */
    xHSSCommandQueue = xQueueCreate(HSS_DEV_QUEUE_LENGTH, 
                                       sizeof(curHssCommand));

    /* Initializing the semaphore */
    mutexSPISem = xSemaphoreCreateMutex();

    /* Configuring the SPI peripheral IOs */
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    MAP_GPIOPinConfigure(GPIO_PA2_SSI0CLK);
    MAP_GPIOPinConfigure(GPIO_PA4_SSI0RX);
    MAP_GPIOPinConfigure(GPIO_PA5_SSI0TX);
    MAP_GPIOPinTypeSSI(GPIO_PORTA_BASE, GPIO_PIN_5 | GPIO_PIN_4 |
                   GPIO_PIN_2);
    GPIOPinTypeGPIOOutput(GPIO_PORTA_BASE, GPIO_PIN_3);
    GPIOPinWrite(GPIO_PORTA_BASE, GPIO_PIN_3, GPIO_PIN_3);
    
    /* Configuring the SPI module */
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_SSI0);
    MAP_SSIClockSourceSet(SSI0_BASE, SSI_CLOCK_SYSTEM);
    MAP_SSIConfigSetExpClk(SSI0_BASE, SysCtlClockGet(), SSI_FRF_MOTO_MODE_1,
                       SSI_MODE_MASTER, HSS_SPI_FREQUENCY, 8);
    MAP_SSIEnable(SSI0_BASE);

    /* Zeroing out our input reports */
    memset((void*)&hssConfig1, 0x00, sizeof(hssConfig1));
    memset((void*)&hssConfig1, 0x00, sizeof(hssConfig2));
    hssConfig1.addr = TPS274C65_ADDR1;
    hssConfig2.addr = TPS274C65_ADDR2;
    memset((void*)&hssFault1, 0x00, sizeof(hssFault1));
    memset((void*)&hssFault2, 0x00, sizeof(hssFault2));
    hssFault1.addr = TPS274C65_ADDR1;
    hssFault2.addr = TPS274C65_ADDR2;

    while(1)
    {
        /* Wait for the next command */
        xQueueReceive(xHSSCommandQueue, &curHssCommand, portMAX_DELAY);
        
        /* Command handler */
        switch(curHssCommand.opcode)
        {
            case HSS_CHANNEL_ENABLE_EVT:
                /* Enable/disable a channel for the high-side switch */
                curState = curHssCommand.payload[0];
                if(curHssCommand.addr == TPS274C65_ADDR1)
                {
                    curEnableState1.byte = curState;
                }
                else if(curHssCommand.addr == TPS274C65_ADDR2)
                {
                    curEnableState2.byte = curState;
                }
                tps274c65_handleGenericWrite(curHssCommand.addr,
                                    TPS274C65_SW_STATE_REG,
                                    curState, curState);
                break;
            case HSS_LED_ENABLE_EVT:
                /* Byte0 = LEDs 1-4, Byte1 = LEDs 5-8 */
                if(curHssCommand.addr == TPS274C65_ADDR1)
                {
                    tps274c65_handleGenericWrite(curHssCommand.addr,
                                       TPS274C65_LED1_4_CTL_REG,
                                       curEnableState1.byte,
                                       curHssCommand.payload[0]);
                    tps274c65_handleGenericWrite(curHssCommand.addr,
                                       TPS274C65_LED_5_8_CTL_REG,
                                       curEnableState1.byte,
                                       curHssCommand.payload[1]);
                }
                else if(curHssCommand.addr == TPS274C65_ADDR2)
                {
                    tps274c65_handleGenericWrite(curHssCommand.addr,
                                       TPS274C65_LED1_4_CTL_REG,
                                       curEnableState2.byte,
                                       curHssCommand.payload[0]);
                    tps274c65_handleGenericWrite(curHssCommand.addr,
                                       TPS274C65_LED_5_8_CTL_REG,
                                       curEnableState2.byte,
                                       curHssCommand.payload[1]);

                }
                break;
            case HSS_UPDATE_CONFIG_REPORT:
                /* Get the configuration status */
                if(curHssCommand.addr == TPS274C65_ADDR1)
                {
                    updateConfigurationData(&hssConfig1, curEnableState1);
                }
                else if (curHssCommand.addr == TPS274C65_ADDR2)
                {
                    updateConfigurationData(&hssConfig2, curEnableState2);
                }
                break;
            case HSS_UPDATE_FAULT_STATUS:
                /* Get the detailed fault status */
                if(curHssCommand.addr == TPS274C65_ADDR1)
                {
                    updateFaultStatus(&hssFault1, curEnableState1);
                }
                else if (curHssCommand.addr == TPS274C65_ADDR2)
                {
                    updateFaultStatus(&hssFault2, curEnableState2);
                }
                break;
            case HSS_CHANGE_SENSE_REPORTING:
                setCurrentSenseReporting((SENSE_READ_MODE)curHssCommand.payload[0],
                                            curHssCommand.payload[1]);
                break;
            case HSS_RAW_REGISTER_READ:
                if(curHssCommand.addr == TPS274C65_ADDR1)
                {
                    tps274c65_handleGenericRead(TPS274C65_ADDR1,
                                                curHssCommand.payload[0],
                                                curEnableState1.byte,
                                                (uint8_t*)&readResp);
                }
                else
                {
                    tps274c65_handleGenericRead(TPS274C65_ADDR2,
                            curHssCommand.payload[0],
                            curEnableState2.byte,
                            (uint8_t*)&readResp);
                }
                rawReadData.addr = curHssCommand.addr;
                rawReadData.dirtyByte = 0x01;
                rawReadData.data = readResp.payload;
                USBHID_HSS_sendReport(HSS_USB_FEATURE_RAWREAD_ID, &rawReadData,
                                            portTICK_PERIOD_MS * 100);
                /* Updating the report */
                if(USBHID_HSS_sendReport(HSS_USB_FEATURE_CONF_ID, &rawReadData, 
                                            portTICK_PERIOD_MS * 100) == false)
                {
                    rawReadData.dirtyByte = 0;
                }
                break;
            case HSS_RAW_REGISTER_WRITE:
                if(curHssCommand.addr == TPS274C65_ADDR1)
                {
                    tps274c65_handleGenericWrite(curHssCommand.addr,
                                                  curHssCommand.payload[0],
                                                  curEnableState1.byte,
                                                  curHssCommand.payload[1]);
                }
                else if(curHssCommand.addr == TPS274C65_ADDR2)
                {
                    tps274c65_handleGenericWrite(curHssCommand.addr,
                                                  curHssCommand.payload[0],
                                                  curEnableState2.byte,
                                                  curHssCommand.payload[1]);
                }
                break;
            case HSS_INITIATE_FIRMWARE_UPDATE:
                ROM_USBDevDisconnect(USB0_BASE);
                ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);
                ROM_GPIOPinTypeUSBAnalog(GPIO_PORTD_BASE, GPIO_PIN_5 | GPIO_PIN_4);

                ROM_IntMasterDisable();
                ROM_SysTickIntDisable();
                ROM_SysTickDisable();

                HWREG(NVIC_DIS0) = 0xffffffff;
                HWREG(NVIC_DIS1) = 0xffffffff;

                ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_USB0);
                ROM_SysCtlPeripheralReset(SYSCTL_PERIPH_USB0);
                ROM_SysCtlUSBPLLEnable();

                ROM_SysCtlDelay(ROM_SysCtlClockGet() / 3);

                ROM_IntMasterEnable();
                ROM_UpdateUSB(0);
                break;
            default:
                break;
        }
    }
}

static bool updateFaultStatus(tHSSDiagnosticData* hssDiag,
                                    TPS274C65_SW_STATE curEnableState)
{
    bool retState = true;
    uint8_t addr = hssDiag->addr;

    /* Getting all of the configuration data from the device */
    tps274c65_handleGenericRead(addr, TPS274C65_FAULT_TYPE_STAT_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState = readResp.header.error;
    hssDiag->faultType.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_FAULT_CH_STAT_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssDiag->channelFault.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_FAULT_GLOBAL_TYPE_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssDiag->globalFault.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_SHRT_VS_CH_STAT_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssDiag->shortFault.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_WB_OFF_CH_STAT_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssDiag->wireOffStateFault.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_WB_ON_CH_STAT_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssDiag->wireOnStateFault.byte = readResp.payload;
    
    /* Getting all of the enable data from the device */
    tps274c65_handleGenericRead(addr, TPS274C65_ILIMIT_CH_STAT_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssDiag->ilimFault.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_THERMAL_SD_CH_STAT_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssDiag->thermalFault.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_THERMAL_WRN_CH_STAT_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssDiag->overTempWarning.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_RVRS_BLK_CH_STAT_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssDiag->revCurrentFault.byte = readResp.payload;

    /* Showing that the device data has been updated */
    hssDiag->dirtyByte = 0x01;

    /* Updating the report */
    if(USBHID_HSS_sendReport(HSS_USB_FEATURE_FAULT_ID, hssDiag, 
                            portTICK_PERIOD_MS * 100) == false)
    {
        hssDiag->dirtyByte = 0;
    }

    return retState;
}

static bool updateConfigurationData(tHSSDataConfig* hssConfig,
                                    TPS274C65_SW_STATE curEnableState)
{
    bool retState = true;
    uint8_t addr = hssConfig->addr;

    /* Getting all of the configuration data from the device */
    tps274c65_handleGenericRead(addr, TPS274C65_DEV_CONFIG1_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState = readResp.header.error;
    hssConfig->config1.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_DEV_CONFIG2_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->config2.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_DEV_CONFIG3_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->config3.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_DEV_CONFIG4_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->config4.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_DEV_CONFIG5_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->config5.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_DEV_CONFIG6_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->config6.byte = readResp.payload;
    
    /* Getting all of the enable data from the device */
    tps274c65_handleGenericRead(addr, TPS274C65_SW_STATE_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->stateSW.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_LED1_4_CTL_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->led1.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_LED_5_8_CTL_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->led2.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_FAULT_MASK_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->faultMask.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_EN_WB_OFF_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->wbMaskOffEn.byte = readResp.payload;
    
    tps274c65_handleGenericRead(addr, TPS274C65_EN_WB_ON_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->wbMaskOnEn.byte = readResp.payload;
    
    tps274c65_handleGenericRead(addr, TPS274C65_EN_SHRT_VS_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->shrtVsMaskEn.byte = readResp.payload;
    
    tps274c65_handleGenericRead(addr, TPS274C65_ADC_ISNS_DIS_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->isnsDis.byte = readResp.payload;
    
    tps274c65_handleGenericRead(addr, TPS274C65_ADC_TSNS_DIS_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->tsnsDis.byte = readResp.payload;
    
    tps274c65_handleGenericRead(addr, TPS274C65_ADC_VSNS_DIS_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->vsnsDis.byte = readResp.payload;

    tps274c65_handleGenericRead(addr, TPS274C65_ADC_CONFIG1_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->adcConf.byte = readResp.payload;
    
    tps274c65_handleGenericRead(addr, TPS274C65_FS_SW_STATE_REG,
                                    curEnableState.byte, (uint8_t*)&readResp);
    retState |= readResp.header.error;
    hssConfig->wdFaultState.byte = readResp.payload;

    /* Showing that the device data has been updated */
    hssConfig->dirtyByte = 0x01;

    /* Updating the report */
    if(USBHID_HSS_sendReport(HSS_USB_FEATURE_CONF_ID, hssConfig, 
                                portTICK_PERIOD_MS * 100) == false)
    {
        hssConfig->dirtyByte = 0;
    }

    return retState;
}

static void spiSendData(uint8_t *data, uint8_t* respData, uint8_t len)
{
    uint8_t ii;
    uint32_t tempData;
    xSemaphoreTake(mutexSPISem, portMAX_DELAY);

    GPIOPinWrite(GPIO_PORTA_BASE, GPIO_PIN_3, 0);

    for(ii=0;ii<len;ii++)
    {
        MAP_SSIDataPut(SSI0_BASE, data[ii]);
        MAP_SSIDataGet(SSI0_BASE, &tempData);
        respData[ii] = tempData;
    }

    /* Waiting for the SPI transaction to finish before de-asserting CS */
    while(HWREG(SSI0_BASE + SSI_O_SR) & SSI_SR_BSY);

    GPIOPinWrite(GPIO_PORTA_BASE, GPIO_PIN_3, GPIO_PIN_3);
    xSemaphoreGive(mutexSPISem);
    
}

void tps274c65_handleGenericWrite(uint8_t addr, uint8_t reg,
                                uint8_t state, uint8_t payload)
{
    /* Constructing the packet */
    tSPIFrameWrite writeCmd = 
    {
        .header = 
        {
            .addr = addr,
            .rw = 1,
            .state = state
        },
        .reg = reg,
        .payload = payload
    };

    /* Sending the write command */
    spiSendData((uint8_t*)&writeCmd, (uint8_t*)&writeResp,
                            sizeof(tSPIFrameWrite));
}

void tps274c65_handleGenericRead(uint8_t addr, uint8_t reg,
                                   uint8_t state, uint8_t* data)
{
    /* Constructing the packet */
    tSPIFrameWrite readCmd = 
    {
        .header = 
        {
            .addr = addr,
            .rw = 0,
            .state = state
        },
        .reg = reg,
        .payload = 0xFF
    };
    
    /* Sending the read command */
    spiSendData((uint8_t*)&readCmd, data,
                        sizeof(tSPIFrameWrite));
}

