/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
#include "hss_definitions.h"
#include "usb_spi_dev.h"
#include "hss_usb_handlers.h"
#include "driverlib/usb.h"
#include "usblib/usblib.h"
#include "usblib/usblibpriv.h"
#include "usblib/device/usbdevice.h"
#include "usblib/usbhid.h"
#include "usblib/device/usbdhid.h"
#include <string.h>

static uint32_t HIDHighSideSwitchRxHandler(void *pvHSSDevice, uint32_t ui32Event,
                                     uint32_t ui32MsgData, void *pvMsgData);
static uint32_t HIDHighSideSwitchTxHandler(void *pvHSSDevice, uint32_t ui32Event,
                                     uint32_t ui32MsgData, void *pvMsgData);
static SemaphoreHandle_t mutexSem;

/*-----------------------------------------------------------------------------+
| Report Descriptor                                                            |
|-----------------------------------------------------------------------------*/
static const uint8_t g_pui8HighSideSwitchReportDescriptor[] =
{
    0x06, 0x00, 0xff,              // USAGE_PAGE (Vendor Defined Page 1)
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    /* This is the feature report used to send data TO the HSS device */
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0x85, HSS_USB_FEATURE_IN_ID,   // REPORT_ID (63)
    0x95, HSS_IN_REPORT_SIZE,      // REPORT_COUNT (30)
    0x75, 0x08,                    // REPORT_SIZE (8)
    0x26, 0xff, 0x00,              // LOGICAL_MAXIMUM (255)
    0x15, 0x00,                    // LOGICAL_MINIMUM (0)
    0xb1, 0x00,                    // FEATURE (Data,Ary,Abs)
    /* This is the feature report used to read device configuration */
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0x85, HSS_USB_FEATURE_CONF_ID, // REPORT_ID (62)
    0x95, HSS_OUT_CONF_REPORT_SIZE,// REPORT_COUNT (30)
    0x75, 0x08,                    // REPORT_SIZE (8)
    0x26, 0xff, 0x00,              // LOGICAL_MAXIMUM (255)
    0x15, 0x00,                    // LOGICAL_MINIMUM (0)
    0xb1, 0x00,                    // FEATURE (Data,Ary,Abs)
    /* This is the feature report used to read current sense */
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0x85, HSS_USB_FEATURE_SNS_ID, // REPORT_ID (61)
    0x95, HSS_OUT_SNS_REPORT_SIZE,// REPORT_COUNT (9)
    0x75, 0x08,                    // REPORT_SIZE (8)
    0x26, 0xff, 0x00,              // LOGICAL_MAXIMUM (255)
    0x15, 0x00,                    // LOGICAL_MINIMUM (0)
    0xb1, 0x00,                    // FEATURE (Data,Ary,Abs) 
    /* This is the feature report used to read fault status */
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0x85, HSS_USB_FEATURE_FAULT_ID,// REPORT_ID (60)
    0x95, HSS_OUT_FAULT_REPORT_SIZE,// REPORT_COUNT (12)
    0x75, 0x08,                    // REPORT_SIZE (8)
    0x26, 0xff, 0x00,              // LOGICAL_MAXIMUM (255)
    0x15, 0x00,                    // LOGICAL_MINIMUM (0)
    0xb1, 0x00,                    // FEATURE (Data,Ary,Abs) 
    /* This is the feature report used to read raw registers */
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0x85, HSS_USB_FEATURE_RAWREAD_ID,// REPORT_ID (59)
    0x95, HSS_OUT_RAWREAD_REPORT_SIZE,// REPORT_COUNT (3)
    0x75, 0x08,                    // REPORT_SIZE (8)
    0x26, 0xff, 0x00,              // LOGICAL_MAXIMUM (255)
    0x15, 0x00,                    // LOGICAL_MINIMUM (0)
    0xb1, 0x00,                    // FEATURE (Data,Ary,Abs) 
    0xc0                           // END_COLLECTION
};

/*-----------------------------------------------------------------------------+
| Device Descriptor                                                            |
|-----------------------------------------------------------------------------*/
static const tHIDDescriptor g_sHighSideSwitchHIDDescriptor =
{
    9,               // Length of this descriptor
    USB_HID_DTYPE_HID,                      // Type code of this descriptor
    0x0101,                                  // Release of USB spec (1.11)
    0,                                      // bCountryCode (not localized)
    1,                                      // bNumDescriptors
    {
        {
            USB_HID_DTYPE_REPORT,    // Report descriptor
            SIZEOF_REPORT_DESCRIPTOR // Size of report descriptor
        }
    }
};

//*****************************************************************************
//
// HID device configuration descriptor.
//
// It is vital that the configuration descriptor bConfigurationValue field
// (byte 6) is 1 for the first configuration and increments by 1 for each
// additional configuration defined here.  This relationship is assumed in the
// device stack for simplicity even though the USB 2.0 specification imposes
// no such restriction on the bConfigurationValue values.
//
// Note that this structure is deliberately located in RAM since we need to
// be able to patch some values in it based on client requirements.
//
//*****************************************************************************
static uint8_t g_pui8HSSDescriptor[] =
{
    //
    // Configuration descriptor header.
    //
    9,                          // Size of the configuration descriptor.
    USB_DTYPE_CONFIGURATION,    // Type of this descriptor.
    USBShort(34),               // The total size of this full structure.
    1,                          // The number of interfaces in this
                                // configuration.
    1,                          // The unique value for this configuration.
    5,                          // The string identifier that describes this
                                // configuration.
    USB_CONF_ATTR_SELF_PWR,     // Bus Powered, Self Powered, remote wake up.
    250,                        // The maximum power in 2mA increments.
};

//*****************************************************************************
//
// The remainder of the configuration descriptor is stored in flash since we
// don't need to modify anything in it at runtime.
//
//*****************************************************************************
static uint8_t g_pui8HIDInterface[HIDINTERFACE_SIZE] =
{
    //
    // HID Device Class Interface Descriptor.
    //
    9,                          // Size of the interface descriptor.
    USB_DTYPE_INTERFACE,        // Type of this descriptor.
    0,                          // The index for this interface.
    0,                          // The alternate setting for this interface.
    1,                          // The number of endpoints used by this
                                // interface.
    USB_CLASS_HID,              // The interface class
    USB_HID_SCLASS_BOOT,        // The interface sub-class.
    USB_HID_PROTOCOL_NONE,      // The interface protocol for the sub-class
                                // specified above.
    4,                          // The string index for this interface.
};

static const uint8_t g_pui8HIDInEndpoint[HIDINENDPOINT_SIZE] =
{
    //
    // Interrupt IN endpoint descriptor
    //
    7,                          // The size of the endpoint descriptor.
    USB_DTYPE_ENDPOINT,         // Descriptor type is an endpoint.
    USB_EP_DESC_IN | USBEPToIndex(USB_EP_1),
    USB_EP_ATTR_INT,            // Endpoint is an interrupt endpoint.
    USBShort(USBFIFOSizeToBytes(USB_FIFO_SZ_64)),
                                // The maximum packet size.
    16,                         // The polling interval for this endpoint.
};

static const uint8_t g_pui8HIDOutEndpoint[HIDOUTENDPOINT_SIZE] =
{
    //
    // Interrupt OUT endpoint descriptor
    //
    7,                          // The size of the endpoint descriptor.
    USB_DTYPE_ENDPOINT,         // Descriptor type is an endpoint.
    USB_EP_DESC_OUT | USBEPToIndex(USB_EP_2),
    USB_EP_ATTR_INT,            // Endpoint is an interrupt endpoint.
    USBShort(USBFIFOSizeToBytes(USB_FIFO_SZ_64)),
                                // The maximum packet size.
    16,                         // The polling interval for this endpoint.
};

//*****************************************************************************
//
// The HID configuration descriptor is defined as four or five sections
// depending upon the client's configuration choice.  These sections are:
//
// 1.  The 9 byte configuration descriptor (RAM).
// 2.  The interface descriptor (RAM).
// 3.  The HID report and physical descriptors (provided by the client)
//     (FLASH).
// 4.  The mandatory interrupt IN endpoint descriptor (FLASH).
// 5.  The optional interrupt OUT endpoint descriptor (FLASH).
//
//*****************************************************************************
static const tConfigSection g_sHIDConfigSection =
{
    sizeof(g_pui8HSSDescriptor),
    g_pui8HSSDescriptor
};

static const tConfigSection g_sHIDInterfaceSection =
{
    sizeof(g_pui8HIDInterface),
    g_pui8HIDInterface
};

static const tConfigSection g_sHIDInEndpointSection =
{
    sizeof(g_pui8HIDInEndpoint),
    g_pui8HIDInEndpoint
};

static const tConfigSection g_sHIDOutEndpointSection =
{
    sizeof(g_pui8HIDOutEndpoint),
    g_pui8HIDOutEndpoint
};

//*****************************************************************************
//
// Place holder for the user's HID descriptor block.
//
//*****************************************************************************
static tConfigSection g_sHIDDescriptorSection =
{
   sizeof(g_sHighSideSwitchHIDDescriptor),
   (const uint8_t *)&g_sHighSideSwitchHIDDescriptor
};


//*****************************************************************************
//
// This array lists all the sections that must be concatenated to make a
// single, complete HID configuration descriptor.
//
//*****************************************************************************
static const tConfigSection *g_psHIDSections[] =
{
    &g_sHIDConfigSection,
    &g_sHIDInterfaceSection,
    &g_sHIDDescriptorSection,
    &g_sHIDInEndpointSection,
    &g_sHIDOutEndpointSection
};

#define NUM_HID_SECTIONS        ((sizeof(g_psHIDSections) /                   \
                                  sizeof(g_psHIDSections[0])) - 1)

//*****************************************************************************
//
// The header for the single configuration we support.  This is the root of
// the data structure that defines all the bits and pieces that are pulled
// together to generate the configuration descriptor.  Note that this must be
// in RAM since we need to include or exclude the final section based on
// client supplied initialization parameters.
//
//*****************************************************************************
static tConfigHeader g_sHIDConfigHeader =
{
    NUM_HID_SECTIONS,
    g_psHIDSections
};

//*****************************************************************************
//
// Configuration Descriptor.
//
//*****************************************************************************
static const tConfigHeader * const g_ppsHIDConfigDescriptors[] =
{
    &g_sHIDConfigHeader
};

//*****************************************************************************
//
// The HID class descriptor table.  For the keyboard class, we have only a
// single report descriptor.
//
//*****************************************************************************
static const uint8_t * const g_pui8HSSClassDescriptors[] =
{
    g_pui8HighSideSwitchReportDescriptor
};

/* Configuration function for all of the lower level USB HID drivers */
void* USBHID_HSS_DeviceInit(void)
{
    void *pvRetcode;
    tUSBDHIDDevice *psHIDDevice;
    tUSBHighSideSwitchDevice *psHighSideSwitchDevice = &g_sHighSideSwitchDevice;
    tConfigDescriptor *pConfigDesc;
    tHighSideSwitchInstance *psInst;

    psHIDDevice = &psHighSideSwitchDevice->sPrivateData.sHIDDevice;
  

    /* Initialize the various fields in our instance structure. */
    psInst = &psHighSideSwitchDevice->sPrivateData; 
    psInst->ui8USBConfigured = 0;
    psInst->ui8Protocol = USB_HID_PROTOCOL_REPORT;
    psInst->sReportIdle.ui8Duration4mS = 125;
    psInst->sReportIdle.ui8ReportID = 0;
    psInst->sReportIdle.ui32TimeSinceReportmS = 0;
    psInst->sReportIdle.ui16TimeTillNextmS = 0;
    memset(psInst->pui8ConfigReport, 0, HSS_OUT_CONF_REPORT_SIZE);
    memset(psInst->pui8SenseReport, 0, HSS_OUT_SNS_REPORT_SIZE);
    memset(psInst->pui8FaultReport, 0, HSS_OUT_FAULT_REPORT_SIZE);
    memset(psInst->pui8FaultReport, 0, HSS_OUT_FAULT_REPORT_SIZE);

    /* Initialize the HID device/driver configuration */
    psHIDDevice->ui16PID = psHighSideSwitchDevice->ui16PID;
    psHIDDevice->ui16VID = psHighSideSwitchDevice->ui16VID;
    psHIDDevice->ui16MaxPowermA = psHighSideSwitchDevice->ui16MaxPowermA;
    psHIDDevice->ui8PwrAttributes = psHighSideSwitchDevice->ui8PwrAttributes;
    psHIDDevice->ui8Subclass = USB_HID_SCLASS_NONE;
    psHIDDevice->ui8Protocol = USB_HID_PROTOCOL_NONE;
    psHIDDevice->ui8NumInputReports = 1;
    psHIDDevice->psReportIdle = 0;
    psHIDDevice->pfnRxCallback = HIDHighSideSwitchRxHandler;
    psHIDDevice->pvRxCBData = (void *)psHighSideSwitchDevice;
    psHIDDevice->pfnTxCallback = HIDHighSideSwitchTxHandler;
    psHIDDevice->pvTxCBData = (void *)psHighSideSwitchDevice;
    psHIDDevice->bUseOutEndpoint = false,

    /* Setting up the descriptors */
    psHIDDevice->psHIDDescriptor = &g_sHighSideSwitchHIDDescriptor;
    psHIDDevice->ppui8ClassDescriptors = g_pui8HSSClassDescriptors;
    psHIDDevice->ppui8StringDescriptors =
                                        psHighSideSwitchDevice->ppui8StringDescriptors;
    psHIDDevice->ui32NumStringDescriptors =
                                        psHighSideSwitchDevice->ui32NumStringDescriptors;
    psHIDDevice->ppsConfigDescriptor = g_ppsHIDConfigDescriptors;

    psHIDDevice->psReportIdle = &psInst->sReportIdle;

    /* Initialize the lower level HID driver */
    pvRetcode = USBDHIDCompositeInit(0, psHIDDevice, 0);
    
    pConfigDesc = (tConfigDescriptor *)g_pui8HSSDescriptor;
    pConfigDesc->bmAttributes = psHighSideSwitchDevice->ui8PwrAttributes;
    pConfigDesc->bMaxPower =  (uint8_t)(psHighSideSwitchDevice->ui16MaxPowermA / 2);

    /* Initializing the semaphore */
    mutexSem = xSemaphoreCreateMutex();

    if(mutexSem == NULL)
    {
        pvRetcode = 0;
    }

    if(pvRetcode)
    {
        //
        // Initialize the lower layer HID driver 
        pvRetcode = USBDHIDInit(0, psHIDDevice);

        return((void *)psHighSideSwitchDevice);
    }
    else
    {
        return((void *)0);
    }
}

//*****************************************************************************
//
// Main HID device class event handler function.
//
// \param pvKeyboardDevice is the event callback pointer provided during
//  USBDHIDInit().This is a pointer to our HID device structure
// (&g_sHIDKeybDevice).
// \param ui32Event identifies the event we are being called back for.
// \param ui32MsgData is an event-specific value.
// \param pvMsgData is an event-specific pointer.
//
// This function is called by the HID device class driver to inform the
// application of particular asynchronous events related to operation of the
// keyboard HID device.
//
// \return Returns a value which is event-specific.
//
//*****************************************************************************
static uint32_t
HIDHighSideSwitchRxHandler(void *pvHSSDevice, uint32_t ui32Event,
                     uint32_t ui32MsgData, void *pvMsgData)
{
    tHighSideSwitchInstance *psInst;
    tUSBHighSideSwitchDevice *psHSSDevice;
    uint8_t inputReport;

    //
    // Get a pointer to our instance data
    //
    psHSSDevice = (tUSBHighSideSwitchDevice *)pvHSSDevice;
    psInst = &psHSSDevice->sPrivateData;

    //
    // Which event were we sent?
    //
    switch (ui32Event)
    {
        //
        // The host has connected to us and configured the device.
        //
        case USB_EVENT_CONNECTED:
        {
            psInst->ui8USBConfigured = true;

            //
            // Pass the information on to the client.
            //
            psHSSDevice->pfnCallback(psHSSDevice->pvCBData,
                                          USB_EVENT_CONNECTED, 0, (void *)0);

            break;
        }

        //
        // The host has disconnected from us.
        //
        case USB_EVENT_DISCONNECTED:
        {
            psInst->ui8USBConfigured = false;

            //
            // Pass the information on to the client.
            //
            psHSSDevice->pfnCallback(psHSSDevice->pvCBData,
                                          USB_EVENT_DISCONNECTED, 0,
                                          (void *)0);

            break;
        }

        //
        // The host is polling us for a particular report and the HID driver
        // is asking for the latest version to transmit.
        //
        case USBD_HID_EVENT_IDLE_TIMEOUT:
        case USBD_HID_EVENT_GET_REPORT:
        {
            /* Parsing out the report ID and returning the right data */
            inputReport = ui32MsgData & 0xFF;
            if(inputReport == HSS_USB_FEATURE_CONF_ID)
            {
                *(uint8_t **)pvMsgData = psInst->pui8ConfigReport;
                psInst->curReportSize = HSS_OUT_CONF_REPORT_SIZE;
                psInst->curReportID = HSS_USB_FEATURE_CONF_ID;
                return(HSS_OUT_CONF_REPORT_SIZE);
            }
            else if(inputReport == HSS_USB_FEATURE_SNS_ID)
            {
                *(uint8_t **)pvMsgData = psInst->pui8SenseReport;
                psInst->curReportSize = HSS_OUT_SNS_REPORT_SIZE;
                psInst->curReportID = HSS_USB_FEATURE_SNS_ID;
                return(HSS_OUT_SNS_REPORT_SIZE);
            }
            else if(inputReport == HSS_USB_FEATURE_FAULT_ID)
            {
                *(uint8_t **)pvMsgData = psInst->pui8FaultReport;
                psInst->curReportSize = HSS_OUT_FAULT_REPORT_SIZE;
                psInst->curReportID = HSS_USB_FEATURE_FAULT_ID;
                return(HSS_OUT_FAULT_REPORT_SIZE);
            }
            else if(inputReport == HSS_USB_FEATURE_RAWREAD_ID)
            {
                *(uint8_t **)pvMsgData = psInst->pui8RawReadReport;
                psInst->curReportSize = HSS_OUT_RAWREAD_REPORT_SIZE;
                psInst->curReportID = HSS_USB_FEATURE_RAWREAD_ID;
                return(HSS_OUT_RAWREAD_REPORT_SIZE);
            } 
        }

        //
        // The device class driver has completed sending a report to the
        // host in response to a Get_Report request.
        //
        case USBD_HID_EVENT_REPORT_SENT:
        {
            //
            // Clear the dirty byte
            //
            if(psInst->curReportID == HSS_USB_FEATURE_CONF_ID)
            {
                psInst->pui8ConfigReport[0] = 0;
            }
            else if(psInst->curReportID == HSS_USB_FEATURE_SNS_ID)
            {
                psInst->pui8SenseReport[0] = 0;
            }
            else if(psInst->curReportID == HSS_USB_FEATURE_FAULT_ID)
            {
                psInst->pui8FaultReport[0] = 0;
            }
            else if(psInst->curReportID == HSS_USB_FEATURE_RAWREAD_ID)
            {
                psInst->pui8RawReadReport[0] = 0;
            }
            psInst->curReportID = 0;
            psInst->curReportSize = 0;
            xSemaphoreGiveFromISR(mutexSem, NULL);
            break;
        }

        //
        // This event is sent in response to a host Set_Report request.  We
        // must return a pointer to a buffer large enough to receive the
        // report into.
        //
        case USBD_HID_EVENT_GET_REPORT_BUFFER:
        {
            if((uint32_t)pvMsgData <= (HSS_IN_REPORT_SIZE+1) )
            {
                //
                // Yes - return our pointer.
                //
                return((uint32_t)psInst->pui8DataBuffer);
            }
            else
            {
                //
                // We are being passed a report that is longer than the
                // only report we expect so return NULL.  This causes the
                // device class driver to stall the request.
                //
                return(0);
            }
        }

        //
        // This event indicates that the host has sent us an Output or
        // Feature report and that the report is now in the buffer we provided
        // on the previous USBD_HID_EVENT_GET_REPORT_BUFFER callback.
        //
        case USBD_HID_EVENT_SET_REPORT:
        {
            /* Pass the command to the user application */
                psHSSDevice->pfnCallback(
                                         psHSSDevice->pvCBData,
                                         HSS_COMMAND_RECEIVED,
                                         psInst->pui8DataBuffer[0],
                                         &psInst->pui8DataBuffer[1]);
            break;
        }

        //
        // The host is asking us to set either boot or report protocol (not
        // that it makes any difference to this particular mouse).
        //
        case USBD_HID_EVENT_SET_PROTOCOL:
        {
            psInst->ui8Protocol = ui32MsgData;
            break;
        }

        //
        // The host is asking us to tell it which protocol we are currently
        // using, boot or request.
        //
        case USBD_HID_EVENT_GET_PROTOCOL:
        {
            return(psInst->ui8Protocol);
        }

        //
        // Pass ERROR, SUSPEND and RESUME to the client unchanged.
        //
        case USB_EVENT_ERROR:
        case USB_EVENT_SUSPEND:
        case USB_EVENT_RESUME:
        case USB_EVENT_LPM_RESUME:
        case USB_EVENT_LPM_SLEEP:
        case USB_EVENT_LPM_ERROR:
        {
            return(psHSSDevice->pfnCallback(
                                         psHSSDevice->pvCBData,
                                         ui32Event, ui32MsgData, pvMsgData));
        }

        //
        // We ignore all other events.
        //
        default:
        {
            break;
        }
    }
    return(0);
}

static uint32_t HIDHighSideSwitchTxHandler(void *pvHSSDevice, uint32_t ui32Event,
                                     uint32_t ui32MsgData, void *pvMsgData)
{
    //
    // We never actually send input reports so this event handler should not 
    // be hit.
    //
    switch (ui32Event)
    {

        default:
        {
            break;
        }
    }

    return(0);
}

/* Sets the HID report for the specified report ID */
bool USBHID_HSS_sendReport(uint8_t reportID, void* packetData, uint32_t timeout)
{
    tUSBHighSideSwitchDevice *psHighSideSwitchDevice = &g_sHighSideSwitchDevice;
    tHighSideSwitchInstance *psInst;

    /* Initialize the various fields in our instance structure. */
    psInst = &psHighSideSwitchDevice->sPrivateData; 
    
    /* If we do not get the semaphore within one second, we should essentially
        assume something is wrong and just return as a missed packet */
    if(xSemaphoreTake(mutexSem, timeout) == pdFALSE)
    {
        return (false);
    }

    switch(reportID)
    {
        case HSS_USB_FEATURE_CONF_ID:
            memcpy(psInst->pui8ConfigReport, packetData, HSS_OUT_CONF_REPORT_SIZE);
            break;
        case HSS_USB_FEATURE_SNS_ID:
            memcpy(psInst->pui8SenseReport, packetData, HSS_OUT_SNS_REPORT_SIZE);
            break;
        case HSS_USB_FEATURE_FAULT_ID:
            memcpy(psInst->pui8FaultReport, packetData, HSS_OUT_FAULT_REPORT_SIZE);
            break;
        case HSS_USB_FEATURE_RAWREAD_ID:
            memcpy(psInst->pui8RawReadReport, packetData, HSS_OUT_RAWREAD_REPORT_SIZE);
            break;
        default:
            xSemaphoreGive(mutexSem);
            return (false);
    }

    psInst->curReportID = reportID;
    return (true);
}
