/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <windows.h>
#include "HSSCommunicationManager.h"

using namespace System;
using namespace System::Runtime::InteropServices;
using namespace std;

HSSCommunicationManager::HSSCommunicationManager()
{
    paramThreadStart = gcnew ParameterizedThreadStart(HSSCommunicationManager::receiveThreadMethod);
}

bool HSSCommunicationManager::openDevice()
{
    HDEVINFO hDevInfoSet;
    SP_DEVINFO_DATA devInfoData;
    SP_DEVICE_INTERFACE_DATA devIfData;
    PSP_DEVICE_INTERFACE_DETAIL_DATA devIfDetailData;
    DWORD dwMemberIdx = 0, dwSize, dwType;
    GUID hidGuid;
    PBYTE byteArrayBuffer;
    string devIDFromRegistry = "";
    HIDP_CAPS devCaps;
    PHIDP_PREPARSED_DATA preProccessData = NULL;

    HidD_GetHidGuid(&hidGuid);
    if (verboseMode)
    {
        printf("HidGuid = {%08lX-%04hX-%04hX-%02hhX%02hhX-%02hhX%02hhX%02hhX%02hhX%02hhX%02hhX}\n",
            hidGuid.Data1, hidGuid.Data2, hidGuid.Data3,
            hidGuid.Data4[0], hidGuid.Data4[1], hidGuid.Data4[2], hidGuid.Data4[3],
            hidGuid.Data4[4], hidGuid.Data4[5], hidGuid.Data4[6], hidGuid.Data4[7]);
    }

    /* Retrieve a list of all present USB devices with a device interface. */
    hDevInfoSet = SetupDiGetClassDevs(&hidGuid, NULL, 0, DIGCF_PRESENT | DIGCF_DEVICEINTERFACE);

    /* Making sure we have valid data */
    if (hDevInfoSet != INVALID_HANDLE_VALUE)
    {
        while (true)
        {
            devInfoData.cbSize = sizeof(SP_DEVINFO_DATA);

            /* Get SP_DEVINFO_DATA for this member. */
            if (!SetupDiEnumDeviceInfo(hDevInfoSet, dwMemberIdx, &devInfoData))
            {
                return false;
            }

            /* Get required size for device property */
            SetupDiGetDeviceRegistryProperty(hDevInfoSet, &devInfoData,
                SPDRP_HARDWAREID, &dwType, NULL, 0, &dwSize);

            /* Allocate required memory for byteArrayBuffer to hold device property. */
            byteArrayBuffer = (PBYTE)malloc(dwSize * sizeof(BYTE));

            /* Get SPDRP_HARDWAREID device property */
            if (SetupDiGetDeviceRegistryProperty(hDevInfoSet, &devInfoData,
                    SPDRP_HARDWAREID, &dwType, byteArrayBuffer, dwSize, NULL))
            {
                String^ devString = gcnew String((wchar_t*)byteArrayBuffer);

                /* Checking to see if we have the right device */
                if ((devString->Contains(this->HSS_BOARD_PID)) &&
                        (devString->Contains(this->HSS_BOARD_VID)))
                {
                    if (verboseMode)
                    {
                        printf("\n--- High side switch found ---\n\n");
                    }
                    free(byteArrayBuffer);
                    
                    /* Finding and enumerating the device */
                    devIfData.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);
                    SetupDiEnumDeviceInterfaces(hDevInfoSet, NULL, &hidGuid, dwMemberIdx, &devIfData);

                    /* Get required size for devIfDetailData. */
                    SetupDiGetDeviceInterfaceDetail(hDevInfoSet, &devIfData, NULL, 0, &dwSize, NULL);

                    /* Allocate required memory for devIfDetailData. */
                    devIfDetailData = (PSP_INTERFACE_DEVICE_DETAIL_DATA)malloc(dwSize);
                    devIfDetailData->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

                    /* Get devIfcDetailData */
                    SetupDiGetDeviceInterfaceDetail(hDevInfoSet, &devIfData, devIfDetailData, dwSize, &dwSize, NULL);
                    if (verboseMode)
                    {
                        printf("DevicePath: %ws\n", devIfDetailData->DevicePath);
                    }

                    /* Get Writehandle of device */
                    hHidDeviceObject = CreateFile((devIfDetailData->DevicePath), 
                                                    GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE,
                                                    0, OPEN_EXISTING, 0, 0);
                    if (verboseMode)
                    {
                        printf("WriteHandle: %lp\n", hHidDeviceObject);
                    }

                    /* Release devIfDetailData memory */
                    free(devIfDetailData);

                    GetSerialNumberString(hHidDeviceObject);
                    GetManufacturerString(hHidDeviceObject);
                    GetProductString(hHidDeviceObject);

                    if (!HidD_GetPreparsedData(hHidDeviceObject, &preProccessData))
                    {
                        if (verboseMode)
                        {
                            printf("Error: HidD_GetPreparsedData failed \n");
                        }
                        return false;
                    }

                    if (!HidP_GetCaps(preProccessData, &devCaps))
                    {
                        if (verboseMode)
                        {
                            printf("Error: HidP_GetCaps failed \n");
                        }
                        return false;
                    }

                    this->featureCapacity = devCaps.FeatureReportByteLength;

                    deviceConnected = true;
                    return true;
                }
            }

            /* Release byteArrayBuffer memory */
            free(byteArrayBuffer);
            dwMemberIdx++;
        }
    }

    return false;
}

bool HSSCommunicationManager::sendPacketData(uint8_t* data, uint8_t len)
{
    uint8_t* buffData;
        
    /* Making sure we are connected*/
    if ((deviceConnected == false) || (len > this->featureCapacity))
    {
        return false;
    }

    /* Allocating our buffer */
    buffData = (uint8_t*)malloc(this->featureCapacity);

    if (!buffData)
    {
        return false;
    }

    /* Filling up the buffer */
    memcpy(buffData, data, len);

    /* Filling out all of the feature report with dummy data */
    if (len != this->featureCapacity)
    {
        memset(buffData + len, 0xBE, (this->featureCapacity - len));
    }

    /* Sending the data */
    if (HidD_SetFeature(hHidDeviceObject, buffData, this->featureCapacity) == false)
    {
        PrintLastError();
        free(buffData);
        return false;
    }
    else
    {
        /* Freeing the buffer */
        free(buffData);
        return false;
    }

    /* Freeing the buffer */
    free(buffData);

    return true;
}

bool HSSCommunicationManager::getPacketData(tHSSReceiveTransaction^ trans)
{
    /* Making sure we are connected */
    if (deviceConnected == false)
    {
        return false;
    }

    Tuple<HSSCommunicationManager^, tHSSReceiveTransaction^>^ param = 
            gcnew Tuple<HSSCommunicationManager^, tHSSReceiveTransaction^>(this, trans);

    Thread^ recThread = gcnew Thread(paramThreadStart);
    recThread->Start(param);

    return true;
}


bool HSSCommunicationManager::isHidDeviceConnected()
{
    return this->deviceConnected;
}

void HSSCommunicationManager::fireReceiveEvent(tHSSReceiveTransaction^ trans)
{
    hssReceiveBufferDelegate(trans);
}

void HSSCommunicationManager::setVerboseMode(bool verboseMode)
{
    HSSCommunicationManager::verboseMode = verboseMode;
}

void HSSCommunicationManager::closeDevice()
{
    CloseHandle(hHidDeviceObject);
    this->deviceConnected = false;
}

void HSSCommunicationManager::PrintLastError()
{
    DWORD dwMessageId = GetLastError();
    LPSTR lpBuffer = nullptr;

    size_t size = FormatMessageA(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL, dwMessageId, 0, (LPSTR)&lpBuffer, 0, NULL);

    if(verboseMode)
    {
        printf("LastError: %s\n", lpBuffer);
    }

    LocalFree(lpBuffer);
}

void HSSCommunicationManager::GetManufacturerString(HANDLE hHidDeviceObject)
{

    PWCHAR Buffer = (PWCHAR)malloc(127);
    if (HidD_GetManufacturerString(hHidDeviceObject, Buffer, 127)) 
    {
        if (verboseMode)
        {
            printf("Manufacturer: %ws\n", Buffer);
        }
    }
    else 
    {
        PrintLastError();
    }
    free(Buffer);
}

void HSSCommunicationManager::GetProductString(HANDLE hHidDeviceObject)
{

    PWCHAR Buffer = (PWCHAR)malloc(127);
    if (HidD_GetProductString(hHidDeviceObject, Buffer, 127))
    {
        if (verboseMode)
        {
            printf("Product: %ws\n", Buffer);
        }
    }
    else
    {
        PrintLastError();
    }
    free(Buffer);
}

void HSSCommunicationManager::GetSerialNumberString(HANDLE hHidDeviceObject)
{

    PWCHAR Buffer = (PWCHAR)malloc(127);
    if (HidD_GetSerialNumberString(hHidDeviceObject, Buffer, 127)) 
    {
        if (verboseMode)
        {
            printf("Serial number: %ws\n", Buffer);
        }
    }
    else 
    {
        PrintLastError();
    }
    free(Buffer);
}

void HSSCommunicationManager::receiveThreadMethod(Object^ obj)
{
    Tuple<HSSCommunicationManager^, tHSSReceiveTransaction^>^ params =
        (Tuple<HSSCommunicationManager^, tHSSReceiveTransaction^>^)obj;
    HSSCommunicationManager^ commManager = params->Item1;
    tHSSReceiveTransaction^ recTransaction = params->Item2;

    performReceiveTransaction(commManager, recTransaction);

    /* Sending the data back to the GUI to get parsed */
    commManager->fireReceiveEvent(recTransaction);

}

bool HSSCommunicationManager::performReceiveTransaction(HSSCommunicationManager^ commManager,
                                                    tHSSReceiveTransaction^ recTransaction)
{
    uint8_t* readData;
    bool resCode = false;

    /* Doing some sanity checks */
    if (recTransaction->receiveSize == 0)
    {
        if (verboseMode)
        {
            printf("ERROR: Transaction size cannot be 0.\n");
        }
        return resCode;
    }

    /* Allocating the buffer */
    readData = (uint8_t*)malloc(commManager->featureCapacity);

    if (readData == NULL)
    {
        return resCode;
    }

    readData[0] = recTransaction->reportID;

    /* Doing the initial delay */
    Sleep(recTransaction->initialDelay);

    for (int ii = 0; ii < recTransaction->retries; ii++)
    {
        if (HidD_GetFeature(commManager->hHidDeviceObject,
                            readData, commManager->featureCapacity) == TRUE)
        {
            /* Checking the first byte to see if it is "dirty" */
            if (readData[0] == 0x01)
            {
                resCode = true;
                break;
            }
        }
        else
        {
            if (verboseMode)
            {
                printf("\n--- HSS Read %d --- Report ID 0x%x Length %d FAILED in %d tries.!\n", ii, recTransaction->reportID,
                    recTransaction->receiveSize, recTransaction->retries);
            }
        }

        /* Sleeping between retries */
        Sleep(recTransaction->retryDelayMS);
        readData[0] = recTransaction->reportID;
    }

    /* Sending the data back to the caller */
    memcpy(recTransaction->recData, readData, recTransaction->receiveSize);

    free(readData);
    return resCode;
}

