/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once
#include <windows.h>
#include <SetupAPI.h>
#include <hidsdi.h>
#include <stdint.h>
#include <stdbool.h>
#include <string>
#include <iostream>

using namespace System;
using namespace System::Threading;

/* Struct used for reading data */
typedef ref struct sHSSReceiveTransaction
{
    uint8_t reportID;
    uint8_t addr;
    uint8_t receiveSize;
    uint8_t retries = 15;
    uint32_t initialDelay = 0;
    uint32_t retryDelayMS = 1;
    uint8_t* recData;

} tHSSReceiveTransaction;

/* Delegates for various events */
delegate void HSSConnectionEventDelegate(bool);
delegate void HSSDeviceReceiveFinished(tHSSReceiveTransaction^ recTrans);

interface struct HSSConnectionEventInterface
{
public:
    event HSSConnectionEventDelegate^ hssConectionDelegate;
    event HSSDeviceReceiveFinished^ hssReceiveBufferDelegate;
};

ref class HSSCommunicationManager : public HSSConnectionEventInterface
{
public:
    HSSCommunicationManager();
    bool openDevice();
    void closeDevice();
    bool sendPacketData(uint8_t* data, uint8_t len);
    bool getPacketData(tHSSReceiveTransaction^ trans);
    bool isHidDeviceConnected();
    void fireReceiveEvent(tHSSReceiveTransaction^ trans);
    void setVerboseMode(bool verboseMode);
    static uint8_t connectionEvent = 0;
    static bool performReceiveTransaction(HSSCommunicationManager^ commManager,
                                            tHSSReceiveTransaction^ recTransaction);

    /* Event Handlers*/
    virtual event HSSConnectionEventDelegate^ hssConectionDelegate;
    virtual event HSSDeviceReceiveFinished^ hssReceiveBufferDelegate;

    /* Constants */
    const static uint8_t HSS_FEATURE_ID = 0x3F;
    const static uint8_t HSS_CONFIG_ID = 0x3E;
    const static uint8_t HSS_SENSE_ID = 0x3D;
    const static uint8_t HSS_FAULT_ID = 0x3C;
    const static uint8_t HSS_RAWREAD_ID = 0x3B;
    const static uint32_t HSS_USB_CMD = 0xCAFEBEEF;

private:
    /* Private constants */
    String^ HSS_BOARD_VID = "VID_1CBE";
    String^ HSS_BOARD_PID = "PID_000D";
    String^ HSS_BOARD_USAGE_PAGE = "FF00";
    uint8_t HSS_BOARD_USAGE = 0x01;

    /* Private Functions */
    void PrintLastError();
    void GetManufacturerString(HANDLE hHidDeviceObject);
    void GetProductString(HANDLE hHidDeviceObject);
    void GetSerialNumberString(HANDLE hHidDeviceObject);
    static void receiveThreadMethod(Object^ obj);

    /* Private Variables */
    HANDLE hHidDeviceObject;
    UINT featureCapacity;
    bool deviceConnected = false;
    ParameterizedThreadStart^ paramThreadStart;
    static bool verboseMode = false;
};
