/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "ConfigurationGui.h"
#include "tps274c65_defines.h"
#include "HSSCommand.h"
#include <Dbt.h>
#include <initguid.h>
#include <usbiodef.h>
#include <iostream>
#include <sstream>
#include <bitset>
#include <cliext/queue>

using namespace System;
using namespace System::Runtime::InteropServices;

/* Callback for handling Windows messages. This is mainly done to detect connect and
    disconnect events*/
INT_PTR WINAPI WinProcCallback(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    LRESULT lRet = 1;

    switch (message)
    {
    case WM_CREATE:
        break;
    case WM_DESTROY:
        break;
    case WM_DEVICECHANGE:
        switch (wParam)
        {
        case DBT_DEVICEREMOVECOMPLETE:
        case DBT_DEVICEARRIVAL:
        {
            PDEV_BROADCAST_DEVICEINTERFACE p = (PDEV_BROADCAST_DEVICEINTERFACE)lParam;
            if (p->dbcc_devicetype == DBT_DEVTYP_DEVICEINTERFACE &&
                p->dbcc_classguid == GUID_DEVINTERFACE_USB_DEVICE)
            {
                if (wcsstr(p->dbcc_name, L"#VID_1CBE&PID_000D#"))
                {                 
                    if (wParam == DBT_DEVICEARRIVAL)
                    {
                        tps274c65configurator::ConfigurationGui::connQueue.Enqueue(0x01);
                    }
                    else
                    {
                        tps274c65configurator::ConfigurationGui::connQueue.Enqueue(0x00);
                    }
                }
            }
        }
        break;
        }
        break;
    default:
        lRet = DefWindowProc(hWnd, message, wParam, lParam);
        break;
    }
    return lRet;
}

/* Initialization function that is called when loading */
System::Void tps274c65configurator::ConfigurationGui::SelectElementForm_Load(System::Object^ sender, System::EventArgs^ e)
{
    WNDCLASSEX wndClass = { 0 };
    bool isClassRegistered = false;
    DEV_BROADCAST_DEVICEINTERFACE NotificationFilter = {
        sizeof(DEV_BROADCAST_DEVICEINTERFACE),
        DBT_DEVTYP_DEVICEINTERFACE,
        0,
        GUID_DEVINTERFACE_USB_DEVICE
    };

    wndClass.cbSize = sizeof(WNDCLASSEX);
    wndClass.hInstance = reinterpret_cast<HINSTANCE>(GetModuleHandle(0));
    wndClass.lpfnWndProc = reinterpret_cast<WNDPROC>(WinProcCallback);
    wndClass.lpszClassName = L"message_class";

    isClassRegistered = RegisterClassEx(&wndClass);

    messageWindow = CreateWindowEx(0, L"message_class", L"message_window", 0, 0, 0, 0, 0,
        HWND_MESSAGE, NULL, GetModuleHandle(0), NULL);

    /* Register for connection events */
    RegisterDeviceNotification(messageWindow, &NotificationFilter, DEVICE_NOTIFY_WINDOW_HANDLE);

    /* Initializing local variables and starting threads */
    this->commManager = gcnew HSSCommunicationManager();
    this->commManager->setVerboseMode(true);
    connThreadStart = gcnew ParameterizedThreadStart(tps274c65configurator::ConfigurationGui::connectionThreadFunction);
    connThread = gcnew Thread(connThreadStart);
    connThread->Start(this);

    senseThreadStart = gcnew ParameterizedThreadStart(tps274c65configurator::ConfigurationGui::senseReadFunctionThread);
    senseThread = gcnew Thread(senseThreadStart);
    senseThread->Start(this);

    faultThreadStart = gcnew ParameterizedThreadStart(tps274c65configurator::ConfigurationGui::faultReadFunctionThread);
    faultThread = gcnew Thread(faultThreadStart);
    faultThread->Start(this);

    recHandler = gcnew HSSDeviceReceiveFinished(this, &ConfigurationGui::bufferReceiveEventHandler);
    commManager->hssReceiveBufferDelegate += recHandler;
    this->condorConfig1 = gcnew CondorConfiguration(UNIT_1_ADDR);
    this->condorConfig2 = gcnew CondorConfiguration(UNIT_2_ADDR);

    deviceSelectorCombo->SelectedIndex = 0;
    dontRunEvents = true;
    senseSourceSelBox->SelectedIndex = 0;
    faultUpdateStatus->SelectedIndex = 0;
    dontRunEvents = false;

    redButton = Image::FromFile("images/redBut.png");
    greenButton = Image::FromFile("images/greenBut.png");
}

void tps274c65configurator::ConfigurationGui::updateConnectionStatus()
{
    if (this->InvokeRequired)
    {
        UpdateConnnectionCallback^ updateCallback = gcnew UpdateConnnectionCallback(this,
            &tps274c65configurator::ConfigurationGui::updateConnectionStatus);
        this->Invoke(updateCallback);
    }
    else
    {
        if (commManager->isHidDeviceConnected())
        {
            connectionLabel->ForeColor = System::Drawing::Color::Green;
            connectionLabel->Text = "Connected";
            mainTabControl->Enabled = true;
            deviceSelectorCombo->Enabled = true;
            loadConfigMenuItem->Enabled = true;
            saveMenuItem->Enabled = true;
            enterDFUModeToolStripMenuItem->Enabled = true;
            logMessageToWindow("High-side switch device found and connected.");
            updateMainGUIFromSelection();
        }
        else
        {
            connectionLabel->ForeColor = System::Drawing::Color::Red;
            connectionLabel->Text = "Disconnected";
            mainTabControl->Enabled = false;
            loadConfigMenuItem->Enabled = false;
            deviceSelectorCombo->Enabled = false;
            saveMenuItem->Enabled = false;
            enterDFUModeToolStripMenuItem->Enabled = false;
            senseChannelAll->CheckState = CheckState::Unchecked;
            isCurrentSenseEnabled = false;
            this->commManager->openDevice();
            logMessageToWindow("High-side switch device disconnected.");
        }
    }
}

/* This function will take a raw ADC value and convert it into the "real" (temperature, current, etc.) */
float tps274c65configurator::ConfigurationGui::convertRawADCValue(uint8_t rawValue, uint8_t format)
{
    if (format == TPS274C65_CURRENT_OFFSET)
    {
        return ceilf((((float)rawValue) * (TPS274C65_KSNS * TPS274C65_ADCREF)) /
                    (255.0f* TPS274C65_RSNS) * 100.0f)/100.0f;
    }
    else if (format == TPS274C65_TEMP_OFFSET)
    {
        return ceilf(((rawValue - 210.0f) / -0.57f) * 100.0f) / 100.0f;
    }
    else if (format == TPS274C65_VOUT_OFFSET)
    {
        return ceilf(((2.0f * rawValue) / 17.0f) * 100.0f) / 100.0f;
    }
    else
    {
        return 0.0f;
    }
}

/* Updates the GUI with new current sense data received from the current sense transaction */
void tps274c65configurator::ConfigurationGui::updateCurrentSenseGUI(tHSSReceiveTransaction^ cmd)
{
    if (deviceSelectorCombo->InvokeRequired)
    {
        UpdateSenseCallback^ updateCallback = gcnew UpdateSenseCallback(this,
            &tps274c65configurator::ConfigurationGui::updateCurrentSenseGUI);
        this->Invoke(updateCallback, cmd);
    }
    else
    {
        uint8_t offset = deviceSelectorCombo->SelectedIndex == 0 ? 0 : 4;
        int dataCount;
        
        if (!int::TryParse(maxSamplesBox->Text, dataCount) || dataCount < 10)
        {
            maxSamplesBox->Text = "10";
            dataCount = 10;
        }

        float normalizedValue = 0.0;
        uint8_t* bufferData = cmd->recData + 1;
        float maxValue = 0.01;

        /* Channel 1 */
        normalizedValue = convertRawADCValue(bufferData[offset], curDataFormat);
        if (normalizedValue > maxValue && senseChart->Series[0]->Enabled)
        {
            maxValue = normalizedValue;
        }
        senseChart->Series[0]->Points->Add(normalizedValue);
        if (normalizedValue < 1 && curDataFormat == TPS274C65_CURRENT_OFFSET)
        {
            normalizedValue = normalizedValue * 1000;
            channelOneLabel->Text = normalizedValue.ToString() + "m" + curSenseUnit;
        }
        else
        {
            channelOneLabel->Text = normalizedValue.ToString() + curSenseUnit;
        }

        while (senseChart->Series[0]->Points->Count > dataCount)
        {
            senseChart->Series[0]->Points->RemoveAt(0);
        }

        /* Channel 2 */
        normalizedValue = convertRawADCValue(bufferData[offset+1], curDataFormat);
        if (normalizedValue > maxValue && senseChart->Series[1]->Enabled)
        {
            maxValue = normalizedValue;
        }
        senseChart->Series[1]->Points->Add(normalizedValue);
        if (normalizedValue < 1 && curDataFormat == TPS274C65_CURRENT_OFFSET)
        {
            normalizedValue = normalizedValue * 1000;
            channelTwoLabel->Text = normalizedValue.ToString() + "m" + curSenseUnit;
        }
        else
        {
            channelTwoLabel->Text = normalizedValue.ToString() + curSenseUnit;
        }
        while (senseChart->Series[1]->Points->Count > dataCount)
        {
            senseChart->Series[1]->Points->RemoveAt(0);
        }

        /* Channel 3 */
        normalizedValue = convertRawADCValue(bufferData[offset+2], curDataFormat);
        if (normalizedValue > maxValue && senseChart->Series[2]->Enabled)
        {
            maxValue = normalizedValue;
        }
        senseChart->Series[2]->Points->Add(normalizedValue);
        if (normalizedValue < 1 && curDataFormat == TPS274C65_CURRENT_OFFSET)
        {
            normalizedValue = normalizedValue * 1000;
            channelThreeLabel->Text = normalizedValue.ToString() + "m" + curSenseUnit;
        }
        else
        {
            channelThreeLabel->Text = normalizedValue.ToString() + curSenseUnit;
        }
        while (senseChart->Series[2]->Points->Count > dataCount)
        {
            senseChart->Series[2]->Points->RemoveAt(0);
        }

        /* Channel 4 */
        normalizedValue = convertRawADCValue(bufferData[offset+3], curDataFormat);
        if (normalizedValue > maxValue && senseChart->Series[3]->Enabled)
        {
            maxValue = normalizedValue;
        }
        senseChart->Series[3]->Points->Add(normalizedValue);
        if (normalizedValue < 1 && curDataFormat == TPS274C65_CURRENT_OFFSET)
        {
            normalizedValue = normalizedValue * 1000;
            channelFourLabel->Text = normalizedValue.ToString() + "m" + curSenseUnit;
        }
        else
        {
            channelFourLabel->Text = normalizedValue.ToString() + curSenseUnit;
        }
        while (senseChart->Series[3]->Points->Count > dataCount)
        {
            senseChart->Series[3]->Points->RemoveAt(0);
        }

        /* Normalizing the chart's scale */
        maxValue = ceil(maxValue * 1.1);
        senseChart->ChartAreas[0]->AxisY->Minimum = 0;
        senseChart->ChartAreas[0]->AxisY->Maximum = maxValue;

        /* Input voltage */
        normalizedValue = convertRawADCValue(bufferData[8], TPS274C65_VOUT_OFFSET);
        inputVoltageValueLabel->Text = normalizedValue.ToString() + "V";
    }
}

/* Gets the current device that is selected */
CondorConfiguration^ tps274c65configurator::ConfigurationGui::getCurrentCondorConfig()
{
    return (curSelectedDevice == UNIT_1_ADDR ? condorConfig1 : condorConfig2);
}

/* Handler for GUI events */
System::Void tps274c65configurator::ConfigurationGui::checkedListBox1_ItemCheck(System::Object^ sender, System::Windows::Forms::ItemCheckEventArgs^ e)
{
    if (deviceSelectorCombo->SelectedIndex == 0)
    {
        updateEnable(UNIT_1_ADDR, e);
    }
    else if (deviceSelectorCombo->SelectedIndex == 1)
    {
        updateEnable(UNIT_2_ADDR, e);
    }
}

void tps274c65configurator::ConfigurationGui::updateEnable(uint8_t addr, System::Windows::Forms::ItemCheckEventArgs^ e)
{
    uint8_t selectedItems = 0;
    UInt32 count = chanEnableBoxGroup->Items->Count;
    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (dontRunEvents == true)
    {
        return;
    }

    selectedItems = (1 << e->Index);

    if (chanEnableBoxGroup->GetItemChecked(0) == true)
    {
        selectedItems |= 0b0001;
    }

    if (chanEnableBoxGroup->GetItemChecked(1) == true)
    {
        selectedItems |= 0b0010;
    }

    if (chanEnableBoxGroup->GetItemChecked(2) == true)
    {
        selectedItems |= 0b0100;
    }

    if (chanEnableBoxGroup->GetItemChecked(3) == true)
    {
        selectedItems |= 0b1000;
    }

    /* Updating with the new state */
    if (e->NewValue == CheckState::Checked)
    {
        selectedItems |= (1 << e->Index);
    }
    else
    {
        selectedItems &= ~(1 << e->Index);
    }

    /* If everything is checked, we need to check the all button */
    dontRunEvents = true;
    if ((chanEnableBoxGroup->CheckedItems->Count == (chanEnableBoxGroup->Items->Count - 1))
        && e->NewValue == CheckState::Checked)
    {
        toggleAllChannelsBox->CheckState = CheckState::Checked;
    }
    else
    {
        toggleAllChannelsBox->CheckState = CheckState::Unchecked;
    }
    dontRunEvents = false;

    /* Sending the enable command */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_CMD_ENABLE, addr, &selectedItems, 1);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    curConfig->setEnable(selectedItems);

    bool changeEnable = (curConfig->getEnable() & 0x0F) == 0 ? true : false;
    inrushLimitBox->Enabled = changeEnable;
    ilimBehaviorBox->Enabled = changeEnable;
    parallel12CheckBox->Enabled = changeEnable;
    parallel34CheckBox->Enabled = changeEnable;
    less800mABox->Enabled = changeEnable;
    greater800mABox->Enabled = changeEnable;
    separateIlimBox->Enabled = changeEnable;
    rcbCheckBox->Enabled = changeEnable;
    chan12CurrentLimitBox->Enabled = changeEnable;
    chan12DurationBox->Enabled = changeEnable;
    chan34CurrentLimitBox->Enabled = changeEnable;
    chan34DurationBox->Enabled = changeEnable;
    rc1Box->Enabled = changeEnable;
    rc2Box->Enabled = changeEnable;
    rc3Box->Enabled = changeEnable;
    rc4Box->Enabled = changeEnable;
    loadConfigMenuItem->Enabled = changeEnable;

    free(cmd);

    logMessageToWindow("Switch enable configuration changed to: 0b{0} ", gcnew String(std::bitset<8>(selectedItems).to_string().data()));
}

System::Void tps274c65configurator::ConfigurationGui::toggleAllChannelsBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    bool selectOrNot = toggleAllChannelsBox->Checked;
    dontRunEvents = true;

    for (int ii = 0; ii < chanEnableBoxGroup->Items->Count - 1; ii++)
    {
        chanEnableBoxGroup->SetItemChecked(ii, selectOrNot);
    }

    dontRunEvents = false;
    chanEnableBoxGroup->SetItemChecked(chanEnableBoxGroup->Items->Count - 1, selectOrNot);
}

void tps274c65configurator::ConfigurationGui::updateFaultGuiFromConfig(CondorConfiguration^ curConfig)
{
    uint8_t curFaultStatus = 0;

    if (this->chanEnableBoxGroup->InvokeRequired)
    {
        array<Object^>^ args = gcnew array<Object^> { curConfig };
        UpdateFaultCallback^ updateCallback = gcnew UpdateFaultCallback(this,
            &tps274c65configurator::ConfigurationGui::updateFaultGuiFromConfig);
        this->Invoke(updateCallback, curConfig);
    }
    else
    {
        /* Updating the channel fault status */
        curFaultStatus = curConfig->getChannelFaultStatus();

        if (curFaultStatus & TPS274C65_CHAN_1)
        {
            faultBox1->Image = redButton;
            channelOneFaultLabel->ForeColor = Color::Red;
            channelOneFaultLabel->Text = "FAULT!";
        }
        else
        {
            faultBox1->Image = greenButton;
            channelOneFaultLabel->ForeColor = Color::Black;
            channelOneFaultLabel->Text = "No Fault";
        }

        if (curFaultStatus & TPS274C65_CHAN_2)
        {
            faultBox2->Image = redButton;
            channelTwoFaultLabel->ForeColor = Color::Red;
            channelTwoFaultLabel->Text = "FAULT!";

        }
        else
        {
            faultBox2->Image = greenButton;
            channelTwoFaultLabel->ForeColor = Color::Black;
            channelTwoFaultLabel->Text = "No Fault";
        }

        if (curFaultStatus & TPS274C65_CHAN_3)
        {
            faultBox3->Image = redButton;
            channelThreeFaultLabel->ForeColor = Color::Red;
            channelThreeFaultLabel->Text = "FAULT!";
        }
        else
        {
            faultBox3->Image = greenButton;
            channelThreeFaultLabel->ForeColor = Color::Black;
            channelThreeFaultLabel->Text = "No Fault";
        }

        if (curFaultStatus & TPS274C65_CHAN_4)
        {
            faultBox4->Image = redButton;
            channelFourFaultLabel->ForeColor = Color::Red;
            channelFourFaultLabel->Text = "FAULT!";
        }
        else
        {
            faultBox4->Image = greenButton;
            channelFourFaultLabel->ForeColor = Color::Black;
            channelFourFaultLabel->Text = "No Fault";
        }

        curFaultStatus = curConfig->getGlobalFaultStatus();

        if (curFaultStatus & TPS274C65_FAULT_GLOBAL_TYPE_POR_MASK)
        {
            porFaultBox->Image = redButton;

            if (porFault->isFaulted() == false)
            {
                logMessageToWindow("Global POR Fault detected.");
            }

            porFault->setFaultState(true);
        }
        else
        {
            porFaultBox->Image = greenButton;

            if (porFault->isFaulted() == true)
            {
                logMessageToWindow("Global POR Fault recovered.");
            }

            porFault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_FAULT_GLOBAL_TYPE_CHIP_THERMALSD_MASK)
        {
            thermalFaultBox->Image = redButton;

            if (tsdFault->isFaulted() == false)
            {
                logMessageToWindow("Global thermal shutdown fault deteched.");
            }

            tsdFault->setFaultState(true);
        }
        else
        {
            thermalFaultBox->Image = greenButton;

            if (tsdFault->isFaulted() == true)
            {
                logMessageToWindow("Global thermal shutdown fault recovered.");
            }

            tsdFault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_FAULT_GLOBAL_TYPE_SPI_ERR_MASK)
        {
            spiFaultBox->Image = redButton;

            if (spiFault->isFaulted() == false)
            {
                logMessageToWindow("Global SPI communication fault detected.");
            }

            spiFault->setFaultState(true);
        }
        else
        {
            spiFaultBox->Image = greenButton;

            if (spiFault->isFaulted() == true)
            {
                logMessageToWindow("Global SPI communication fault recovered.");
            }

            spiFault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_FAULT_GLOBAL_TYPE_WD_ERR_MASK)
        {
            wdtFaultBox->Image = redButton;

            if (wdFault->isFaulted() == false)
            {
                logMessageToWindow("Global SPI watchdog fault detected.");
            }

            wdFault->setFaultState(true);
        }
        else
        {
            wdtFaultBox->Image = greenButton;

            if (wdFault->isFaulted() == true)
            {
                logMessageToWindow("Global SPI watchdog fault recovered.");
            }

            wdFault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_FAULT_GLOBAL_TYPE_VDD_UVLO_MASK)
        {
            vddUVFaultBox->Image = redButton;

            if (vdduvloFault->isFaulted() == false)
            {
                logMessageToWindow("Global VDD undervoltage fault detected.");
            }

            vdduvloFault->setFaultState(true);
        }
        else
        {
            vddUVFaultBox->Image = greenButton;

            if (vdduvloFault->isFaulted() == true)
            {
                logMessageToWindow("Global VDD undervoltage fault recovered.");
            }

            vdduvloFault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_FAULT_GLOBAL_TYPE_VS_UV_WRN_MASK)
        {
            uvWarnBox->Image = redButton;

            if (uvWarnFault->isFaulted() == false)
            {
                logMessageToWindow("Supply undervoltage warning fault detected.");
            }

            uvWarnFault->setFaultState(true);
        }
        else
        {
            uvWarnBox->Image = greenButton;

            if (uvWarnFault->isFaulted() == true)
            {
                logMessageToWindow("Supply undervoltage warning fault recovered.");
            }

            uvWarnFault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_FAULT_GLOBAL_TYPE_VS_UV_MASK)
        {
            uvFaultBox->Image = redButton;

            if (uvFault->isFaulted() == false)
            {
                logMessageToWindow("Supply undervoltage fault detected");
            }

            uvFault->setFaultState(true);
        }
        else
        {
            uvFaultBox->Image = greenButton;

            if (uvFault->isFaulted() == true)
            {
                logMessageToWindow("Supply undervoltage fault recovered");
            }
            uvFault->setFaultState(false);
        }

        curFaultStatus = curConfig->getShortFaultStatus();

        if (curFaultStatus & TPS274C65_CHAN_1)
        {
            shtsFaultBoxCh1->Image = redButton;

            if (sht1Fault->isFaulted() == false)
            {
                logMessageToWindow("Short-to-supply (ch1) fault detected");
            }

            sht1Fault->setFaultState(true);
        }
        else
        {
            shtsFaultBoxCh1->Image = greenButton;

            if (sht1Fault->isFaulted() == true)
            {
                logMessageToWindow("Short-to-supply (ch1) fault recovered");
            }

            sht1Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_2)
        {
            shtsFaultBoxCh2->Image = redButton;

            if (sht2Fault->isFaulted() == false)
            {
                logMessageToWindow("Short-to-supply (ch2) fault detected");
            }

            sht2Fault->setFaultState(true);
        }
        else
        {
            shtsFaultBoxCh2->Image = greenButton;

            if (sht2Fault->isFaulted() == true)
            {
                logMessageToWindow("Short-to-supply (ch2) fault recovered");
            }

            sht2Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_3)
        {
            if (sht3Fault->isFaulted() == false)
            {
                logMessageToWindow("Short-to-supply (ch3) fault detected");
            }

            sht3Fault->setFaultState(true);
            shtsFaultBoxCh3->Image = redButton;
        }
        else
        {

            if (sht3Fault->isFaulted() == true)
            {
                logMessageToWindow("Short-to-supply (ch3) fault detected");
            }

            sht3Fault->setFaultState(false);
            shtsFaultBoxCh3->Image = greenButton;
        }

        if (curFaultStatus & TPS274C65_CHAN_4)
        {
            if (sht4Fault->isFaulted() == false)
            {
                logMessageToWindow("Short-to-supply (ch4) fault detected");
            }

            sht4Fault->setFaultState(true);
            shtsFaultBoxCh4->Image = redButton;
        }
        else
        {
            if (sht4Fault->isFaulted() == true)
            {
                logMessageToWindow("Short-to-supply (ch4) fault detected");
            }

            sht4Fault->setFaultState(false);
            shtsFaultBoxCh4->Image = greenButton;
        }

        curFaultStatus = curConfig->getWireBreakOffStatus();

        if (curFaultStatus & TPS274C65_CHAN_1)
        {
            if (wbOff1Fault->isFaulted() == false)
            {
                logMessageToWindow("Open-load/Wirebreak (off-state) fault detected (ch1)");
            }

            wbOff1Fault->setFaultState(true);
            wbOffFaultCh1->Image = redButton;
        }
        else
        {
            if (wbOff1Fault->isFaulted() == true)
            {
                logMessageToWindow("Open-load/Wirebreak (off-state) fault recovered (ch1)");
            }

            wbOff1Fault->setFaultState(false);
            wbOffFaultCh1->Image = greenButton;
        }

        if (curFaultStatus & TPS274C65_CHAN_2)
        {
            if (wbOff2Fault->isFaulted() == false)
            {
                logMessageToWindow("Open-load/Wirebreak (off-state) fault detected (ch2)");
            }

            wbOffFaultCh2->Image = redButton;
            wbOff2Fault->setFaultState(true);
        }
        else
        {
            if (wbOff2Fault->isFaulted() == true)
            {
                logMessageToWindow("Open-load/Wirebreak (off-state) fault recovered (ch2)");
            }

            wbOffFaultCh2->Image = greenButton;
            wbOff2Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_3)
        {
            if (wbOff3Fault->isFaulted() == false)
            {
                logMessageToWindow("Open-load/Wirebreak (off-state) fault detected (ch3)");
            }

            wbOffFaultCh3->Image = redButton;
            wbOff3Fault->setFaultState(true);
        }
        else
        {
            if (wbOff3Fault->isFaulted() == true)
            {
                logMessageToWindow("Open-load/Wirebreak (off-state) fault recovered (ch3)");
            }

            wbOffFaultCh3->Image = greenButton;
            wbOff3Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_4)
        {
            if (wbOff4Fault->isFaulted() == false)
            {
                logMessageToWindow("Open-load/Wirebreak (off-state) fault detected (ch4)");
            }

            wbOffFaultCh4->Image = redButton;
            wbOff4Fault->setFaultState(true);
        }
        else
        {
            if (wbOff4Fault->isFaulted() == true)
            {
                logMessageToWindow("Open-load/Wirebreak (off-state) fault recovered (ch4)");
            }

            wbOffFaultCh4->Image = greenButton;
            wbOff4Fault->setFaultState(false);
        }


        curFaultStatus = curConfig->getWireBreakOnStatus();

        if (curFaultStatus & TPS274C65_CHAN_1)
        {
            if (wbOn1Fault->isFaulted() == false)
            {
                logMessageToWindow("Open-load/Wirebreak (on-state) fault detected (ch1)");
            }

            wbOnFaultCh1->Image = redButton;
            wbOn1Fault->setFaultState(true);
        }
        else
        {
            if (wbOn1Fault->isFaulted() == true)
            {
                logMessageToWindow("Open-load/Wirebreak (on-state) fault recovered (ch1)");
            }

            wbOnFaultCh1->Image = greenButton;
            wbOn1Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_2)
        {
            if (wbOn2Fault->isFaulted() == false)
            {
                logMessageToWindow("Open-load/Wirebreak (on-state) fault detected (ch2)");
            }

            wbOnFaultCh2->Image = redButton;
            wbOn2Fault->setFaultState(true);
        }
        else
        {
            if (wbOn2Fault->isFaulted() == true)
            {
                logMessageToWindow("Open-load/Wirebreak (on-state) fault recovered (ch2)");
            }

            wbOnFaultCh2->Image = greenButton;
            wbOn2Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_3)
        {
            if (wbOn3Fault->isFaulted() == false)
            {
                logMessageToWindow("Open-load/Wirebreak (on-state) fault detected (ch3)");
            }

            wbOnFaultCh3->Image = redButton;
            wbOn3Fault->setFaultState(true);
        }
        else
        {
            if (wbOn3Fault->isFaulted() == true)
            {
                logMessageToWindow("Open-load/Wirebreak (on-state) fault recovered (ch3)");
            }

            wbOnFaultCh3->Image = greenButton;
            wbOn3Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_4)
        {
            if (wbOn4Fault->isFaulted() == false)
            {
                logMessageToWindow("Open-load/Wirebreak (on-state) fault detected (ch4)");
            }

            wbOnFaultCh4->Image = redButton;
            wbOn4Fault->setFaultState(true);
        }
        else
        {
            if (wbOn4Fault->isFaulted() == true)
            {
                logMessageToWindow("Open-load/Wirebreak (on-state) fault recovered (ch4)");
            }

            wbOnFaultCh4->Image = greenButton;
            wbOn4Fault->setFaultState(false);
        }


        curFaultStatus = curConfig->getCurrentLimitStatus();

        if (curFaultStatus & TPS274C65_CHAN_1)
        {
            if (ilim1Fault->isFaulted() == false)
            {
                logMessageToWindow("Current limit fault detected (ch1)");
            }

            ilimFaultBoxCh1->Image = redButton;
            ilim1Fault->setFaultState(true);
        }
        else
        {
            if (ilim1Fault->isFaulted() == true)
            {
                logMessageToWindow("Current limit fault recovered (ch1)");
            }

            ilimFaultBoxCh1->Image = greenButton;
            ilim1Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_2)
        {
            if (ilim2Fault->isFaulted() == false)
            {
                logMessageToWindow("Current limit fault detected (ch2)");
            }

            ilimFaultBoxCh2->Image = redButton;
            ilim2Fault->setFaultState(true);
        }
        else
        {
            if (ilim2Fault->isFaulted() == true)
            {
                logMessageToWindow("Current limit fault recovered (ch2)");
            }

            ilimFaultBoxCh2->Image = greenButton;
            ilim2Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_3)
        {
            if (ilim3Fault->isFaulted() == false)
            {
                logMessageToWindow("Current limit fault detected (ch3)");
            }

            ilimFaultBoxCh3->Image = redButton;
            ilim3Fault->setFaultState(true);
        }
        else
        {
            if (ilim3Fault->isFaulted() == true)
            {
                logMessageToWindow("Current limit fault recovered (ch3)");
            }

            ilimFaultBoxCh3->Image = greenButton;
            ilim3Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_4)
        {
            if (ilim4Fault->isFaulted() == false)
            {
                logMessageToWindow("Current limit fault detected (ch4)");
            }

            ilimFaultBoxCh4->Image = redButton;
            ilim4Fault->setFaultState(true);
        }
        else
        {
            if (ilim4Fault->isFaulted() == true)
            {
                logMessageToWindow("Current limit fault recovered (ch4)");
            }

            ilimFaultBoxCh4->Image = greenButton;
            ilim4Fault->setFaultState(false);
        }

        curFaultStatus = curConfig->getThermalFaultStatus();

        if (curFaultStatus & TPS274C65_CHAN_1)
        {
            if (tsd1Fault->isFaulted() == false)
            {
                logMessageToWindow("Thermal shutdown fault detected (ch1)");
            }

            tsdFaultBoxCh1->Image = redButton;
            tsd1Fault->setFaultState(true);
        }
        else
        {
            if (tsd1Fault->isFaulted() == true)
            {
                logMessageToWindow("Thermal shutdown fault recovered (ch1)");
            }

            tsdFaultBoxCh1->Image = greenButton;
            tsd1Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_2)
        {
            if (tsd2Fault->isFaulted() == false)
            {
                logMessageToWindow("Thermal shutdown fault detected (ch2)");
            }

            tsdFaultBoxCh2->Image = redButton;
            tsd2Fault->setFaultState(true);
        }
        else
        {
            if (tsd3Fault->isFaulted() == true)
            {
                logMessageToWindow("Thermal shutdown fault recovered (ch2)");
            }

            tsdFaultBoxCh2->Image = greenButton;
            tsd2Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_3)
        {
            if (tsd3Fault->isFaulted() == false)
            {
                logMessageToWindow("Thermal shutdown fault detected (ch3)");
            }

            tsdFaultBoxCh3->Image = redButton;
            tsd3Fault->setFaultState(true);
        }
        else
        {
            if (tsd3Fault->isFaulted() == true)
            {
                logMessageToWindow("Thermal shutdown fault recovered (ch3)");
            }

            tsdFaultBoxCh3->Image = greenButton;
            tsd3Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_4)
        {
            if (tsd4Fault->isFaulted() == false)
            {
                logMessageToWindow("Thermal shutdown fault detected (ch4)");
            }

            tsdFaultBoxCh4->Image = redButton;
            tsd4Fault->setFaultState(true);
        }
        else
        {
            if (tsd4Fault->isFaulted() == true)
            {
                logMessageToWindow("Thermal shutdown fault recovered (ch4)");
            }

            tsdFaultBoxCh4->Image = greenButton;
            tsd4Fault->setFaultState(false);
        }

        curFaultStatus = curConfig->getOverTempWarningStatus();

        if (curFaultStatus & TPS274C65_CHAN_1)
        {
            if (tw1Fault->isFaulted() == false)
            {
                logMessageToWindow("Thermal warning detected (ch1)");
            }

            tWarnBoxCh1->Image = redButton;
            tw1Fault->setFaultState(true);
        }
        else
        {
            if (tw1Fault->isFaulted() == true)
            {
                logMessageToWindow("Thermal warning recovered (ch1)");
            }

            tWarnBoxCh1->Image = greenButton;
            tw1Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_2)
        {
            if (tw2Fault->isFaulted() == false)
            {
                logMessageToWindow("Thermal warning detected (ch2)");
            }

            tWarnBoxCh2->Image = redButton;
            tw2Fault->setFaultState(true);
        }
        else
        {
            if (tw2Fault->isFaulted() == true)
            {
                logMessageToWindow("Thermal warning recovered (ch2)");
            }

            tWarnBoxCh2->Image = greenButton;
            tw2Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_3)
        {
            if (tw3Fault->isFaulted() == false)
            {
                logMessageToWindow("Thermal warning detected (ch3)");
            }

            tWarnBoxCh3->Image = redButton;
            tw3Fault->setFaultState(true);
        }
        else
        {
            if (tw3Fault->isFaulted() == true)
            {
                logMessageToWindow("Thermal warning recovered (ch3)");
            }

            tWarnBoxCh3->Image = greenButton;
            tw3Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_4)
        {
            if (tw4Fault->isFaulted() == false)
            {
                logMessageToWindow("Thermal warning detected (ch4)");
            }
            tWarnBoxCh4->Image = redButton;
            tw4Fault->setFaultState(true);
        }
        else
        {
            if (tw4Fault->isFaulted() == true)
            {
                logMessageToWindow("Thermal warning recovered (ch4)");
            }
            tWarnBoxCh4->Image = greenButton;
            tw4Fault->setFaultState(false);
        }

        curFaultStatus = curConfig->getReverseCurrentStatus();

        if (curFaultStatus & TPS274C65_CHAN_1)
        {
            if (rvsr1Fault->isFaulted() == false)
            {
                logMessageToWindow("Reverse current fault detected (ch1)");
            }

            rvsrBoxCh1->Image = redButton;
            rvsr1Fault->setFaultState(true);
        }
        else
        {
            if (rvsr1Fault->isFaulted() == true)
            {
                logMessageToWindow("Reverse current fault recovered (ch1)");
            }

            rvsrBoxCh1->Image = greenButton;
            rvsr1Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_2)
        {
            if (rvsr2Fault->isFaulted() == false)
            {
                logMessageToWindow("Reverse current fault detected (ch2)");
            }

            rvsrBoxCh2->Image = redButton;
            rvsr2Fault->setFaultState(true);
        }
        else
        {
            if (rvsr2Fault->isFaulted() == true)
            {
                logMessageToWindow("Reverse current fault recovered (ch2)");
            }

            rvsrBoxCh2->Image = greenButton;
            rvsr2Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_3)
        {
            if (rvsr3Fault->isFaulted() == false)
            {
                logMessageToWindow("Reverse current fault detected (ch3)");
            }

            rvsrBoxCh3->Image = redButton;
            rvsr3Fault->setFaultState(true);
        }
        else
        {
            if (rvsr3Fault->isFaulted() == true)
            {
                logMessageToWindow("Reverse current fault recovered (ch3)");
            }

            rvsrBoxCh3->Image = greenButton;
            rvsr3Fault->setFaultState(false);
        }

        if (curFaultStatus & TPS274C65_CHAN_4)
        {
            if (rvsr4Fault->isFaulted() == false)
            {
                logMessageToWindow("Reverse current fault detected (ch4)");
            }

            rvsrBoxCh4->Image = redButton;
            rvsr4Fault->setFaultState(true);
        }
        else
        {
            if (rvsr4Fault->isFaulted() == true)
            {
                logMessageToWindow("Reverse current fault recovered (ch4)");
            }

            rvsrBoxCh4->Image = greenButton;
            rvsr4Fault->setFaultState(false);
        }

        handleFaultClick(curFault);
    }
}

System::Void tps274c65configurator::ConfigurationGui::ledToggleBoxGroup_ItemCheck(System::Object^ sender, System::Windows::Forms::ItemCheckEventArgs^ e)
{
    uint8_t selectedDevice;
    uint8_t selectedItems[2] = { 0,0 };
    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (dontRunEvents == true)
    {
        return;
    }

    /* Finding which condor to use */
    selectedDevice = getCurrentCondorConfig()->getDeviceAddress();

    /* Finding out which LEDs are enabled currently */
    if (ledToggleBoxGroup->GetItemChecked(0) == true)
    {
        selectedItems[0] |= 0b0001;
    }

    if (ledToggleBoxGroup->GetItemChecked(1) == true)
    {
        selectedItems[0] |= 0b0010;
    }

    if (ledToggleBoxGroup->GetItemChecked(2) == true)
    {
        selectedItems[0] |= 0b0100;
    }

    if (ledToggleBoxGroup->GetItemChecked(3) == true)
    {
        selectedItems[0] |= 0b1000;
    }

    if (ledToggleBoxGroup->GetItemChecked(4) == true)
    {
        selectedItems[1] |= 0b0001;
    }

    if (ledToggleBoxGroup->GetItemChecked(5) == true)
    {
        selectedItems[1] |= 0b0010;
    }

    if (ledToggleBoxGroup->GetItemChecked(6) == true)
    {
        selectedItems[1] |= 0b0100;
    }

    if (ledToggleBoxGroup->GetItemChecked(7) == true)
    {
        selectedItems[1] |= 0b1000;
    }

    /* Updating the state */
    if (e->NewValue == CheckState::Checked)
    {
        if (e->Index > 3)
        {
            selectedItems[1] |= (1 << ((e->Index) - 4));
        }
        else
        {
            selectedItems[0] |= (1 << ((e->Index)));
        }
    }
    else
    {
        if (e->Index > 3)
        {
            selectedItems[1] &= ~(1 << ((e->Index) - 4));
        }
        else
        {
            selectedItems[0] &= ~(1 << ((e->Index)));
        }
    }

    /* If everything is checked, we need to check the all button */
    dontRunEvents = true;
    if ((ledToggleBoxGroup->CheckedItems->Count == (ledToggleBoxGroup->Items->Count - 1))
        && e->NewValue == CheckState::Checked)
    {
        toggleAllLEDChannelsBox->CheckState = CheckState::Checked;
    }
    else
    {
        toggleAllLEDChannelsBox->CheckState = CheckState::Unchecked;
    }
    dontRunEvents = false;

    /* Sending the command to enable the LEDs */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_LED_ENABLE, selectedDevice,
        (uint8_t*)selectedItems, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    curConfig->setLED1(selectedItems[0]);
    curConfig->setLED2(selectedItems[1]);

    logMessageToWindow("Switch enable configuration changed to: 0b{0} {1} ", gcnew String(std::bitset<8>(selectedItems[0]).to_string().data()),
                                                                                            gcnew String(std::bitset<8>(selectedItems[1]).to_string().data()));

    free(cmd);
}


System::Void tps274c65configurator::ConfigurationGui::toggleAllLEDChannelsBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    bool selectOrNot = toggleAllLEDChannelsBox->Checked;
    dontRunEvents = true;

    for (int ii = 0; ii < ledToggleBoxGroup->Items->Count - 1; ii++)
    {
        ledToggleBoxGroup->SetItemChecked(ii, selectOrNot);
    }

    dontRunEvents = false;
    ledToggleBoxGroup->SetItemChecked(ledToggleBoxGroup->Items->Count - 1, selectOrNot);
}

void tps274c65configurator::ConfigurationGui::updateMainGUIFromSelection(void)
{
    tHSSReceiveTransaction^ trans;
    int selectedDevice;

    /* Finding which device to use */
    selectedDevice = getCurrentCondorConfig()->getDeviceAddress();

    /* Telling the HSS to update the configuration data for the selected address */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_UPDATE_CONFIG, selectedDevice, NULL, 0);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());

    /* Getting the updated data */
    trans = gcnew tHSSReceiveTransaction;
    trans->addr = selectedDevice;
    trans->recData = (uint8_t*)malloc(128);
    trans->reportID = HSSCommunicationManager::HSS_CONFIG_ID;
    trans->receiveSize = HSSCommand::HSS_TPS274C65_UPDATE_CONFIG_RESP_SIZE;
    this->commManager->getPacketData(trans);

}

System::Void tps274c65configurator::ConfigurationGui::deviceSelectorCombo_SelectedIndexChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (deviceSelectorCombo->SelectedIndex == 0)
    {
        curSelectedDevice = UNIT_1_ADDR;
    }
    else
    {
        curSelectedDevice = UNIT_2_ADDR;
    }

    updateMainGUIFromSelection();
}

System::Void tps274c65configurator::ConfigurationGui::updateGuiFromConfig(bool dontRunEventUpdate)
{
    uint8_t tempVal1;
    bool oldConfigValue;
    CondorConfiguration^ curConfig;

    if (this->chanEnableBoxGroup->InvokeRequired)
    {
        UpdateConfigCallback^ updateCallback = gcnew UpdateConfigCallback(this,
            &tps274c65configurator::ConfigurationGui::updateGuiFromConfig);
        this->Invoke(updateCallback, dontRunEventUpdate);
    }
    else
    {
        oldConfigValue = dontRunEvents;
        dontRunEvents = dontRunEventUpdate;
        
        curConfig = getCurrentCondorConfig();

        /* Enable boxes */
        tempVal1 = curConfig->getEnable();
        for (int ii = 0; ii < chanEnableBoxGroup->Items->Count; ii++)
        {
            if ((1 << ii) & tempVal1)
            {
                chanEnableBoxGroup->SetItemCheckState(ii, CheckState::Checked);
            }
            else
            {
                chanEnableBoxGroup->SetItemCheckState(ii, CheckState::Unchecked);
            }
            chanEnableBoxGroup->SetSelected(ii, false);
        }

        /* LED boxes */
        tempVal1 = (curConfig->getLED1() << 4);
        tempVal1 |= (curConfig->getLED2() & 0x0F);
        for (int ii = 0; ii < ledToggleBoxGroup->Items->Count; ii++)
        {
            if ((1 << ii) & tempVal1)
            {
                ledToggleBoxGroup->SetItemCheckState(ii, CheckState::Checked);
            }
            else
            {
                ledToggleBoxGroup->SetItemCheckState(ii, CheckState::Unchecked);
            }
            ledToggleBoxGroup->SetSelected(ii, false);
        }

        /* If everything is checked, we need to check the all button */
        if (chanEnableBoxGroup->CheckedItems->Count == chanEnableBoxGroup->Items->Count)
        {
            toggleAllChannelsBox->CheckState = CheckState::Checked;
        }
        else
        {
            toggleAllChannelsBox->CheckState = CheckState::Unchecked;
        }

        if (ledToggleBoxGroup->CheckedItems->Count == ledToggleBoxGroup->Items->Count)
        {
            toggleAllLEDChannelsBox->CheckState = CheckState::Checked;
        }
        else
        {
            toggleAllLEDChannelsBox->CheckState = CheckState::Unchecked;
        }

        /* Updating all of the device configuration settings */
        tempVal1 = curConfig->getConfig1() & TPS274C65_DEV_CONFIG1_ILIM_REG_12_MASK;
        chan12CurrentLimitBox->SelectedIndex = tempVal1;
        tempVal1 = (curConfig->getConfig1() & TPS274C65_DEV_CONFIG1_ILIM_DURATION_12_MASK) >> 4;
        chan12DurationBox->SelectedIndex = tempVal1;

        tempVal1 = curConfig->getConfig2() & TPS274C65_DEV_CONFIG2_ILIM_REG_34_MASK;
        chan34CurrentLimitBox->SelectedIndex = tempVal1;
        tempVal1 = (curConfig->getConfig2() & TPS274C65_DEV_CONFIG2_ILIM_DURATION_34_MASK) >> 4;
        chan34DurationBox->SelectedIndex = tempVal1;

        if (curConfig->getConfig3() & TPS274C65_DEV_CONFIG3_ILIM_SET_MASK)
        {
            separateIlimBox->CheckState = CheckState::Checked;
            chan34CurrentLimitBox->Enabled = true;
            chan34DurationBox->Enabled = true;
        }
        else
        {
            separateIlimBox->CheckState = CheckState::Unchecked;
            chan34CurrentLimitBox->Enabled = false;
            chan34DurationBox->Enabled = false;
        }

        tempVal1 = curConfig->getConfig3() & TPS274C65_DEV_CONFIG3_INRUSH_LIMIT_MASK;
        inrushLimitBox->SelectedIndex = tempVal1;


        if (curConfig->getConfig3() & TPS274C65_DEV_CONFIG3_ILIM_CONFIG_MASK)
        {
            ilimBehaviorBox->SelectedIndex = 1;
        }
        else
        {
            ilimBehaviorBox->SelectedIndex = 0;
        }

        if (curConfig->getConfig3() & TPS274C65_DEV_CONFIG3_RCB_DIS_MASK)
        {
            rcbCheckBox->CheckState = CheckState::Unchecked;
            rc1Box->Enabled = false;
            rc2Box->Enabled = false;
            rc3Box->Enabled = false;
            rc4Box->Enabled = false;
        }
        else
        {
            rcbCheckBox->CheckState = CheckState::Checked;
            rc1Box->Enabled = true;
            rc2Box->Enabled = true;
            rc3Box->Enabled = true;
            rc4Box->Enabled = true;
        }

        if (curConfig->getConfig3() & TPS274C65_DEV_CONFIG3_PARALLEL_12_MASK)
        {
            parallel12CheckBox->CheckState = CheckState::Checked;
        }
        else
        {
            parallel12CheckBox->CheckState = CheckState::Unchecked;
        }

        if (curConfig->getConfig3() & TPS274C65_DEV_CONFIG3_PARALLEL_34_MASK)
        {
            parallel34CheckBox->CheckState = CheckState::Checked;
        }
        else
        {
            parallel34CheckBox->CheckState = CheckState::Unchecked;
        }

        if (curConfig->getConfig3() & TPS274C65_DEV_CONFIG3_ISNS_RANGE_MASK)
        {
            greater800mABox->Checked = true;
            less800mABox->Checked = false;
        }
        else
        {
            greater800mABox->Checked = false;
            less800mABox->Checked = true;
        }

        tempVal1 = (curConfig->getConfig4() & TPS274C65_DEV_CONFIG4_WB_OFF_PU_MASK) >> 3;
        wbOffCurrrentBox->SelectedIndex = tempVal1;

        tempVal1 = (curConfig->getConfig4() & TPS274C65_DEV_CONFIG4_WB_ON_THD_MASK);
        wbOnThreshBox->SelectedIndex = tempVal1;

        tempVal1 = (curConfig->getConfig4() & TPS274C65_DEV_CONFIG4_WD_TO_MASK) >> 5;
        wdTimeoutDurationBox->SelectedIndex = tempVal1;

        if (curConfig->getConfig4() & TPS274C65_DEV_CONFIG4_WD_EN_MASK)
        {
            wdEnableBox->CheckState = CheckState::Checked;
        }
        else
        {
            wdEnableBox->CheckState = CheckState::Unchecked;
        }

        if (curConfig->getConfig5() & TPS274C65_DEV_CONFIG5_FLT_LTCH_DIS_MASK)
        {
            latchDisableBox->CheckState = CheckState::Unchecked;
        }
        else
        {
            latchDisableBox->CheckState = CheckState::Checked;
        }

        if (curConfig->getConfig5() & TPS274C65_DEV_CONFIG5_SW_FS_CFG_MASK)
        {
            fsKeepBox->CheckState = CheckState::Checked;
            keepStateBox1->Enabled = false;
            keepStateBox2->Enabled = false;
            keepStateBox3->Enabled = false;
            keepStateBox4->Enabled = false;
        }
        else
        {
            fsKeepBox->CheckState = CheckState::Unchecked;
            keepStateBox1->Enabled = true;
            keepStateBox2->Enabled = true;
            keepStateBox3->Enabled = true;
            keepStateBox4->Enabled = true;
        }

        tempVal1 = (curConfig->getConfig5() & TPS274C65_DEV_CONFIG5_WB_SVS_BLANK_MASK) >> 2;
        wbBlankBox->SelectedIndex = tempVal1;

        if (curConfig->getConfig5() & TPS274C65_DEV_CONFIG5_AUTO_RETRY_DIS_MASK)
        {
            latchRadioButton->Checked = false;
            retryRadioButton->Checked = true;
        }
        else
        {
            latchRadioButton->Checked = true;
            retryRadioButton->Checked = false;
        }

        if (curConfig->getConfig5() & TPS274C65_DEV_CONFIG5_ADC_EN_MASK)
        {
            adcEnableBox->CheckState = CheckState::Checked;
        }
        else
        {
            adcEnableBox->CheckState = CheckState::Unchecked;
        }

        if (curConfig->getConfig6() & TPS274C65_DEV_CONFIG6_RCB_CH1_MASK)
        {
            rc1Box->CheckState = CheckState::Checked;
        }
        else
        {
            rc1Box->CheckState = CheckState::Unchecked;
        }

        if (curConfig->getConfig6() & TPS274C65_DEV_CONFIG6_RCB_CH2_MASK)
        {
            rc2Box->CheckState = CheckState::Checked;
        }
        else
        {
            rc2Box->CheckState = CheckState::Unchecked;
        }

        if (curConfig->getConfig6() & TPS274C65_DEV_CONFIG6_RCB_CH3_MASK)
        {
            rc3Box->CheckState = CheckState::Checked;
        }
        else
        {
            rc3Box->CheckState = CheckState::Unchecked;
        }

        if (curConfig->getConfig6() & TPS274C65_DEV_CONFIG6_RCB_CH4_MASK)
        {
            rc4Box->CheckState = CheckState::Checked;
        }
        else
        {
            rc4Box->CheckState = CheckState::Unchecked;
        }

        tempVal1 = curConfig->getFaultMask();
        if (tempVal1 & TPS274C65_FAULT_MASK_MASK_VS_UV_MASK)
        {
            uvloMaskBox->CheckState = CheckState::Checked;
        }
        else
        {
            uvloMaskBox->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_FAULT_MASK_MASK_WB_ON_MASK)
        {
            wbOnMaskBox->CheckState = CheckState::Checked;
        }
        else
        {
            wbOnMaskBox->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_FAULT_MASK_MASK_WB_OFF_MASK)
        {
            wbOffMaskBox->CheckState = CheckState::Checked;
        }
        else
        {
            wbOffMaskBox->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_FAULT_MASK_MASK_SHRT_VS_MASK)
        {
            vsShortMaskBox->CheckState = CheckState::Checked;
        }
        else
        {
            vsShortMaskBox->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_FAULT_MASK_MASK_RVRS_BLK_MASK)
        {
            rcbMaskBox->CheckState = CheckState::Checked;
        }
        else
        {
            rcbMaskBox->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_FAULT_MASK_MASK_ILIMIT_MASK)
        {
            ilimMaskBox->CheckState = CheckState::Checked;
        }
        else
        {
            ilimMaskBox->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_FAULT_MASK_MASK_WD_ERR_MASK)
        {
            wdErrorMaskBox->CheckState = CheckState::Checked;
        }
        else
        {
            wdErrorMaskBox->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_FAULT_MASK_MASK_SPI_ERR_MASK)
        {
            spiMaskBox->CheckState = CheckState::Checked;
        }
        else
        {
            spiMaskBox->CheckState = CheckState::Unchecked;
        }

        tempVal1 = curConfig->getWireBreakOff();
        if (tempVal1 & TPS274C65_EN_WB_OFF_WB_OFF_CH1_EN_MASK)
        {
            wbOffCh1->CheckState = CheckState::Checked;
        }
        else
        {
            wbOffCh1->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_EN_WB_OFF_WB_OFF_CH2_EN_MASK)
        {
            wbOffCh2->CheckState = CheckState::Checked;
        }
        else
        {
            wbOffCh2->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_EN_WB_OFF_WB_OFF_CH3_EN_MASK)
        {
            wbOffCh3->CheckState = CheckState::Checked;
        }
        else
        {
            wbOffCh3->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_EN_WB_OFF_WB_OFF_CH4_EN_MASK)
        {
            wbOffCh4->CheckState = CheckState::Checked;
        }
        else
        {
            wbOffCh4->CheckState = CheckState::Unchecked;
        }

        tempVal1 = curConfig->getWireBreakOn();
        if (tempVal1 & TPS274C65_EN_WB_OFF_WB_OFF_CH1_EN_MASK)
        {
            wbOnCh1->CheckState = CheckState::Checked;
        }
        else
        {
            wbOnCh1->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_EN_WB_OFF_WB_OFF_CH2_EN_MASK)
        {
            wbOnCh2->CheckState = CheckState::Checked;
        }
        else
        {
            wbOnCh2->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_EN_WB_OFF_WB_OFF_CH3_EN_MASK)
        {
            wbOnCh3->CheckState = CheckState::Checked;
        }
        else
        {
            wbOnCh3->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_EN_WB_OFF_WB_OFF_CH4_EN_MASK)
        {
            wbOnCh4->CheckState = CheckState::Checked;
        }
        else
        {
            wbOnCh4->CheckState = CheckState::Unchecked;
        }

        tempVal1 = curConfig->getVSShort();
        if (tempVal1 & TPS274C65_EN_SHRT_VS_SHRT_VS_CH1_EN_MASK)
        {
            shtVsCh1->CheckState = CheckState::Checked;
        }
        else
        {
            shtVsCh1->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_EN_SHRT_VS_SHRT_VS_CH2_EN_MASK)
        {
            shtVsCh2->CheckState = CheckState::Checked;
        }
        else
        {
            shtVsCh2->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_EN_SHRT_VS_SHRT_VS_CH3_EN_MASK)
        {
            shtVsCh3->CheckState = CheckState::Checked;
        }
        else
        {
            shtVsCh3->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_EN_SHRT_VS_SHRT_VS_CH4_EN_MASK)
        {
            shtVsCh4->CheckState = CheckState::Checked;
        }
        else
        {
            shtVsCh4->CheckState = CheckState::Unchecked;
        }

        tempVal1 = curConfig->getADCConf();
        if (tempVal1 & TPS274C65_ADC_CONFIG1_ADC_VS_DIS_MASK)
        {
            adcInVoltageEnableBox->CheckState = CheckState::Unchecked;
        }
        else
        {
            adcInVoltageEnableBox->CheckState = CheckState::Checked;
        }

        if (tempVal1 & TPS274C65_ADC_CONFIG1_ADC_ISNS_DIS_MASK)
        {
            adcCurEnableBox->CheckState = CheckState::Unchecked;
        }
        else
        {
            adcCurEnableBox->CheckState = CheckState::Checked;
        }

        if (tempVal1 & TPS274C65_ADC_CONFIG1_ADC_VSNS_DIS_MASK)
        {
            adcOutVoltageEnableBox->CheckState = CheckState::Unchecked;
        }
        else
        {
            adcOutVoltageEnableBox->CheckState = CheckState::Checked;
        }

        if (tempVal1 & TPS274C65_ADC_CONFIG1_ADC_TSNS_DIS_MASK)
        {
            adcTempEanableBox->CheckState = CheckState::Unchecked;
        }
        else
        {
            adcTempEanableBox->CheckState = CheckState::Checked;
        }

        tempVal1 = curConfig->getVSNSConf();
        if (tempVal1 & TPS274C65_ADC_VSNS_DIS_VSNS_DIS_CH1_MASK)
        {
            voutDisBox1->CheckState = CheckState::Unchecked;
        }
        else
        {
            voutDisBox1->CheckState = CheckState::Checked;
        }

        if (tempVal1 & TPS274C65_ADC_VSNS_DIS_VSNS_DIS_CH2_MASK)
        {
            voutDisBox2->CheckState = CheckState::Unchecked;
        }
        else
        {
            voutDisBox2->CheckState = CheckState::Checked;
        }

        if (tempVal1 & TPS274C65_ADC_VSNS_DIS_VSNS_DIS_CH3_MASK)
        {
            voutDisBox3->CheckState = CheckState::Unchecked;
        }
        else
        {
            voutDisBox3->CheckState = CheckState::Checked;
        }

        if (tempVal1 & TPS274C65_ADC_VSNS_DIS_VSNS_DIS_CH4_MASK)
        {
            voutDisBox4->CheckState = CheckState::Unchecked;
        }
        else
        {
            voutDisBox4->CheckState = CheckState::Checked;
        }

        tempVal1 = curConfig->getISNSConf();
        if (tempVal1 & TPS274C65_ADC_ISNS_DIS_ISNS_DIS_CH1_MASK)
        {
            curDisBox1->CheckState = CheckState::Unchecked;
        }
        else
        {
            curDisBox1->CheckState = CheckState::Checked;
        }

        if (tempVal1 & TPS274C65_ADC_ISNS_DIS_ISNS_DIS_CH2_MASK)
        {
            curDisBox2->CheckState = CheckState::Unchecked;
        }
        else
        {
            curDisBox2->CheckState = CheckState::Checked;
        }

        if (tempVal1 & TPS274C65_ADC_ISNS_DIS_ISNS_DIS_CH3_MASK)
        {
            curDisBox3->CheckState = CheckState::Unchecked;
        }
        else
        {
            curDisBox3->CheckState = CheckState::Checked;
        }

        if (tempVal1 & TPS274C65_ADC_ISNS_DIS_ISNS_DIS_CH4_MASK)
        {
            curDisBox4->CheckState = CheckState::Unchecked;
        }
        else
        {
            curDisBox4->CheckState = CheckState::Checked;
        }

        tempVal1 = curConfig->getTSNSConf();
        if (tempVal1 & TPS274C65_ADC_TSNS_DIS_TSNS_DIS_CH1_MASK)
        {
            tsnsDisBox1->CheckState = CheckState::Unchecked;
        }
        else
        {
            tsnsDisBox1->CheckState = CheckState::Checked;
        }

        if (tempVal1 & TPS274C65_ADC_TSNS_DIS_TSNS_DIS_CH2_MASK)
        {
            tsnsDisBox2->CheckState = CheckState::Unchecked;
        }
        else
        {
            tsnsDisBox2->CheckState = CheckState::Checked;
        }

        if (tempVal1 & TPS274C65_ADC_TSNS_DIS_TSNS_DIS_CH3_MASK)
        {
            tsnsDisBox3->CheckState = CheckState::Unchecked;
        }
        else
        {
            tsnsDisBox3->CheckState = CheckState::Checked;
        }

        if (tempVal1 & TPS274C65_ADC_TSNS_DIS_TSNS_DIS_CH4_MASK)
        {
            tsnsDisBox4->CheckState = CheckState::Unchecked;
        }
        else
        {
            tsnsDisBox4->CheckState = CheckState::Checked;
        }

        tempVal1 = curConfig->getWDFaultChannels();
        if (tempVal1 & TPS274C65_FS_SW_STATE_CH1_FS_ON_MASK)
        {
            keepStateBox1->CheckState = CheckState::Checked;
        }
        else
        {
            keepStateBox1->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_FS_SW_STATE_CH2_FS_ON_MASK)
        {
            keepStateBox2->CheckState = CheckState::Checked;
        }
        else
        {
            keepStateBox2->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_FS_SW_STATE_CH3_FS_ON_MASK)
        {
            keepStateBox3->CheckState = CheckState::Checked;
        }
        else
        {
            keepStateBox3->CheckState = CheckState::Unchecked;
        }

        if (tempVal1 & TPS274C65_FS_SW_STATE_CH4_FS_ON_MASK)
        {
            keepStateBox4->CheckState = CheckState::Checked;
        }
        else
        {
            keepStateBox4->CheckState = CheckState::Unchecked;
        }

        /* Greying out any relevant fields that cannot be changed while 
            the switch is enabled */
        bool changeEnable = (curConfig->getEnable() & 0x0F) == 0 ? true : false;
        inrushLimitBox->Enabled = changeEnable;
        ilimBehaviorBox->Enabled = changeEnable;
        parallel12CheckBox->Enabled = changeEnable;
        parallel34CheckBox->Enabled = changeEnable;
        less800mABox->Enabled = changeEnable;
        greater800mABox->Enabled = changeEnable;
        separateIlimBox->Enabled = changeEnable;
        rcbCheckBox->Enabled = changeEnable;
        chan12CurrentLimitBox->Enabled = changeEnable;
        chan12DurationBox->Enabled = changeEnable;
        chan34CurrentLimitBox->Enabled = changeEnable;
        chan34DurationBox->Enabled = changeEnable;
        rc1Box->Enabled = changeEnable;
        rc2Box->Enabled = changeEnable;
        rc3Box->Enabled = changeEnable;
        rc4Box->Enabled = changeEnable;
        loadConfigMenuItem->Enabled = changeEnable;
        dontRunEvents = oldConfigValue;
    }
}

/* Event handler that gets called when the user clicks to enable/disable channels for the
    current sense. */
System::Void tps274c65configurator::ConfigurationGui::senseChannelBox_ItemCheck(System::Object^ sender, System::Windows::Forms::ItemCheckEventArgs^ e)
{
    uint8_t selectedItems = 0;
    UInt32 count = senseChannelBox->Items->Count;
    bool updateChannelSensing = false;

    /* If we are updating via the "All" box, do not run this event */
    if (dontRunEvents == true)
    {
        return;
    }

    /* Getting the check status */
    if (senseChannelBox->GetItemChecked(0) == true)
    {
        selectedItems |= 0b0001;
    }

    if (senseChannelBox->GetItemChecked(1) == true)
    {
        selectedItems |= 0b0010;
    }

    if (senseChannelBox->GetItemChecked(2) == true)
    {
        selectedItems |= 0b0100;
    }

    if (senseChannelBox->GetItemChecked(3) == true)
    {
        selectedItems |= 0b1000;
    }

    /* Updating with the new state */
    if (e->NewValue == CheckState::Checked)
    {
        senseChart->Series[e->Index]->Enabled = true;
        selectedItems |= (1 << e->Index);
    }
    else
    {
        senseChart->Series[e->Index]->Enabled = false;
        selectedItems &= ~(1 << e->Index);
    }

    /* If everything is disabled, disable the MCU from polling the values */
    if (selectedItems == 0)
    {
        updateChannelSensing = true;
        isCurrentSenseEnabled = false;
    }
    else if ((isCurrentSenseEnabled == false) && selectedItems != 0)
    {
        updateChannelSensing = true;
        isCurrentSenseEnabled = true;
    }

    /* If everything is checked, we need to check the all button */
    dontRunEvents = true;
    if ((senseChannelBox->CheckedItems->Count == (senseChannelBox->Items->Count - 1))
        && e->NewValue == CheckState::Checked)
    {
        senseChannelAll->CheckState = CheckState::Checked;
    }
    else
    {
        senseChannelAll->CheckState = CheckState::Unchecked;
    }
    dontRunEvents = false;

    /* If all boxes are checked or no boxes are checked, we need to enable or disable the MCU
         from polling the sense values */
    if (updateChannelSensing == true)
    {
        /* Seeing what we need to set the offset as */
        uint8_t senseSource = TPS274C65_CURRENT_OFFSET;

        if (senseSourceSelBox->SelectedIndex == 1)
        {
            senseSource = TPS274C65_TEMP_OFFSET;
        }
        else if (senseSourceSelBox->SelectedIndex == 2)
        {
            senseSource = TPS274C65_VOUT_OFFSET;
        }

        /* Constructing the packet to enable the device to do sampling of the
             sense values */
        uint8_t senseEnablePayload[] = { senseSource, (uint8_t)(selectedItems == 0 ? 0 : 1) };
        HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_SENSE_STATE_CONFIG, 0,
            (uint8_t*)senseEnablePayload, 2);
        commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());

        free(cmd);

        if (selectedItems == 0)
        {
            channelOneLabel->Text = "";
            channelTwoLabel->Text = "";
            channelThreeLabel->Text = "";
            channelFourLabel->Text = "";
            channelFourLabel->Text = "";
            inputVoltageValueLabel->Visible = false;
            inputVoltageLabel->Visible = false;
        }
        else
        {
            inputVoltageValueLabel->Visible = true;
            inputVoltageLabel->Visible = true;
        }

        logMessageToWindow("Sense enabled on channels changed to: 0b{0} ", gcnew String(std::bitset<8>(selectedItems).to_string().data()));
    }
}

System::Void tps274c65configurator::ConfigurationGui::senseChannelAll_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    bool selectOrNot = senseChannelAll->Checked;

    for (int ii = 0; ii < senseChannelBox->Items->Count; ii++)
    {
        senseChannelBox->SetItemChecked(ii, selectOrNot);
    }
}

System::Void tps274c65configurator::ConfigurationGui::senseSourceSelBox_SelectedIndexChanged(System::Object^ sender, System::EventArgs^ e)
{
    String^ graphTitle = gcnew String("");
    String^ yTitle = gcnew String("");
    bool oldEnableStatus = isCurrentSenseEnabled;

    if (dontRunEvents == true)
    {
        return;
    }

    /* Getting the right offset */
    if (senseSourceSelBox->SelectedIndex == 0)
    {
        curDataFormat = TPS274C65_CURRENT_OFFSET;
        graphTitle = "Current Sense";
        yTitle = "Current (A)";
        curSenseUnit = "A";
        logMessageToWindow("Sensing source changed to load current");
    }
    else if (senseSourceSelBox->SelectedIndex == 1)
    {
        curDataFormat = TPS274C65_TEMP_OFFSET;
        graphTitle = "FET Junction Temperature";
        yTitle = "Temperature (C)";
        curSenseUnit = "C";
        logMessageToWindow("Sensing source changed to FET junction temperature");
    }
    else
    {
        curDataFormat = TPS274C65_VOUT_OFFSET;
        graphTitle = "Output Voltage";
        yTitle = "Voltage (V)";
        curSenseUnit = "V";
        logMessageToWindow("Sensing source changed to output voltage");
    }

    /* Disabling sampling before changing the sense format*/
    if (isCurrentSenseEnabled)
    {
        isCurrentSenseEnabled = false;
        /* Constructing the packet to enable the device to do sampling of the
             sense values */
        uint8_t senseEnablePayload[] = { curDataFormat , 0 };
        HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_SENSE_STATE_CONFIG, 0, (uint8_t*)senseEnablePayload, 2);
        commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());

        free(cmd);
    }

    /* Cleaning up the graph and changing the titles */
    for (int ii = 0; ii < senseChart->Series->Count; ii++)
    {
        while (senseChart->Series[ii]->Points->Count != 0)
        {
            senseChart->Series[ii]->Points->RemoveAt(0);
        }
    }

    /* Updating all of the labels */
    senseChart->Titles[0]->Text = graphTitle;
    senseChart->ChartAreas[0]->AxisY->Title = yTitle;

    /* Updating the poll sense */
    uint8_t senseEnablePayload[] = { curDataFormat , oldEnableStatus ? 1 : 0};
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_SENSE_STATE_CONFIG, 0, (uint8_t*)senseEnablePayload, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    if (oldEnableStatus)
    {
        isCurrentSenseEnabled = true;
    }
}

System::Void tps274c65configurator::ConfigurationGui::chan12CurrentLimitBox_SelectedIndexChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();
    curConfig->setConfig1(chan12CurrentLimitBox->SelectedIndex | (curConfig->getConfig1() & ~TPS274C65_DEV_CONFIG1_ILIM_REG_12_MASK));
    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG1_REG, curConfig->getConfig1() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::chan12DurationBox_SelectedIndexChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();
    curConfig->setConfig1((chan12DurationBox->SelectedIndex << 4) | (curConfig->getConfig1() & ~TPS274C65_DEV_CONFIG1_ILIM_DURATION_12_MASK));
    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG1_REG, curConfig->getConfig1() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::chan34CurrentLimitBox_SelectedIndexChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();
    curConfig->setConfig2(chan34CurrentLimitBox->SelectedIndex | (curConfig->getConfig2() & ~TPS274C65_DEV_CONFIG2_ILIM_REG_34_MASK));
    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG2_REG, curConfig->getConfig2() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::chan34DurationBox_SelectedIndexChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();
    curConfig->setConfig2((chan34DurationBox->SelectedIndex << 4) | (curConfig->getConfig2() & ~TPS274C65_DEV_CONFIG2_ILIM_DURATION_34_MASK));
    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG2_REG, curConfig->getConfig2() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::separateIlimBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (separateIlimBox->Checked)
    {
        curConfig->setConfig3(curConfig->getConfig3() | TPS274C65_DEV_CONFIG3_ILIM_SET_MASK);
        chan34CurrentLimitBox->Enabled = true;
        chan34DurationBox->Enabled = true;
    }
    else
    {
        curConfig->setConfig3(curConfig->getConfig3() & ~TPS274C65_DEV_CONFIG3_ILIM_SET_MASK);
        chan34CurrentLimitBox->Enabled = false;
        chan34DurationBox->Enabled = false;
    }

    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG3_REG, curConfig->getConfig3() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::ilimBehaviorBox_SelectedIndexChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (ilimBehaviorBox->SelectedIndex == 1)
    {
        curConfig->setConfig3(curConfig->getConfig3() | TPS274C65_DEV_CONFIG3_ILIM_CONFIG_MASK);
    }
    else
    {
        curConfig->setConfig3(curConfig->getConfig3() & ~TPS274C65_DEV_CONFIG3_ILIM_CONFIG_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG3_REG, curConfig->getConfig3() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::inrushLimitBox_SelectedIndexChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();
    curConfig->setConfig3((inrushLimitBox->SelectedIndex) | (curConfig->getConfig3() & ~TPS274C65_DEV_CONFIG3_INRUSH_LIMIT_MASK));
    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG3_REG, curConfig->getConfig3() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::wbOnCh1_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (wbOnCh1->Checked)
    {
        curConfig->setWireBreakOn(curConfig->getWireBreakOn() | TPS274C65_EN_WB_ON_WB_ON_CH1_EN_MASK);
    }
    else
    {
        curConfig->setWireBreakOn(curConfig->getWireBreakOn() & ~TPS274C65_EN_WB_ON_WB_ON_CH1_EN_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_EN_WB_ON_REG, curConfig->getWireBreakOn() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::wbOnCh2_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (wbOnCh2->Checked)
    {
        curConfig->setWireBreakOn(curConfig->getWireBreakOn() | TPS274C65_EN_WB_ON_WB_ON_CH2_EN_MASK);
    }
    else
    {
        curConfig->setWireBreakOn(curConfig->getWireBreakOn() & ~TPS274C65_EN_WB_ON_WB_ON_CH2_EN_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_EN_WB_ON_REG, curConfig->getWireBreakOn() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::wbOnCh3_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (wbOnCh3->Checked)
    {
        curConfig->setWireBreakOn(curConfig->getWireBreakOn() | TPS274C65_EN_WB_ON_WB_ON_CH3_EN_MASK);
    }
    else
    {
        curConfig->setWireBreakOn(curConfig->getWireBreakOn() & ~TPS274C65_EN_WB_ON_WB_ON_CH3_EN_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_EN_WB_ON_REG, curConfig->getWireBreakOn() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::wbOnCh4_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (wbOnCh4->Checked)
    {
        curConfig->setWireBreakOn(curConfig->getWireBreakOn() | TPS274C65_EN_WB_ON_WB_ON_CH4_EN_MASK);
    }
    else
    {
        curConfig->setWireBreakOn(curConfig->getWireBreakOn() & ~TPS274C65_EN_WB_ON_WB_ON_CH4_EN_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_EN_WB_ON_REG, curConfig->getWireBreakOn() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::wbOffCh1_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (wbOffCh1->Checked)
    {
        curConfig->setWireBreakOff(curConfig->getWireBreakOff() | TPS274C65_EN_WB_OFF_WB_OFF_CH1_EN_MASK);
    }
    else
    {
        curConfig->setWireBreakOff(curConfig->getWireBreakOff() & ~TPS274C65_EN_WB_OFF_WB_OFF_CH1_EN_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_EN_WB_OFF_REG, curConfig->getWireBreakOff() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::wbOffCh2_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (wbOffCh2->Checked)
    {
        curConfig->setWireBreakOff(curConfig->getWireBreakOff() | TPS274C65_EN_WB_OFF_WB_OFF_CH2_EN_MASK);
    }
    else
    {
        curConfig->setWireBreakOff(curConfig->getWireBreakOff() & ~TPS274C65_EN_WB_OFF_WB_OFF_CH2_EN_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_EN_WB_OFF_REG, curConfig->getWireBreakOff() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::wbOffCh3_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (wbOffCh3->Checked)
    {
        curConfig->setWireBreakOff(curConfig->getWireBreakOff() | TPS274C65_EN_WB_OFF_WB_OFF_CH3_EN_MASK);
    }
    else
    {
        curConfig->setWireBreakOff(curConfig->getWireBreakOff() & ~TPS274C65_EN_WB_OFF_WB_OFF_CH3_EN_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_EN_WB_OFF_REG, curConfig->getWireBreakOff() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::wbOffCh4_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (wbOffCh4->Checked)
    {
        curConfig->setWireBreakOff(curConfig->getWireBreakOff() | TPS274C65_EN_WB_OFF_WB_OFF_CH4_EN_MASK);
    }
    else
    {
        curConfig->setWireBreakOff(curConfig->getWireBreakOff() & ~TPS274C65_EN_WB_OFF_WB_OFF_CH4_EN_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_EN_WB_OFF_REG, curConfig->getWireBreakOff() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::shtVsCh1_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (shtVsCh1->Checked)
    {
        curConfig->setVsShort(curConfig->getVSShort() | TPS274C65_EN_SHRT_VS_SHRT_VS_CH1_EN_MASK);
    }
    else
    {
        curConfig->setVsShort(curConfig->getVSShort() & ~TPS274C65_EN_SHRT_VS_SHRT_VS_CH1_EN_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_EN_SHRT_VS_REG, curConfig->getVSShort() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::shtVsCh2_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (shtVsCh2->Checked)
    {
        curConfig->setVsShort(curConfig->getVSShort() | TPS274C65_EN_SHRT_VS_SHRT_VS_CH2_EN_MASK);
    }
    else
    {
        curConfig->setVsShort(curConfig->getVSShort() & ~TPS274C65_EN_SHRT_VS_SHRT_VS_CH2_EN_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_EN_SHRT_VS_REG, curConfig->getVSShort() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::shtVsCh3_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (shtVsCh3->Checked)
    {
        curConfig->setVsShort(curConfig->getVSShort() | TPS274C65_EN_SHRT_VS_SHRT_VS_CH3_EN_MASK);
    }
    else
    {
        curConfig->setVsShort(curConfig->getVSShort() & ~TPS274C65_EN_SHRT_VS_SHRT_VS_CH3_EN_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_EN_SHRT_VS_REG, curConfig->getVSShort() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::shtVsCh4_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (shtVsCh4->Checked)
    {
        curConfig->setVsShort(curConfig->getVSShort() | TPS274C65_EN_SHRT_VS_SHRT_VS_CH4_EN_MASK);
    }
    else
    {
        curConfig->setVsShort(curConfig->getVSShort() & ~TPS274C65_EN_SHRT_VS_SHRT_VS_CH4_EN_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_EN_SHRT_VS_REG, curConfig->getVSShort() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::spiMaskBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (spiMaskBox->Checked)
    {
        curConfig->setFaultMask(curConfig->getFaultMask() | TPS274C65_FAULT_MASK_MASK_SPI_ERR_MASK);
    }
    else
    {
        curConfig->setFaultMask(curConfig->getFaultMask() & ~TPS274C65_FAULT_MASK_MASK_SPI_ERR_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_FAULT_MASK_REG, curConfig->getFaultMask() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::wdErrorMaskBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (wdErrorMaskBox->Checked)
    {
        curConfig->setFaultMask(curConfig->getFaultMask() | TPS274C65_FAULT_MASK_MASK_WD_ERR_MASK);
    }
    else
    {
        curConfig->setFaultMask(curConfig->getFaultMask() & ~TPS274C65_FAULT_MASK_MASK_WD_ERR_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_FAULT_MASK_REG, curConfig->getFaultMask() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::ilimMaskBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (ilimMaskBox->Checked)
    {
        curConfig->setFaultMask(curConfig->getFaultMask() | TPS274C65_FAULT_MASK_MASK_ILIMIT_MASK);
    }
    else
    {
        curConfig->setFaultMask(curConfig->getFaultMask() & ~TPS274C65_FAULT_MASK_MASK_ILIMIT_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_FAULT_MASK_REG, curConfig->getFaultMask() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::rcbMaskBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (rcbMaskBox->Checked)
    {
        curConfig->setFaultMask(curConfig->getFaultMask() | TPS274C65_FAULT_MASK_MASK_RVRS_BLK_MASK);
    }
    else
    {
        curConfig->setFaultMask(curConfig->getFaultMask() & ~TPS274C65_FAULT_MASK_MASK_RVRS_BLK_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_FAULT_MASK_REG, curConfig->getFaultMask() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::vsShortMaskBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (vsShortMaskBox->Checked)
    {
        curConfig->setFaultMask(curConfig->getFaultMask() | TPS274C65_FAULT_MASK_MASK_SHRT_VS_MASK);
    }
    else
    {
        curConfig->setFaultMask(curConfig->getFaultMask() & ~TPS274C65_FAULT_MASK_MASK_SHRT_VS_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_FAULT_MASK_REG, curConfig->getFaultMask() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::wbOffMaskBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (wbOffMaskBox->Checked)
    {
        curConfig->setFaultMask(curConfig->getFaultMask() | TPS274C65_FAULT_MASK_MASK_WB_OFF_MASK);
    }
    else
    {
        curConfig->setFaultMask(curConfig->getFaultMask() & ~TPS274C65_FAULT_MASK_MASK_WB_OFF_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_FAULT_MASK_REG, curConfig->getFaultMask() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::wbOnMaskBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (wbOnMaskBox->Checked)
    {
        curConfig->setFaultMask(curConfig->getFaultMask() | TPS274C65_FAULT_MASK_MASK_WB_ON_MASK);
    }
    else
    {
        curConfig->setFaultMask(curConfig->getFaultMask() & ~TPS274C65_FAULT_MASK_MASK_WB_ON_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_FAULT_MASK_REG, curConfig->getFaultMask() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::uvloMaskBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (uvloMaskBox->Checked)
    {
        curConfig->setFaultMask(curConfig->getFaultMask() | TPS274C65_FAULT_MASK_MASK_VS_UV_MASK);
    }
    else
    {
        curConfig->setFaultMask(curConfig->getFaultMask() & ~TPS274C65_FAULT_MASK_MASK_VS_UV_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_FAULT_MASK_REG, curConfig->getFaultMask() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::wbOffCurrrentBox_SelectedIndexChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();
    curConfig->setConfig4((wbOffCurrrentBox->SelectedIndex << 3) | (curConfig->getConfig4() & ~TPS274C65_DEV_CONFIG4_WB_OFF_PU_MASK));
    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG4_REG, curConfig->getConfig4() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::wbOnThreshBox_SelectedIndexChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();
    curConfig->setConfig4((wbOnThreshBox->SelectedIndex) | (curConfig->getConfig4() & ~TPS274C65_DEV_CONFIG4_WB_ON_THD_MASK));
    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG4_REG, curConfig->getConfig4() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::wbBlankBox_SelectedIndexChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();
    curConfig->setConfig5((wbBlankBox->SelectedIndex << 2) | (curConfig->getConfig5() & ~TPS274C65_DEV_CONFIG5_WB_SVS_BLANK_MASK));
    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG5_REG, curConfig->getConfig5() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::latchRadioButton_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (latchRadioButton->Checked == false)
    {
        curConfig->setConfig5(curConfig->getConfig5() | TPS274C65_DEV_CONFIG5_AUTO_RETRY_DIS_MASK);
    }
    else
    {
        curConfig->setConfig5(curConfig->getConfig5() & ~TPS274C65_DEV_CONFIG5_AUTO_RETRY_DIS_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG5_REG, curConfig->getConfig5() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::parallel12CheckBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (parallel12CheckBox->Checked)
    {
        curConfig->setConfig3(curConfig->getConfig3() | TPS274C65_DEV_CONFIG3_PARALLEL_12_MASK);
    }
    else
    {
        curConfig->setConfig3(curConfig->getConfig3() & ~TPS274C65_DEV_CONFIG3_PARALLEL_12_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG3_REG, curConfig->getConfig3() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::parallel34CheckBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (parallel34CheckBox->Checked)
    {
        curConfig->setConfig3(curConfig->getConfig3() | TPS274C65_DEV_CONFIG3_PARALLEL_34_MASK);
    }
    else
    {
        curConfig->setConfig3(curConfig->getConfig3() & ~TPS274C65_DEV_CONFIG3_PARALLEL_34_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG3_REG, curConfig->getConfig3() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::wdEnableBox_CheckStateChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (wdEnableBox->Checked)
    {
        curConfig->setConfig4(curConfig->getConfig4() | TPS274C65_DEV_CONFIG4_WD_EN_MASK);
    }
    else
    {
        curConfig->setConfig4(curConfig->getConfig4() & ~TPS274C65_DEV_CONFIG4_WD_EN_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG4_REG, curConfig->getConfig4() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::wdTimeoutDurationBox_SelectedIndexChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();
    curConfig->setConfig4((wdTimeoutDurationBox->SelectedIndex << 5) | (curConfig->getConfig4() & ~TPS274C65_DEV_CONFIG4_WD_TO_MASK));
    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG4_REG, curConfig->getConfig4() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::fsKeepBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (fsKeepBox->Checked)
    {
        curConfig->setConfig5(curConfig->getConfig5() | TPS274C65_DEV_CONFIG5_SW_FS_CFG_MASK);
        keepStateBox1->Enabled = false;
        keepStateBox2->Enabled = false;
        keepStateBox3->Enabled = false;
        keepStateBox4->Enabled = false;
    }
    else
    {
        curConfig->setConfig5(curConfig->getConfig5() & ~TPS274C65_DEV_CONFIG5_SW_FS_CFG_MASK);
        keepStateBox1->Enabled = true;
        keepStateBox2->Enabled = true;
        keepStateBox3->Enabled = true;
        keepStateBox4->Enabled = true;
    }

    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG5_REG, curConfig->getConfig5() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::rcbCheckBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (rcbCheckBox->Checked)
    {
        curConfig->setConfig3(curConfig->getConfig3() & ~TPS274C65_DEV_CONFIG3_RCB_DIS_MASK);
        rc1Box->Enabled = true;
        rc2Box->Enabled = true;
        rc3Box->Enabled = true;
        rc4Box->Enabled = true;
    }
    else
    {
        curConfig->setConfig3(curConfig->getConfig3() | TPS274C65_DEV_CONFIG3_RCB_DIS_MASK);
        rc1Box->Enabled = false;
        rc2Box->Enabled = false;
        rc3Box->Enabled = false;
        rc4Box->Enabled = false;
    }

    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG3_REG, curConfig->getConfig3() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::rc1Box_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (rc1Box->Checked)
    {
        curConfig->setConfig6(curConfig->getConfig6() | TPS274C65_DEV_CONFIG6_RCB_CH1_MASK);
    }
    else
    {
        curConfig->setConfig6(curConfig->getConfig6() & ~TPS274C65_DEV_CONFIG6_RCB_CH1_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG6_REG, curConfig->getConfig6() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::rc2Box_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (rc2Box->Checked)
    {
        curConfig->setConfig6(curConfig->getConfig6() | TPS274C65_DEV_CONFIG6_RCB_CH2_MASK);
    }
    else
    {
        curConfig->setConfig6(curConfig->getConfig6() & ~TPS274C65_DEV_CONFIG6_RCB_CH2_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG6_REG, curConfig->getConfig6() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::rc3Box_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (rc3Box->Checked)
    {
        curConfig->setConfig6(curConfig->getConfig6() | TPS274C65_DEV_CONFIG6_RCB_CH3_MASK);
    }
    else
    {
        curConfig->setConfig6(curConfig->getConfig6() & ~TPS274C65_DEV_CONFIG6_RCB_CH3_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG6_REG, curConfig->getConfig6() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::rc4Box_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (rc4Box->Checked)
    {
        curConfig->setConfig6(curConfig->getConfig6() | TPS274C65_DEV_CONFIG6_RCB_CH4_MASK);
    }
    else
    {
        curConfig->setConfig6(curConfig->getConfig6() & ~TPS274C65_DEV_CONFIG6_RCB_CH4_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG6_REG, curConfig->getConfig6() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::latchDisableBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (latchDisableBox->Checked)
    {
        curConfig->setConfig5(curConfig->getConfig5() & ~TPS274C65_DEV_CONFIG5_FLT_LTCH_DIS_MASK);
    }
    else
    {
        curConfig->setConfig5(curConfig->getConfig5() | TPS274C65_DEV_CONFIG5_FLT_LTCH_DIS_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG5_REG, curConfig->getConfig5() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::adcEnableBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (adcEnableBox->Checked)
    {
        curConfig->setConfig5(curConfig->getConfig5() | TPS274C65_DEV_CONFIG5_ADC_EN_MASK);
    }
    else
    {
        curConfig->setConfig5(curConfig->getConfig5() & ~TPS274C65_DEV_CONFIG5_ADC_EN_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG5_REG, curConfig->getConfig5() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::adcCurEnableBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (adcCurEnableBox->Checked)
    {
        curConfig->setADCConf(curConfig->getADCConf() & ~TPS274C65_ADC_CONFIG1_ADC_ISNS_DIS_MASK);
    }
    else
    {
        curConfig->setADCConf(curConfig->getADCConf() | TPS274C65_ADC_CONFIG1_ADC_ISNS_DIS_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_ADC_CONFIG1_REG, curConfig->getADCConf() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::adcInVoltageEnableBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (adcInVoltageEnableBox->Checked)
    {
        curConfig->setADCConf(curConfig->getADCConf() & ~TPS274C65_ADC_CONFIG1_ADC_VS_DIS_MASK);
    }
    else
    {
        curConfig->setADCConf(curConfig->getADCConf() | TPS274C65_ADC_CONFIG1_ADC_VS_DIS_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_ADC_CONFIG1_REG, curConfig->getADCConf() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::adcOutVoltageEnableBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (adcOutVoltageEnableBox->Checked)
    {
        curConfig->setADCConf(curConfig->getADCConf() & ~TPS274C65_ADC_CONFIG1_ADC_VSNS_DIS_MASK);
    }
    else
    {
        curConfig->setADCConf(curConfig->getADCConf() | TPS274C65_ADC_CONFIG1_ADC_VSNS_DIS_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_ADC_CONFIG1_REG, curConfig->getADCConf() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::adcTempEanableBox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (adcTempEanableBox->Checked)
    {
        curConfig->setADCConf(curConfig->getADCConf() & ~TPS274C65_ADC_CONFIG1_ADC_TSNS_DIS_MASK);
    }
    else
    {
        curConfig->setADCConf(curConfig->getADCConf() | TPS274C65_ADC_CONFIG1_ADC_TSNS_DIS_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_ADC_CONFIG1_REG, curConfig->getADCConf() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::less800mABox_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (less800mABox->Checked)
    {
        curConfig->setConfig3(curConfig->getConfig3() & ~TPS274C65_DEV_CONFIG3_ISNS_RANGE_MASK);
    }
    else
    {
        curConfig->setConfig3(curConfig->getConfig3() | TPS274C65_DEV_CONFIG3_ISNS_RANGE_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_DEV_CONFIG3_REG, curConfig->getConfig3() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::tsnsDisBox1_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (tsnsDisBox1->Checked)
    {
        curConfig->setTSNSConf(curConfig->getTSNSConf() & ~TPS274C65_ADC_TSNS_DIS_TSNS_DIS_CH1_MASK);
    }
    else
    {
        curConfig->setTSNSConf(curConfig->getTSNSConf() | TPS274C65_ADC_TSNS_DIS_TSNS_DIS_CH1_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_ADC_TSNS_DIS_REG, curConfig->getTSNSConf() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::tsnsDisBox2_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (tsnsDisBox2->Checked)
    {
        curConfig->setTSNSConf(curConfig->getTSNSConf() & ~TPS274C65_ADC_TSNS_DIS_TSNS_DIS_CH2_MASK);
    }
    else
    {
        curConfig->setTSNSConf(curConfig->getTSNSConf() | TPS274C65_ADC_TSNS_DIS_TSNS_DIS_CH2_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_ADC_TSNS_DIS_REG, curConfig->getTSNSConf() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::tsnsDisBox3_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (tsnsDisBox3->Checked)
    {
        curConfig->setTSNSConf(curConfig->getTSNSConf() & ~TPS274C65_ADC_TSNS_DIS_TSNS_DIS_CH3_MASK);
    }
    else
    {
        curConfig->setTSNSConf(curConfig->getTSNSConf() | TPS274C65_ADC_TSNS_DIS_TSNS_DIS_CH3_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_ADC_TSNS_DIS_REG, curConfig->getTSNSConf() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::tsnsDisBox4_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (tsnsDisBox4->Checked)
    {
        curConfig->setTSNSConf(curConfig->getTSNSConf() & ~TPS274C65_ADC_TSNS_DIS_TSNS_DIS_CH4_MASK);
    }
    else
    {
        curConfig->setTSNSConf(curConfig->getTSNSConf() | TPS274C65_ADC_TSNS_DIS_TSNS_DIS_CH4_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_ADC_TSNS_DIS_REG, curConfig->getTSNSConf() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::voutDisBox1_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (voutDisBox1->Checked)
    {
        curConfig->setVSNSConf(curConfig->getVSNSConf() & ~TPS274C65_ADC_VSNS_DIS_VSNS_DIS_CH1_MASK);
    }
    else
    {
        curConfig->setVSNSConf(curConfig->getVSNSConf() | TPS274C65_ADC_VSNS_DIS_VSNS_DIS_CH1_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_ADC_VSNS_DIS_REG, curConfig->getVSNSConf() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::voutDisBox2_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (voutDisBox2->Checked)
    {
        curConfig->setVSNSConf(curConfig->getVSNSConf() & ~TPS274C65_ADC_VSNS_DIS_VSNS_DIS_CH2_MASK);
    }
    else
    {
        curConfig->setVSNSConf(curConfig->getVSNSConf() | TPS274C65_ADC_VSNS_DIS_VSNS_DIS_CH2_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_ADC_VSNS_DIS_REG, curConfig->getVSNSConf() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::voutDisBox3_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (voutDisBox3->Checked)
    {
        curConfig->setVSNSConf(curConfig->getVSNSConf() & ~TPS274C65_ADC_VSNS_DIS_VSNS_DIS_CH3_MASK);
    }
    else
    {
        curConfig->setVSNSConf(curConfig->getVSNSConf() | TPS274C65_ADC_VSNS_DIS_VSNS_DIS_CH3_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_ADC_VSNS_DIS_REG, curConfig->getVSNSConf() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::voutDisBox4_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (voutDisBox4->Checked)
    {
        curConfig->setVSNSConf(curConfig->getVSNSConf() & ~TPS274C65_ADC_VSNS_DIS_VSNS_DIS_CH4_MASK);
    }
    else
    {
        curConfig->setVSNSConf(curConfig->getVSNSConf() | TPS274C65_ADC_VSNS_DIS_VSNS_DIS_CH4_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_ADC_VSNS_DIS_REG, curConfig->getVSNSConf() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::curDisBox1_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (curDisBox1->Checked)
    {
        curConfig->setISNSConf(curConfig->getISNSConf() & ~TPS274C65_ADC_ISNS_DIS_ISNS_DIS_CH1_MASK);
    }
    else
    {
        curConfig->setISNSConf(curConfig->getISNSConf() | TPS274C65_ADC_ISNS_DIS_ISNS_DIS_CH1_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_ADC_ISNS_DIS_REG, curConfig->getISNSConf() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::curDisBox2_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (curDisBox2->Checked)
    {
        curConfig->setISNSConf(curConfig->getISNSConf() & ~TPS274C65_ADC_ISNS_DIS_ISNS_DIS_CH2_MASK);
    }
    else
    {
        curConfig->setISNSConf(curConfig->getISNSConf() | TPS274C65_ADC_ISNS_DIS_ISNS_DIS_CH2_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_ADC_ISNS_DIS_REG, curConfig->getISNSConf() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::curDisBox3_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (curDisBox3->Checked)
    {
        curConfig->setISNSConf(curConfig->getISNSConf() & ~TPS274C65_ADC_ISNS_DIS_ISNS_DIS_CH3_MASK);
    }
    else
    {
        curConfig->setISNSConf(curConfig->getISNSConf() | TPS274C65_ADC_ISNS_DIS_ISNS_DIS_CH3_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_ADC_ISNS_DIS_REG, curConfig->getISNSConf() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::curDisBox4_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (curDisBox4->Checked)
    {
        curConfig->setISNSConf(curConfig->getISNSConf() & ~TPS274C65_ADC_ISNS_DIS_ISNS_DIS_CH4_MASK);
    }
    else
    {
        curConfig->setISNSConf(curConfig->getISNSConf() | TPS274C65_ADC_ISNS_DIS_ISNS_DIS_CH4_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_ADC_ISNS_DIS_REG, curConfig->getISNSConf() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::keepStateBox1_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (keepStateBox1->Checked)
    {
        curConfig->setWDFaultChannels(curConfig->getWDFaultChannels() | TPS274C65_FS_SW_STATE_CH1_FS_ON_MASK);
    }
    else
    {
        curConfig->setWDFaultChannels(curConfig->getWDFaultChannels() & ~TPS274C65_FS_SW_STATE_CH1_FS_ON_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_FS_SW_STATE_REG, curConfig->getWDFaultChannels() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::keepStateBox2_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (keepStateBox2->Checked)
    {
        curConfig->setWDFaultChannels(curConfig->getWDFaultChannels() | TPS274C65_FS_SW_STATE_CH2_FS_ON_MASK);
    }
    else
    {
        curConfig->setWDFaultChannels(curConfig->getWDFaultChannels() & ~TPS274C65_FS_SW_STATE_CH2_FS_ON_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_FS_SW_STATE_REG, curConfig->getWDFaultChannels() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::keepStateBox3_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (keepStateBox3->Checked)
    {
        curConfig->setWDFaultChannels(curConfig->getWDFaultChannels() | TPS274C65_FS_SW_STATE_CH3_FS_ON_MASK);
    }
    else
    {
        curConfig->setWDFaultChannels(curConfig->getWDFaultChannels() & ~TPS274C65_FS_SW_STATE_CH3_FS_ON_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_FS_SW_STATE_REG, curConfig->getWDFaultChannels() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

System::Void tps274c65configurator::ConfigurationGui::keepStateBox4_CheckedChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (dontRunEvents == true)
    {
        return;
    }

    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    if (keepStateBox4->Checked)
    {
        curConfig->setWDFaultChannels(curConfig->getWDFaultChannels() | TPS274C65_FS_SW_STATE_CH4_FS_ON_MASK);
    }
    else
    {
        curConfig->setWDFaultChannels(curConfig->getWDFaultChannels() & ~TPS274C65_FS_SW_STATE_CH4_FS_ON_MASK);
    }

    uint8_t tempBuffer[2] = { TPS274C65_FS_SW_STATE_REG, curConfig->getWDFaultChannels() };

    /* Sending the updated data */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
}

int tps274c65configurator::ConfigurationGui::char2int(char input)
{
    if (input >= '0' && input <= '9')
        return input - '0';
    if (input >= 'A' && input <= 'F')
        return input - 'A' + 10;
    if (input >= 'a' && input <= 'f')
        return input - 'a' + 10;
    return 0;
}

// This function assumes src to be a zero terminated sanitized string with
// an even number of [0-9a-f] characters, and target to be sufficiently large
void tps274c65configurator::ConfigurationGui::hex2bin(const char* src, uint8_t* target)
{
    while (*src && src[1])
    {
        *(target++) = char2int(*src) * 16 + char2int(src[1]);
        src += 2;
    }
}

System::Void tps274c65configurator::ConfigurationGui::sendCommandButton_Click(System::Object^ sender, System::EventArgs^ e)
{
    String^ curCommand = commandBox->Text;
    array<String^>^ splitString = curCommand->Split(' ');
    CondorConfiguration^ curConfig = getCurrentCondorConfig();
    uint8_t curOpCode;
    uint8_t readRegAddr;
    String^ tempString;
    char* convertedString;
    uint8_t payload[2] = { 0,0 };
    HSSCommand* cmd;
    tHSSReceiveTransaction^ readTrans;

    if (splitString->Length == 0)
    {
        commandLogBox->AppendText("INVALID COMMAND: " + curCommand + Environment::NewLine);
        return;
    }

    if (splitString[0]->ToUpper() == "READ")
    {
        if (splitString->Length != 2)
        {
            commandLogBox->AppendText("INVALID FORMAT! Read Command:  READ <REG_NUM_HEX>" + Environment::NewLine);
            return;
        }
        curOpCode = HSSCommand::HSS_TPS274C65_RAW_READ;

        tempString = splitString[1]->Replace("0x", "");
        convertedString = (char*)(void*)Marshal::StringToHGlobalAnsi(tempString);
        hex2bin(convertedString, &readRegAddr);
        payload[0] = readRegAddr;

        cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_READ, curConfig->getDeviceAddress(), payload, 1);
        commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
        commandLogBox->AppendText("READ REGISTER: 0x" + splitString[1] + "\r\n");

        readTrans = gcnew tHSSReceiveTransaction;
        readTrans->addr = curConfig->getDeviceAddress();
        readTrans->recData = (uint8_t*)malloc(128);
        readTrans->reportID = HSSCommunicationManager::HSS_RAWREAD_ID;
        readTrans->receiveSize = HSSCommand::HSS_TPS274C65_RAW_READ_RESP_SIZE;
        commManager->getPacketData(readTrans);
    }
    else if (splitString[0] == "WRITE")
    {
        if (splitString->Length != 3)
        {
            commandLogBox->AppendText("INVALID FORMAT! Read Command:  WRITE <REG_NUM_HEX> <REG_WRITE_DATA_HEX>" + Environment::NewLine);
            return;
        }

        curOpCode = HSSCommand::HSS_TPS274C65_RAW_WRITE;

        tempString = splitString[1]->Replace("0x", "");
        convertedString = (char*)(void*)Marshal::StringToHGlobalAnsi(tempString);
        hex2bin(convertedString, &readRegAddr);
        payload[0] = readRegAddr;

        tempString = splitString[2]->Replace("0x", "");
        convertedString = (char*)(void*)Marshal::StringToHGlobalAnsi(tempString);
        hex2bin(convertedString, &readRegAddr);
        payload[1] = readRegAddr;

        cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), payload, 2);
        commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
        commandLogBox->AppendText("WRITE REGISTER: 0x" + splitString[1] + " 0x" + splitString[2] + "\r\n");
        commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    }
    else
    {
        commandLogBox->AppendText("INVALID COMMAND: " + splitString[0] + Environment::NewLine);
    }
}

System::Void tps274c65configurator::ConfigurationGui::commandBox_KeyDown(System::Object^ sender, System::Windows::Forms::KeyEventArgs^ e)
{
    if (e->KeyCode == Keys::Enter)
    {
        sendCommandButton_Click(sender, e);
    }
}

System::Void tps274c65configurator::ConfigurationGui::porFaultBox_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = porFault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::uvFaultBox_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = uvFault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::uvWarnBox_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = uvWarnFault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::handleFaultClick(HSSFault^ fault)
{
    this->faultDescriptionLabel->Text = fault->getFaultDescription();
    this->faultNameLabel->Text = fault->getFaultName();

    if (fault->isFaulted())
    {
        currentFaultLabelStatus->Text = "Faulted";
        currentFaultLabelStatus->ForeColor = Color::Red;
    }
    else
    {
        currentFaultLabelStatus->Text = "Not Faulted";
        currentFaultLabelStatus->ForeColor = Color::Green;
    }

    numberFaultLabel->Text = fault->getFaultCount().ToString();
    dateFaultLabel->Text = fault->getFaultDate();
}

System::Void tps274c65configurator::ConfigurationGui::resetStatButton_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault->resetFaultState();
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::thermalFaultBox_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = tsdFault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::spiFaultBox_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = spiFault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::wdtFaultBox_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = wdFault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::vddUVFaultBox_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = vdduvloFault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::shtsFaultBoxCh1_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = sht1Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::shtsFaultBoxCh2_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = sht2Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::shtsFaultBoxCh3_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = sht3Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::shtsFaultBoxCh4_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = sht4Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::wbOffFaultCh1_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = wbOff1Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::wbOffFaultCh2_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = wbOff2Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::wbOffFaultCh3_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = wbOff3Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::wbOffFaultCh4_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = wbOff4Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::wbOnFaultCh1_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = wbOn1Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::wbOnFaultCh2_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = wbOn2Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::wbOnFaultCh3_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = wbOn3Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::wbOnFaultCh4_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = wbOn4Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::ilimFaultBoxCh1_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = ilim1Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::ilimFaultBoxCh2_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = ilim2Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::ilimFaultBoxCh3_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = ilim3Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::ilimFaultBoxCh4_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = ilim4Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::tsdFaultBoxCh1_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = tsd1Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::tsdFaultBoxCh2_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = tsd2Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::tsdFaultBoxCh3_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = tsd3Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::tsdFaultBoxCh4_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = tsd4Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::tWarnBoxCh1_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = tw1Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::tWarnBoxCh2_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = tw2Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::tWarnBoxCh3_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = tw3Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::tWarnBoxCh4_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = tw4Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::rvsrBoxCh1_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = rvsr1Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::rvsrBoxCh2_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = rvsr2Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::rvsrBoxCh3_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = rvsr3Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::rvsrBoxCh4_Click(System::Object^ sender, System::EventArgs^ e)
{
    curFault = rvsr4Fault;
    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::faultBox1_Click_1(System::Object^ sender, System::EventArgs^ e)
{
    mainTabControl->SelectedIndex = 1;
}

System::Void tps274c65configurator::ConfigurationGui::enterDFUModeToolStripMenuItem_Click(System::Object^ sender, System::EventArgs^ e)
{
    uint8_t tempBuffer[1];
    HSSCommand* cmd;

    /* Sending the DFU command. The device will disconnect after this */
    tempBuffer[0] = 0;
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_ENTER_WRITE, condorConfig1->getDeviceAddress(), tempBuffer, 1);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    MessageBox::Show("Device Firmware Update (DFU) entry command sent. Use dfuprog.exe to update firmware.",
        "DFU Entry Command Sent",
        MessageBoxButtons::OK, MessageBoxIcon::Information);
}

System::Void tps274c65configurator::ConfigurationGui::aboutSoftwareToolStripMenuItem_Click(System::Object^ sender, System::EventArgs^ e)
{
    String^ target = "https://www.ti.com/lit/pdf/slvucn6";
    System::Diagnostics::Process::Start(target);
}

System::Void tps274c65configurator::ConfigurationGui::clearAllStatsButton_Click(System::Object^ sender, System::EventArgs^ e)
{
    porFault->resetFaultState();
    uvFault->resetFaultState();
    uvWarnFault->resetFaultState();
    tsdFault->resetFaultState();
    spiFault->resetFaultState();
    wdFault->resetFaultState();
    vdduvloFault->resetFaultState();
    sht1Fault->resetFaultState();
    sht2Fault->resetFaultState();
    sht3Fault->resetFaultState();
    sht4Fault->resetFaultState();
    wbOff1Fault->resetFaultState();
    wbOff2Fault->resetFaultState();
    wbOff3Fault->resetFaultState();
    wbOff4Fault->resetFaultState();
    wbOn1Fault->resetFaultState();
    wbOn2Fault->resetFaultState();
    wbOn3Fault->resetFaultState();
    wbOn4Fault->resetFaultState();
    ilim1Fault->resetFaultState();
    ilim2Fault->resetFaultState();
    ilim3Fault->resetFaultState();
    ilim4Fault->resetFaultState();
    tsd1Fault->resetFaultState();
    tsd2Fault->resetFaultState();
    tsd3Fault->resetFaultState();
    tsd4Fault->resetFaultState();
    tw1Fault->resetFaultState();
    tw2Fault->resetFaultState();
    tw3Fault->resetFaultState();
    tw4Fault->resetFaultState();
    rvsr1Fault->resetFaultState();
    rvsr2Fault->resetFaultState();
    rvsr3Fault->resetFaultState();
    rvsr4Fault->resetFaultState();

    handleFaultClick(curFault);
}

System::Void tps274c65configurator::ConfigurationGui::faultUpdateStatus_SelectedIndexChanged(System::Object^ sender, System::EventArgs^ e)
{
    if (faultUpdateStatus->SelectedIndex == 0)
    {
        manualUpdateEnabled = false;
        manualUpdateButton->Enabled = false;
    }
    else
    {
        manualUpdateEnabled = true;
        manualUpdateButton->Enabled = true;
    }
}

System::Void tps274c65configurator::ConfigurationGui::manualUpdateButton_Click(System::Object^ sender, System::EventArgs^ e)
{
    HSSCommand* faultStatus1;
    HSSCommand* faultStatus2;
    tHSSReceiveTransaction^ faultTrans1;
    tHSSReceiveTransaction^ faultTrans2;
    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    /* Reading the update fault data */
    if (curConfig->getDeviceAddress() == UNIT_1_ADDR)
    {
        faultStatus1 = new HSSCommand(HSSCommand::HSS_TPS274C65_FAULT_STATE_CONFIG, UNIT_1_ADDR, NULL, 0);
        commManager->sendPacketData(faultStatus1->getBufferData(), faultStatus1->getCommandSize());
        faultTrans1 = gcnew tHSSReceiveTransaction;
        faultTrans1->addr = UNIT_1_ADDR;
        faultTrans1->recData = (uint8_t*)malloc(128);
        faultTrans1->reportID = HSSCommunicationManager::HSS_FAULT_ID;
        faultTrans1->receiveSize = HSSCommand::HSS_TPS274C65_FAULT_STATE_CONFIG_RESP_SIZE;
        commManager->getPacketData(faultTrans1);

        free(faultStatus1);
    }
    else
    {
        faultStatus2 = new HSSCommand(HSSCommand::HSS_TPS274C65_FAULT_STATE_CONFIG, UNIT_2_ADDR, NULL, 0);
        commManager->sendPacketData(faultStatus2->getBufferData(), faultStatus2->getCommandSize());
        faultTrans2 = gcnew tHSSReceiveTransaction;
        faultTrans2->addr = UNIT_2_ADDR;
        faultTrans2->recData = (uint8_t*)malloc(128);
        faultTrans2->reportID = HSSCommunicationManager::HSS_FAULT_ID;
        faultTrans2->receiveSize = HSSCommand::HSS_TPS274C65_FAULT_STATE_CONFIG_RESP_SIZE;
        commManager->getPacketData(faultTrans2);
        free(faultStatus2);
    }
}

System::Void tps274c65configurator::ConfigurationGui::saveMenuItem_Click(System::Object^ sender, System::EventArgs^ e)
{
    SaveFileDialog^ saveFileDialog1 = gcnew SaveFileDialog;
    System::IO::StreamWriter^ sw;

    saveFileDialog1->Filter = "txt files (*.txt)|*.txt";
    saveFileDialog1->FilterIndex = 2;
    saveFileDialog1->RestoreDirectory = true;

    if (saveFileDialog1->ShowDialog() == System::Windows::Forms::DialogResult::OK)
    {
        sw = gcnew IO::StreamWriter(saveFileDialog1->FileName);

        sw->WriteLine(magicFileWord);
        sw->WriteLine("/* Device Address #1 (0x02) */");
        sw->WriteLine(condorConfig1->getDeviceAddress());
        sw->WriteLine("/* TPS274C65_SW_STATE */");
        sw->WriteLine("{0},{1}", TPS274C65_SW_STATE_REG, condorConfig1->getEnable());
        sw->WriteLine("/* TPS274C65_LED1_4_CTL */");
        sw->WriteLine("{0},{1}", TPS274C65_LED1_4_CTL_REG, condorConfig1->getLED1());
        sw->WriteLine("/* TPS274C65_LED_5_8_CTL */");
        sw->WriteLine("{0},{1}", TPS274C65_LED_5_8_CTL_REG, condorConfig1->getLED2());
        sw->WriteLine("/* TPS274C65_FS_SW_STATE */");
        sw->WriteLine("{0},{1}", TPS274C65_FS_SW_STATE_REG, condorConfig1->getWDFaultChannels());
        sw->WriteLine("/* TPS274C65_DEV_CONFIG1 */");
        sw->WriteLine("{0},{1}", TPS274C65_DEV_CONFIG1_REG, condorConfig1->getConfig1());
        sw->WriteLine("/* TPS274C65_DEV_CONFIG2 */");
        sw->WriteLine("{0},{1}", TPS274C65_DEV_CONFIG2_REG, condorConfig1->getConfig2());
        sw->WriteLine("/* TPS274C65_DEV_CONFIG3 */");
        sw->WriteLine("{0},{1}", TPS274C65_DEV_CONFIG3_REG, condorConfig1->getConfig3());
        sw->WriteLine("/* TPS274C65_DEV_CONFIG4 */");
        sw->WriteLine("{0},{1}", TPS274C65_DEV_CONFIG4_REG, condorConfig1->getConfig4());
        sw->WriteLine("/* TPS274C65_DEV_CONFIG5 */");
        sw->WriteLine("{0},{1}", TPS274C65_DEV_CONFIG5_REG, condorConfig1->getConfig5());
        sw->WriteLine("/* TPS274C65_DEV_CONFIG6 */");
        sw->WriteLine("{0},{1}", TPS274C65_DEV_CONFIG6_REG, condorConfig1->getConfig6());
        sw->WriteLine("/* TPS274C65_FAULT_MASK */");
        sw->WriteLine("{0},{1}", TPS274C65_FAULT_MASK_REG, condorConfig1->getFaultMask());
        sw->WriteLine("/* TPS274C65_EN_WB_OFF */");
        sw->WriteLine("{0},{1}", TPS274C65_EN_WB_OFF_REG, condorConfig1->getWireBreakOff());
        sw->WriteLine("/* TPS274C65_EN_WB_ON */");
        sw->WriteLine("{0},{1}", TPS274C65_EN_WB_ON_REG, condorConfig1->getWireBreakOn());
        sw->WriteLine("/* TPS274C65_EN_SHRT_VS */");
        sw->WriteLine("{0},{1}", TPS274C65_EN_SHRT_VS_REG, condorConfig1->getVSShort());
        sw->WriteLine("/* TPS274C65_ADC_ISNS_DIS */");
        sw->WriteLine("{0},{1}", TPS274C65_ADC_ISNS_DIS_REG, condorConfig1->getISNSConf());
        sw->WriteLine("/* TPS274C65_ADC_TSNS_DIS */");
        sw->WriteLine("{0},{1}", TPS274C65_ADC_TSNS_DIS_REG, condorConfig1->getTSNSConf());
        sw->WriteLine("/* TPS274C65_ADC_VSNS_DIS */");
        sw->WriteLine("{0},{1}", TPS274C65_ADC_VSNS_DIS_REG, condorConfig1->getVSNSConf());
        sw->WriteLine("/* TPS274C65_ADC_CONFIG1 */");
        sw->WriteLine("{0},{1}", TPS274C65_ADC_CONFIG1_REG, condorConfig1->getADCConf());

        sw->WriteLine("/* Device Address #2 (0x05) */");
        sw->WriteLine(condorConfig2->getDeviceAddress());
        sw->WriteLine("/* TPS274C65_SW_STATE */");
        sw->WriteLine("{0},{1}", TPS274C65_SW_STATE_REG, condorConfig2->getEnable());
        sw->WriteLine("/* TPS274C65_LED1_4_CTL */");
        sw->WriteLine("{0},{1}", TPS274C65_LED1_4_CTL_REG, condorConfig2->getLED1());
        sw->WriteLine("/* TPS274C65_LED_5_8_CTL */");
        sw->WriteLine("{0},{1}", TPS274C65_LED_5_8_CTL_REG, condorConfig2->getLED2());
        sw->WriteLine("/* TPS274C65_FS_SW_STATE */");
        sw->WriteLine("{0},{1}", TPS274C65_FS_SW_STATE_REG, condorConfig2->getWDFaultChannels());
        sw->WriteLine("/* TPS274C65_DEV_CONFIG1 */");
        sw->WriteLine("{0},{1}", TPS274C65_DEV_CONFIG1_REG, condorConfig2->getConfig1());
        sw->WriteLine("/* TPS274C65_DEV_CONFIG2 */");
        sw->WriteLine("{0},{1}", TPS274C65_DEV_CONFIG2_REG, condorConfig2->getConfig2());
        sw->WriteLine("/* TPS274C65_DEV_CONFIG3 */");
        sw->WriteLine("{0},{1}", TPS274C65_DEV_CONFIG3_REG, condorConfig2->getConfig3());
        sw->WriteLine("/* TPS274C65_DEV_CONFIG4 */");
        sw->WriteLine("{0},{1}", TPS274C65_DEV_CONFIG4_REG, condorConfig2->getConfig4());
        sw->WriteLine("/* TPS274C65_DEV_CONFIG5 */");
        sw->WriteLine("{0},{1}", TPS274C65_DEV_CONFIG5_REG, condorConfig2->getConfig5());
        sw->WriteLine("/* TPS274C65_DEV_CONFIG6 */");
        sw->WriteLine("{0},{1}", TPS274C65_DEV_CONFIG6_REG, condorConfig2->getConfig6());
        sw->WriteLine("/* TPS274C65_FAULT_MASK */");
        sw->WriteLine("{0},{1}", TPS274C65_FAULT_MASK_REG, condorConfig2->getFaultMask());
        sw->WriteLine("/* TPS274C65_EN_WB_OFF */");
        sw->WriteLine("{0},{1}", TPS274C65_EN_WB_OFF_REG, condorConfig2->getWireBreakOff());
        sw->WriteLine("/* TPS274C65_EN_WB_ON */");
        sw->WriteLine("{0},{1}", TPS274C65_EN_WB_ON_REG, condorConfig2->getWireBreakOn());
        sw->WriteLine("/* TPS274C65_EN_SHRT_VS */");
        sw->WriteLine("{0},{1}", TPS274C65_EN_SHRT_VS_REG, condorConfig2->getVSShort());
        sw->WriteLine("/* TPS274C65_ADC_ISNS_DIS */");
        sw->WriteLine("{0},{1}", TPS274C65_ADC_ISNS_DIS_REG, condorConfig2->getISNSConf());
        sw->WriteLine("/* TPS274C65_ADC_TSNS_DIS */");
        sw->WriteLine("{0},{1}", TPS274C65_ADC_TSNS_DIS_REG, condorConfig2->getTSNSConf());
        sw->WriteLine("/* TPS274C65_ADC_VSNS_DIS */");
        sw->WriteLine("{0},{1}", TPS274C65_ADC_VSNS_DIS_REG, condorConfig2->getVSNSConf());
        sw->WriteLine("/* TPS274C65_ADC_CONFIG1 */");
        sw->WriteLine("{0},{1}", TPS274C65_ADC_CONFIG1_REG, condorConfig2->getADCConf());

        sw->Close();

        logMessageToWindow("Saved configuration to {0}", saveFileDialog1->FileName);

        MessageBox::Show("Configuration saved tp " + saveFileDialog1->FileName, "Configuration Saved",
            MessageBoxButtons::OK, MessageBoxIcon::Information);

    }
}

System::Void tps274c65configurator::ConfigurationGui::loadConfigMenuItem_Click(System::Object^ sender, System::EventArgs^ e)
{
    OpenFileDialog^ openFileDialog1 = gcnew OpenFileDialog;
    System::IO::StreamReader^ sr;
    String^ curString;
    array<String^>^ splitString;
    uint8_t curReg, curVal;
    tHSSReceiveTransaction^ trans;

    if ((condorConfig1->getEnable() & 0xF) || (condorConfig2->getEnable() & 0xF))
    {
        MessageBox::Show("Cannot load configuration while channels are enabled. Disable all channels (on both devices) and try again.",
                            "Channels Enabled!",
                            MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
        return;
    }

    openFileDialog1->Filter = "txt files (*.txt)|*.txt";
    openFileDialog1->FilterIndex = 2;
    openFileDialog1->RestoreDirectory = true;

    if (openFileDialog1->ShowDialog() == System::Windows::Forms::DialogResult::OK)
    {
        sr = gcnew IO::StreamReader(openFileDialog1->FileName);
        curString = sr->ReadLine();
        if (curString != magicFileWord)
        {
            MessageBox::Show("File is in wrong format.", "Error Opening File!",
                    MessageBoxButtons::OK, MessageBoxIcon::Error);
            sr->Close();
            return;
        }

        /* Getting the first address */
        curString = sr->ReadLine();
        curString = sr->ReadLine();
        if (curString != UNIT_1_ADDR.ToString())
        {
            MessageBox::Show("Parse Error!", "Error parsing the opened file.",
                MessageBoxButtons::OK, MessageBoxIcon::Error);
            sr->Close();
            return;
        }
        curString = sr->ReadLine();
        if (curString == nullptr)
        {
            sr->Close();
            return;
        }

        do 
        {
            if (curString->Contains("/*"))
            {
                curString = sr->ReadLine();
                continue;
            }

            splitString = curString->Split(',');
            if (splitString->Length != 2)
            {
                curString = sr->ReadLine();
                continue;
            }
            curReg = int::Parse(splitString[0]);
            curVal = int::Parse(splitString[1]);

            switch(curReg)
            {
            case TPS274C65_FS_SW_STATE_REG:
                condorConfig1->setWDFaultChannels(curVal);
                break;
            case TPS274C65_DEV_CONFIG1_REG:
                condorConfig1->setConfig1(curVal);
                break;
            case TPS274C65_DEV_CONFIG2_REG:
                condorConfig1->setConfig2(curVal);
                break;
            case TPS274C65_DEV_CONFIG3_REG:
                condorConfig1->setConfig3(curVal);
                break;
            case TPS274C65_DEV_CONFIG4_REG:
                condorConfig1->setConfig4(curVal);
                break;
            case TPS274C65_DEV_CONFIG5_REG:
                condorConfig1->setConfig5(curVal);
                break;
            case TPS274C65_DEV_CONFIG6_REG:
                condorConfig1->setConfig6(curVal);
                break;
            case TPS274C65_FAULT_MASK_REG:
                condorConfig1->setFaultMask(curVal);
                break;
            case TPS274C65_EN_WB_OFF_REG:
                condorConfig1->setWireBreakOff(curVal);
                break;
            case TPS274C65_EN_WB_ON_REG:
                condorConfig1->setWireBreakOn(curVal);
                break;
            case TPS274C65_EN_SHRT_VS_REG:
                condorConfig1->setVsShort(curVal);
                break;
            case TPS274C65_ADC_ISNS_DIS_REG:
                condorConfig1->setISNSConf(curVal);
                break;
            case TPS274C65_ADC_TSNS_DIS_REG:
                condorConfig1->setTSNSConf(curVal);
                break;
            case TPS274C65_ADC_VSNS_DIS_REG:
                condorConfig1->setVSNSConf(curVal);
                break;
            case TPS274C65_ADC_CONFIG1_REG:
                condorConfig1->setADCConf(curVal);
                break;
            case TPS274C65_SW_STATE_REG:
                condorConfig1->setEnable(curVal);
                break;
            case TPS274C65_LED1_4_CTL_REG:
                condorConfig1->setLED1(curVal);
                break;
            case TPS274C65_LED_5_8_CTL_REG:
                condorConfig1->setLED2(curVal);
                break;
            default:
                break;
            }
            curString = sr->ReadLine();
        } while (curString != nullptr && curString != UNIT_2_ADDR.ToString());

        if (curString == nullptr)
        {
            return;
        }

        while (curString != nullptr)
        {
            if (curString->Contains("/*"))
            {
                curString = sr->ReadLine();
                continue;
            }

            splitString = curString->Split(',');
            if (splitString->Length != 2)
            {
                curString = sr->ReadLine();
                continue;
            }
            curReg = int::Parse(splitString[0]);
            curVal = int::Parse(splitString[1]);

            switch (curReg)
            {
            case TPS274C65_FS_SW_STATE_REG:
                condorConfig2->setWDFaultChannels(curVal);
                break;
            case TPS274C65_DEV_CONFIG1_REG:
                condorConfig2->setConfig1(curVal);
                break;
            case TPS274C65_DEV_CONFIG2_REG:
                condorConfig2->setConfig2(curVal);
                break;
            case TPS274C65_DEV_CONFIG3_REG:
                condorConfig2->setConfig3(curVal);
                break;
            case TPS274C65_DEV_CONFIG4_REG:
                condorConfig2->setConfig4(curVal);
                break;
            case TPS274C65_DEV_CONFIG5_REG:
                condorConfig2->setConfig5(curVal);
                break;
            case TPS274C65_DEV_CONFIG6_REG:
                condorConfig2->setConfig6(curVal);
                break;
            case TPS274C65_FAULT_MASK_REG:
                condorConfig2->setFaultMask(curVal);
                break;
            case TPS274C65_EN_WB_OFF_REG:
                condorConfig2->setWireBreakOff(curVal);
                break;
            case TPS274C65_EN_WB_ON_REG:
                condorConfig2->setWireBreakOn(curVal);
                break;
            case TPS274C65_EN_SHRT_VS_REG:
                condorConfig2->setVsShort(curVal);
                break;
            case TPS274C65_ADC_ISNS_DIS_REG:
                condorConfig2->setISNSConf(curVal);
                break;
            case TPS274C65_ADC_TSNS_DIS_REG:
                condorConfig2->setTSNSConf(curVal);
                break;
            case TPS274C65_ADC_VSNS_DIS_REG:
                condorConfig2->setVSNSConf(curVal);
                break;
            case TPS274C65_ADC_CONFIG1_REG:
                condorConfig2->setADCConf(curVal);
                break;
            case TPS274C65_SW_STATE_REG:
                condorConfig2->setEnable(curVal);
                break;
            case TPS274C65_LED1_4_CTL_REG:
                condorConfig2->setLED1(curVal);
                break;
            case TPS274C65_LED_5_8_CTL_REG:
                condorConfig2->setLED2(curVal);
                break;
            default:
                break;
            }
            curString = sr->ReadLine();
        }

        sr->Close();

        rawWriteFromConfig(condorConfig1);
        rawWriteFromConfig(condorConfig2);

        updateGuiFromConfig(true);

        logMessageToWindow("Loaded {0} file configuration into device", openFileDialog1->FileName);

        MessageBox::Show("Configuration loaded from " + openFileDialog1->FileName, "Configuration Loaded",
            MessageBoxButtons::OK, MessageBoxIcon::Information);
    }
}

System::Void tps274c65configurator::ConfigurationGui::rawWriteFromConfig(CondorConfiguration^ curConfig)
{
    uint8_t tempBuffer[2];
    HSSCommand* cmd;

    /* Sending the updated data */
    tempBuffer[0] = TPS274C65_DEV_CONFIG1_REG;
    tempBuffer[1] = curConfig->getConfig1();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    tempBuffer[0] = TPS274C65_DEV_CONFIG2_REG;
    tempBuffer[1] = curConfig->getConfig2();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    tempBuffer[0] = TPS274C65_DEV_CONFIG3_REG;
    tempBuffer[1] = curConfig->getConfig3();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    tempBuffer[0] = TPS274C65_DEV_CONFIG4_REG;
    tempBuffer[1] = curConfig->getConfig4();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    tempBuffer[0] = TPS274C65_DEV_CONFIG5_REG;
    tempBuffer[1] = curConfig->getConfig5();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    tempBuffer[0] = TPS274C65_DEV_CONFIG6_REG;
    tempBuffer[1] = curConfig->getConfig6();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    tempBuffer[0] = TPS274C65_LED1_4_CTL_REG;
    tempBuffer[1] = curConfig->getLED1();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    tempBuffer[0] = TPS274C65_LED_5_8_CTL_REG;
    tempBuffer[1] = curConfig->getLED2();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    tempBuffer[0] = TPS274C65_FS_SW_STATE_REG;
    tempBuffer[1] = curConfig->getWDFaultChannels();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    tempBuffer[0] = TPS274C65_FAULT_MASK_REG;
    tempBuffer[1] = curConfig->getFaultMask();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    tempBuffer[0] = TPS274C65_EN_WB_OFF_REG;
    tempBuffer[1] = curConfig->getWireBreakOff();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    tempBuffer[0] = TPS274C65_EN_WB_ON_REG;
    tempBuffer[1] = curConfig->getWireBreakOn();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    tempBuffer[0] = TPS274C65_EN_SHRT_VS_REG;
    tempBuffer[1] = curConfig->getVSShort();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    tempBuffer[0] = TPS274C65_ADC_ISNS_DIS_REG;
    tempBuffer[1] = curConfig->getISNSConf();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    tempBuffer[0] = TPS274C65_ADC_TSNS_DIS_REG;
    tempBuffer[1] = curConfig->getTSNSConf();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    tempBuffer[0] = TPS274C65_ADC_VSNS_DIS_REG;
    tempBuffer[1] = curConfig->getVSNSConf();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    tempBuffer[0] = TPS274C65_ADC_CONFIG1_REG;
    tempBuffer[1] = curConfig->getADCConf();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_RAW_WRITE, curConfig->getDeviceAddress(), tempBuffer, 2);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);

    /* Sending enable last as enabling the device will sometimes prevent config register writes */
    tempBuffer[0] = curConfig->getEnable();
    cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_CMD_ENABLE, curConfig->getDeviceAddress(), tempBuffer, 1);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());
    free(cmd);
}

System::Void tps274c65configurator::ConfigurationGui::mainTabControl_SelectedIndexChanged(System::Object^ sender, System::EventArgs^ e)
{
    tHSSReceiveTransaction^ trans;
    CondorConfiguration^ curConfig;

    /* Making sure we actually have to update the data */
    if (mainTabControl->SelectedIndex != 0 && mainTabControl->SelectedIndex != 2)
    {
        return;
    }

    /* Getting the current configuration */
    curConfig = getCurrentCondorConfig();

    /* Telling the HSS to update the configuration data for the selected address */
    HSSCommand* cmd = new HSSCommand(HSSCommand::HSS_TPS274C65_UPDATE_CONFIG, curConfig->getDeviceAddress(), NULL, 0);
    commManager->sendPacketData(cmd->getBufferData(), cmd->getCommandSize());

    /* Getting the updated data */
    trans = gcnew tHSSReceiveTransaction;
    trans->addr = curConfig->getDeviceAddress();
    trans->recData = (uint8_t*)malloc(128);
    trans->reportID = HSSCommunicationManager::HSS_CONFIG_ID;
    trans->receiveSize = HSSCommand::HSS_TPS274C65_UPDATE_CONFIG_RESP_SIZE;
    this->commManager->getPacketData(trans);
}

void tps274c65configurator::ConfigurationGui::logMessageToWindow(String^ str, ... array<Object^>^ args)
{
    CondorConfiguration^ curConfig = getCurrentCondorConfig();

    String^ tempString = String::Format(str, args);
    String^ dateString = DateTime::Now.ToString("[hh:mm:ss] ");

    this->logTextBox->AppendText(dateString + "[ADDR " + curConfig->getDeviceAddress().ToString() + "] " + tempString + Environment::NewLine);
}

/* Thread Functions */

/* Thread function that polls for faults */
void tps274c65configurator::ConfigurationGui::faultReadFunctionThread(Object^ obj)
{
    ConfigurationGui^ guiObj = (ConfigurationGui^)obj;
    HSSCommand* faultStatus1;
    HSSCommand* faultStatus2;
    CondorConfiguration^ curConfig;
    tHSSReceiveTransaction^ faultTrans1;
    tHSSReceiveTransaction^ faultTrans2;

    while (1)
    {
        Sleep(500);

        if (guiObj->commManager->isHidDeviceConnected() && (manualUpdateEnabled == false))
        {
            if (curSelectedDevice == UNIT_1_ADDR)
            {
                faultStatus1 = new HSSCommand(HSSCommand::HSS_TPS274C65_FAULT_STATE_CONFIG, UNIT_1_ADDR, NULL, 0);
                guiObj->commManager->sendPacketData(faultStatus1->getBufferData(), faultStatus1->getCommandSize());
                faultTrans1 = gcnew tHSSReceiveTransaction;
                faultTrans1->addr = UNIT_1_ADDR;
                faultTrans1->recData = (uint8_t*)malloc(128);
                faultTrans1->reportID = HSSCommunicationManager::HSS_FAULT_ID;
                faultTrans1->receiveSize = HSSCommand::HSS_TPS274C65_FAULT_STATE_CONFIG_RESP_SIZE;
                guiObj->commManager->getPacketData(faultTrans1);

                free(faultStatus1);
            }
            else
            {
                faultStatus2 = new HSSCommand(HSSCommand::HSS_TPS274C65_FAULT_STATE_CONFIG, UNIT_2_ADDR, NULL, 0);
                guiObj->commManager->sendPacketData(faultStatus2->getBufferData(), faultStatus2->getCommandSize());
                faultTrans2 = gcnew tHSSReceiveTransaction;
                faultTrans2->addr = UNIT_2_ADDR;
                faultTrans2->recData = (uint8_t*)malloc(128);
                faultTrans2->reportID = HSSCommunicationManager::HSS_FAULT_ID;
                faultTrans2->receiveSize = HSSCommand::HSS_TPS274C65_FAULT_STATE_CONFIG_RESP_SIZE;
                guiObj->commManager->getPacketData(faultTrans2);
                free(faultStatus2);
            }
        }

        /* See if the form is closing */
        if (guiObj->isShuttingDown)
        {
            return;
        }
    }
}

void tps274c65configurator::ConfigurationGui::handleRawReadResponse(sHSSReceiveTransaction^ trans)
{
    if (this->InvokeRequired)
    {
        UpdateSenseCallback^ updateCallback = gcnew UpdateSenseCallback(this,
            &tps274c65configurator::ConfigurationGui::handleRawReadResponse);
        this->Invoke(updateCallback, trans);
    }
    else
    {
        char szNum[32];
        sprintf(szNum, "%x", trans->recData[2]);
        String^ tempString = gcnew String(szNum);
        this->commandLogBox->AppendText("Register read result (hex): 0x" + tempString + Environment::NewLine);
    }
}

/* Thread Function that polls for sense data */
void tps274c65configurator::ConfigurationGui::senseReadFunctionThread(Object^ obj)
{
    ConfigurationGui^ guiObj = (ConfigurationGui^)obj;
    tHSSReceiveTransaction^ senseTrans;

    while (1)
    {
        Sleep(300);

        if (guiObj->commManager->isHidDeviceConnected())
        {
            if (guiObj->isCurrentSenseEnabled)
            {
                /* Getting the sense data */
                senseTrans = gcnew tHSSReceiveTransaction;
                senseTrans->addr = 0;
                senseTrans->recData = (uint8_t*)malloc(128);
                senseTrans->reportID = HSSCommunicationManager::HSS_SENSE_ID;
                senseTrans->receiveSize = HSSCommand::HSS_TPS274C65_SENSE_STATE_CONFIG_RESP_SIZE;
                guiObj->commManager->getPacketData(senseTrans);
            }
        }

        /* See if the form is closing */
        if (guiObj->isShuttingDown)
        {
            return;
        }
    }
}

void tps274c65configurator::ConfigurationGui::connectionThreadFunction(Object^ obj)
{
    ConfigurationGui^ guiObj = (ConfigurationGui^)obj;
    uint8_t status;

    /* Queuing a single connect event on startup */
    connQueue.Enqueue(0x01);

    while (1)
    {
        Sleep(200);
        if (connQueue.Count != 0)
        {
            status = connQueue.Dequeue();
            /* Disconnect Event */
            if (status == 0)
            {
                guiObj->commManager->closeDevice();
            }
            /* Connect Event */
            else
            {
                guiObj->commManager->openDevice();
            }

            /* Update the GUI for the updated connection status */
            guiObj->updateConnectionStatus();
        }

        /* See if the form is closing */
        if (guiObj->isShuttingDown)
        {
            return;
        }
    }
}

/* Main event handler from received commands from the device */
void tps274c65configurator::ConfigurationGui::bufferReceiveEventHandler(tHSSReceiveTransaction^ cmd)
{
    /* Making sure the buffer is valid */
    if (cmd->recData == NULL)
    {
        return;
    }

    /* Handles the receive event */
    switch (cmd->reportID)
    {
    case HSSCommunicationManager::HSS_CONFIG_ID:
        if (cmd->addr == UNIT_1_ADDR)
        {
            this->condorConfig1->updateFromBuffer(cmd->recData + 2);
        }
        else
        {
            this->condorConfig2->updateFromBuffer(cmd->recData + 2);
        }
        if (cmd->addr == curSelectedDevice)
        {
            this->updateGuiFromConfig(true);
        }
        break;
    case HSSCommunicationManager::HSS_SENSE_ID:
        updateCurrentSenseGUI(cmd);
        break;
    case HSSCommunicationManager::HSS_FAULT_ID:
        if (cmd->addr == UNIT_1_ADDR)
        {
            this->condorConfig1->setFaultStatus(cmd->recData  + 2);

            if (curSelectedDevice == UNIT_1_ADDR)
            {
                this->updateFaultGuiFromConfig(condorConfig1);
            }
        }
        else if(cmd->addr == UNIT_2_ADDR)
        {
            this->condorConfig2->setFaultStatus(cmd->recData + 2);

            if (curSelectedDevice == UNIT_2_ADDR)
            {
                this->updateFaultGuiFromConfig(condorConfig2);
            }
        } 
        break;
    case HSSCommunicationManager::HSS_RAWREAD_ID:
        this->handleRawReadResponse(cmd);
        break;
    default:
        return;
    }

    free(cmd->recData);
}
